@@ -17,3 +17,6 @@
 """
 Extentions to the Twisted Framework.
 """
+
+from twext import patches
+patches; del(patches)
