@@ -1,5 +1,5 @@
 from twisted.application.reactors import Reactor
 
 caldav_kqueue = Reactor(
-    'caldav_kqueue', 'kqreactor',
+    'caldav_kqueue', 'twext.internet.kqreactor',
     'kqueue(2)-based reactor.')
