@@ -1,5 +1,6 @@
+# -*- test-case-name: twistedcaldav -*-
 ##
-# Copyright (c) 2005-2007 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2011 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,25 +16,16 @@
 ##
 
 """
-WebDAV support for Twisted Web2.
+CalDAV support for Twext.Web2.
 
-See draft spec: http://ietf.webdav.org/caldav/draft-dusseault-caldav.txt
+See RFC 4791.
 """
 
-#
-# Set __version__
-#
-
-try:
-    from twistedcaldav.version import version as __version__
-except ImportError:
-    __version__ = None
-
 #
 # Load in suitable file extension/content-type map from OS X
 #
 
-from twisted.web2.static import File, loadMimeTypes
+from twext.web2.static import File, loadMimeTypes
 
 File.contentTypes = loadMimeTypes(("/etc/apache2/mime.types", "/etc/httpd/mime.types",))
 
@@ -41,18 +33,22 @@ File.contentTypes = loadMimeTypes(("/etc/apache2/mime.types", "/etc/httpd/mime.t
 # Register additional WebDAV XML elements
 #
 
-import twisted.web2.dav.davxml
+import twext.web2.dav.davxml
 import twistedcaldav.caldavxml
+import twistedcaldav.carddavxml
+import twistedcaldav.mkcolxml
 import twistedcaldav.customxml
 
-twisted.web2.dav.davxml.registerElements(twistedcaldav.caldavxml)
-twisted.web2.dav.davxml.registerElements(twistedcaldav.customxml)
+twext.web2.dav.davxml.registerElements(twistedcaldav.caldavxml)
+twext.web2.dav.davxml.registerElements(twistedcaldav.customxml)
+twext.web2.dav.davxml.registerElements(twistedcaldav.carddavxml)
+twext.web2.dav.davxml.registerElements(twistedcaldav.mkcolxml)
 
 #
 # DefaultHTTPHandler
 #
 
-from twisted.web2.http_headers import DefaultHTTPHandler, last, singleHeader
+from twext.web2.http_headers import DefaultHTTPHandler, last, singleHeader
 
 DefaultHTTPHandler.updateParsers({
     "If-Schedule-Tag-Match": (last, str),
@@ -61,3 +57,5 @@ DefaultHTTPHandler.updateGenerators({
     "Schedule-Tag": (str, singleHeader),
 })
 
+from pycalendar.calendar import PyCalendar
+PyCalendar.setPRODID("-//CALENDARSERVER.ORG//NONSGML Version 1//EN")
