@@ -42,11 +42,11 @@ import kerberos
 
 from twisted.cred import checkers, credentials, error
 from twisted.internet.defer import succeed
-from twisted.web2 import responsecode
-from twisted.web2.auth.interfaces import ICredentialFactory
-from twisted.web2.dav.auth import IPrincipalCredentials
+from twext.web2 import responsecode
+from twext.web2.auth.interfaces import ICredentialFactory
+from twext.web2.dav.auth import IPrincipalCredentials
 
-from twistedcaldav.log import LoggingMixIn
+from twext.python.log import LoggingMixIn
 
 class KerberosCredentialFactoryBase(LoggingMixIn):
     """
@@ -179,7 +179,12 @@ class BasicKerberosCredentialsChecker(LoggingMixIn):
                 self.log_error("%s" % (ex[0],))
                 raise error.UnauthorizedLogin("Bad credentials for: %s (%s: %s)" % (pcreds.authnURI, ex[0], ex[1],))
             else:
-                return succeed((pcreds.authnURI, pcreds.authzURI,))
+                return succeed((
+                    pcreds.authnPrincipal.principalURL(),
+                    pcreds.authzPrincipal.principalURL(),
+                    pcreds.authnPrincipal,
+                    pcreds.authzPrincipal,
+                ))
         
         raise error.UnauthorizedLogin("Bad credentials for: %s" % (pcreds.authnURI,))
 
@@ -307,7 +312,12 @@ class NegotiateCredentialsChecker(object):
 
         creds = pcreds.credentials
         if isinstance(creds, NegotiateCredentials):
-            return succeed((pcreds.authnURI, pcreds.authzURI,))
+            return succeed((
+                pcreds.authnPrincipal.principalURL(),
+                pcreds.authzPrincipal.principalURL(),
+                pcreds.authnPrincipal,
+                pcreds.authzPrincipal,
+            ))
         
         raise error.UnauthorizedLogin("Bad credentials for: %s" % (pcreds.authnURI,))
 
