@@ -38,19 +38,20 @@ class AggregateDirectoryService(DirectoryService):
     """
     baseGUID = "06FB225F-39E7-4D34-B1D1-29925F5E619B"
 
-    def __init__(self, services):
+    def __init__(self, services, groupMembershipCache):
         super(AggregateDirectoryService, self).__init__()
 
         realmName = None
         recordTypes = {}
+        self.groupMembershipCache = groupMembershipCache
 
         for service in services:
             service = IDirectoryService(service)
 
             if service.realmName != realmName:
                 assert realmName is None, (
-                    "Aggregated directory services must have the same realm name: %r != %r"
-                    % (service.realmName, realmName)
+                    "Aggregated directory services must have the same realm name: %r != %r\nServices: %r"
+                    % (service.realmName, realmName, services)
                 )
                 realmName = service.realmName
 
@@ -69,12 +70,20 @@ class AggregateDirectoryService(DirectoryService):
         self.realmName = realmName
         self._recordTypes = recordTypes
 
+        # FIXME: This is a temporary workaround until new data store is in
+        # place.  During the purging of deprovisioned users' data, we need
+        # to be able to look up records by guid and shortName.  The purge
+        # tool sticks temporary fake records in here.
+        self._tmpRecords = {
+            "guids" : { },
+            "shortNames" : { },
+        }
+
     def __repr__(self):
         return "<%s (%s): %r>" % (self.__class__.__name__, self.realmName, self._recordTypes)
 
     #
     # Define calendarHomesCollection as a property so we can set it on contained services
-    # See CalendarHomeProvisioningFile.__init__()
     #
     def _getCalendarHomesCollection(self):
         return self._calendarHomesCollection
@@ -86,6 +95,19 @@ class AggregateDirectoryService(DirectoryService):
 
     calendarHomesCollection = property(_getCalendarHomesCollection, _setCalendarHomesCollection)
 
+    #
+    # Define addressBookHomesCollection as a property so we can set it on contained services
+    #
+    def _getAddressBookHomesCollection(self):
+        return self._addressBookHomesCollection
+
+    def _setAddressBookHomesCollection(self, value):
+        for service in self._recordTypes.values():
+            service.addressBookHomesCollection = value
+        self._addressBookHomesCollection = value
+
+    addressBookHomesCollection = property(_getAddressBookHomesCollection, _setAddressBookHomesCollection)
+
     def recordTypes(self):
         return set(self._recordTypes)
 
@@ -97,11 +119,29 @@ class AggregateDirectoryService(DirectoryService):
             return records
 
     def recordWithShortName(self, recordType, shortName):
+
+        # FIXME: These temporary records shouldn't be needed when we move
+        # to the new data store API.  They're currently needed when purging
+        # deprovisioned users' data.
+        record = self._tmpRecords["shortNames"].get(shortName, None)
+        if record:
+            return record
+
         return self._query("recordWithShortName", recordType, shortName)
 
     def recordWithUID(self, uid):
+
+        # FIXME: These temporary records shouldn't be needed when we move
+        # to the new data store API.  They're currently needed when purging
+        # deprovisioned users' data.
+        record = self._tmpRecords["guids"].get(uid, None)
+        if record:
+            return record
+
         return self._queryAll("recordWithUID", uid)
 
+    recordWithGUID = recordWithUID
+
     def recordWithAuthID(self, authID):
         return self._queryAll("recordWithAuthID", authID)
 
@@ -124,6 +164,15 @@ class AggregateDirectoryService(DirectoryService):
 
         returnValue(itertools.chain(*generators))
 
+    def getGroups(self, guids):
+        """
+        Returns a set of group records for the list of guids passed in.  For
+        any group that also contains subgroups, those subgroups' records are
+        also returned, and so on.
+        """
+        recordType = self.recordType_groups
+        service = self.serviceForRecordType(recordType)
+        return service.getGroups(guids)
 
 
     def serviceForRecordType(self, recordType):
@@ -145,12 +194,20 @@ class AggregateDirectoryService(DirectoryService):
 
     def _queryAll(self, query, *args):
         for service in self._recordTypes.values():
-            record = getattr(service, query)(*args)
+            try:
+                record = getattr(service, query)(*args)
+            except UnknownRecordTypeError:
+                record = None
             if record is not None:
                 return record
         else:
             return None
 
+    def flushCaches(self):
+        for service in self._recordTypes.values():
+            if hasattr(service, "_initCaches"):
+                service._initCaches()
+
     userRecordTypes = [DirectoryService.recordType_users]
 
     def requestAvatarId(self, credentials):
@@ -168,6 +225,43 @@ class AggregateDirectoryService(DirectoryService):
                     results.append(result)
         return results
 
+
+    def getExternalProxyAssignments(self):
+        service = self.serviceForRecordType(self.recordType_locations)
+        return service.getExternalProxyAssignments()
+
+
+    def createRecord(self, recordType, guid=None, shortNames=(), authIDs=set(),
+        fullName=None, firstName=None, lastName=None, emailAddresses=set(),
+        uid=None, password=None, **kwargs):
+        service = self.serviceForRecordType(recordType)
+        return service.createRecord(recordType, guid=guid,
+            shortNames=shortNames, authIDs=authIDs, fullName=fullName,
+            firstName=firstName, lastName=lastName,
+            emailAddresses=emailAddresses, uid=uid, password=password, **kwargs)
+
+    def updateRecord(self, recordType, guid=None, shortNames=(), authIDs=set(),
+        fullName=None, firstName=None, lastName=None, emailAddresses=set(),
+        uid=None, password=None, **kwargs):
+        service = self.serviceForRecordType(recordType)
+        return service.updateRecord(recordType, guid=guid,
+            shortNames=shortNames,
+            authIDs=authIDs, fullName=fullName, firstName=firstName,
+            lastName=lastName, emailAddresses=emailAddresses, uid=uid,
+            password=password, **kwargs)
+
+    def destroyRecord(self, recordType, guid=None):
+        service = self.serviceForRecordType(recordType)
+        return service.destroyRecord(recordType, guid=guid)
+
+    def setRealm(self, realmName):
+        """
+        Set a new realm name for this and nested services
+        """
+        self.realmName = realmName
+        for service in self._recordTypes.values():
+            service.setRealm(realmName)
+
 class DuplicateRecordTypeError(DirectoryError):
     """
     Duplicate record type.
