@@ -1,5 +1,6 @@
+# -*- test-case-name: twistedcaldav.directory.test.test_proxyprincipalmembers -*-
 ##
-# Copyright (c) 2006-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2010 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -20,30 +21,43 @@ Implements a calendar user proxy principal.
 
 __all__ = [
     "CalendarUserProxyPrincipalResource",
-    "CalendarUserProxyDatabase",
+    "ProxyDB",
+    "ProxyDBService",
+    "ProxySqliteDB",
+    "ProxyPostgreSQLDB",
 ]
 
-from twisted.internet.defer import returnValue
-from twisted.internet.defer import succeed, inlineCallbacks
-from twisted.web2 import responsecode
-from twisted.web2.http import HTTPError, StatusResponse
-from twisted.web2.dav import davxml
-from twisted.web2.dav.element.base import dav_namespace
-from twisted.web2.dav.util import joinURL
-from twisted.web2.dav.noneprops import NonePropertyStore
+import itertools
+import time
 
-from twistedcaldav.config import config
-from twistedcaldav.extensions import DAVFile, DAVPrincipalResource
+from twisted.internet.defer import succeed, inlineCallbacks, returnValue
+from twext.web2 import responsecode
+from twext.web2.http import HTTPError, StatusResponse
+from twext.web2.dav import davxml
+from twext.web2.dav.element.base import dav_namespace
+from twext.web2.dav.util import joinURL
+from twext.web2.dav.noneprops import NonePropertyStore
+
+from twext.python.log import Logger, LoggingMixIn
+
+from twisted.web.template import XMLFile, Element, renderer
+from twisted.python.modules import getModule
+from twistedcaldav.extensions import DirectoryElement
+from twistedcaldav.directory.principal import formatLink
+from twistedcaldav.directory.principal import formatLinks
+from twistedcaldav.directory.principal import formatPrincipals
+
+from twistedcaldav.config import config, fullServerPath
+from twistedcaldav.database import AbstractADBAPIDatabase, ADBAPISqliteMixin,\
+    ADBAPIPostgreSQLMixin
+from twistedcaldav.extensions import DAVPrincipalResource,\
+    DAVResourceWithChildrenMixin
 from twistedcaldav.extensions import ReadOnlyWritePropertiesResourceMixIn
 from twistedcaldav.memcacher import Memcacher
 from twistedcaldav.resource import CalDAVComplianceMixIn
-from twistedcaldav.directory.util import NotFilePath
-from twistedcaldav.sql import AbstractSQLDatabase, db_prefix
-from twistedcaldav.log import LoggingMixIn
 
-import itertools
-import os
-import time
+thisModule = getModule(__name__)
+log = Logger()
 
 class PermissionsMixIn (ReadOnlyWritePropertiesResourceMixIn):
     def defaultAccessControlList(self):
@@ -73,14 +87,79 @@ class PermissionsMixIn (ReadOnlyWritePropertiesResourceMixIn):
 
         return davxml.ACL(*aces)
 
-    def accessControlList(self, request, inheritance=True, expanding=False, inherited_aces=None):
+
+    def accessControlList(self, request, inheritance=True, expanding=False,
+                          inherited_aces=None):
         # Permissions here are fixed, and are not subject to inheritance rules, etc.
         return succeed(self.defaultAccessControlList())
 
-class CalendarUserProxyPrincipalResource (CalDAVComplianceMixIn, PermissionsMixIn, DAVPrincipalResource, DAVFile):
+
+
+class ProxyPrincipalDetailElement(Element):
+    """
+    A L{ProxyPrincipalDetailElement} is an L{Element} that can render the
+    details of a L{CalendarUserProxyPrincipalResource}.
+    """
+
+    loader = XMLFile(thisModule.filePath.sibling(
+        "calendar-user-proxy-principal-resource.html").open()
+    )
+
+    def __init__(self, resource):
+        super(ProxyPrincipalDetailElement, self).__init__()
+        self.resource = resource
+
+
+    @renderer
+    def principal(self, request, tag):
+        """
+        Top-level renderer in the template.
+        """
+        record = self.resource.parent.record
+        resource = self.resource
+        parent = self.resource.parent
+        return tag.fillSlots(
+            directoryGUID=record.service.guid,
+            realm=record.service.realmName,
+            guid=record.guid,
+            recordType=record.recordType,
+            shortNames=record.shortNames,
+            fullName=record.fullName,
+            principalUID=parent.principalUID(),
+            principalURL=formatLink(parent.principalURL()),
+            proxyPrincipalUID=resource.principalUID(),
+            proxyPrincipalURL=formatLink(resource.principalURL()),
+            alternateURIs=formatLinks(resource.alternateURIs()),
+            groupMembers=resource.groupMembers().addCallback(formatPrincipals),
+            groupMemberships=resource.groupMemberships().addCallback(
+                formatPrincipals
+            ),
+        )
+
+
+
+class ProxyPrincipalElement(DirectoryElement):
+    """
+    L{ProxyPrincipalElement} is a renderer for a
+    L{CalendarUserProxyPrincipalResource}.
+    """
+
+    @renderer
+    def resourceDetail(self, request, tag):
+        """
+        Render the proxy principal's details.
+        """
+        return ProxyPrincipalDetailElement(self.resource)
+
+
+
+class CalendarUserProxyPrincipalResource (
+        CalDAVComplianceMixIn, PermissionsMixIn, DAVResourceWithChildrenMixin,
+        DAVPrincipalResource, LoggingMixIn):
     """
     Calendar user proxy principal resource.
     """
+
     def __init__(self, parent, proxyType):
         """
         @param parent: the parent of this resource.
@@ -93,47 +172,43 @@ class CalendarUserProxyPrincipalResource (CalDAVComplianceMixIn, PermissionsMixI
 
         url = joinURL(parent.principalURL(), proxyType) + slash
 
-        super(CalendarUserProxyPrincipalResource, self).__init__(NotFilePath(isdir=True), url)
+        super(CalendarUserProxyPrincipalResource, self).__init__()
+        DAVResourceWithChildrenMixin.__init__(self)
 
-        self.parent      = parent
-        self.proxyType   = proxyType
-        self.pcollection = self.parent.parent.parent # FIXME: if this is supposed to be public, it needs a better name
-        self._url        = url
+        self.parent          = parent
+        self.proxyType       = proxyType
+        self._url            = url
 
-        # Not terribly useful at present because we don't have a way
-        # to map a GUID back to the correct principal.
-        #self.guid = uuidFromName(self.parent.principalUID(), proxyType)
-
-        # Principal UID is parent's GUID plus the proxy type; this we
-        # can easily map back to a principal.
-        self.uid = "%s#%s" % (self.parent.principalUID(), proxyType)
+        # FIXME: if this is supposed to be public, it needs a better name:
+        self.pcollection     = self.parent.parent.parent
 
+        # Principal UID is parent's GUID plus the proxy type; this we can easily
+        # map back to a principal.
+        self.uid             = "%s#%s" % (self.parent.principalUID(), proxyType)
         self._alternate_urls = tuple(
             joinURL(url, proxyType) + slash
             for url in parent.alternateURIs()
             if url.startswith("/")
         )
 
+
     def __str__(self):
         return "%s [%s]" % (self.parent, self.proxyType)
 
+
     def _index(self):
         """
         Return the SQL database for this group principal.
 
-        @return: the L{CalendarUserProxyDatabase} for the principal collection.
+        @return: the L{ProxyDB} for the principal collection.
         """
-
-        # The db is located in the principal collection root
-        if not hasattr(self.pcollection, "calendar_user_proxy_db"):
-            setattr(self.pcollection, "calendar_user_proxy_db", CalendarUserProxyDatabase(config.DataRoot))
-        return self.pcollection.calendar_user_proxy_db
+        return ProxyDBService
 
     def resourceType(self):
         if self.proxyType == "calendar-proxy-read":
-            return davxml.ResourceType.calendarproxyread
+            return davxml.ResourceType.calendarproxyread #@UndefinedVariable
         elif self.proxyType == "calendar-proxy-write":
-            return davxml.ResourceType.calendarproxywrite
+            return davxml.ResourceType.calendarproxywrite #@UndefinedVariable
         else:
             return super(CalendarUserProxyPrincipalResource, self).resourceType()
 
@@ -157,24 +232,29 @@ class CalendarUserProxyPrincipalResource (CalDAVComplianceMixIn, PermissionsMixI
             self._dead_properties = NonePropertyStore(self)
         return self._dead_properties
 
+
     def writeProperty(self, property, request):
         assert isinstance(property, davxml.WebDAVElement)
 
         if property.qname() == (dav_namespace, "group-member-set"):
             return self.setGroupMemberSet(property, request)
 
-        return super(CalendarUserProxyPrincipalResource, self).writeProperty(property, request)
+        return super(CalendarUserProxyPrincipalResource, self).writeProperty(
+            property, request)
+
 
     @inlineCallbacks
     def setGroupMemberSet(self, new_members, request):
-        # FIXME: as defined right now it is not possible to specify a calendar-user-proxy group as
-        # a member of any other group since the directory service does not know how to lookup
-        # these special resource UIDs.
+        # FIXME: as defined right now it is not possible to specify a
+        # calendar-user-proxy group as a member of any other group since the
+        # directory service does not know how to lookup these special resource
+        # UIDs.
         #
-        # Really, c-u-p principals should be treated the same way as any other principal, so
-        # they should be allowed as members of groups.
+        # Really, c-u-p principals should be treated the same way as any other
+        # principal, so they should be allowed as members of groups.
         #
-        # This implementation now raises an exception for any principal it cannot find.
+        # This implementation now raises an exception for any principal it
+        # cannot find.
 
         # Break out the list into a set of URIs.
         members = [str(h) for h in new_members.children]
@@ -188,87 +268,48 @@ class CalendarUserProxyPrincipalResource (CalDAVComplianceMixIn, PermissionsMixI
             if principal is None or principal.principalURL() != uri:
                 raise HTTPError(StatusResponse(
                     responsecode.BAD_REQUEST,
-                    "Attempt to use a non-existent principal %s as a group member of %s." % (uri, self.principalURL(),)
+                    "Attempt to use a non-existent principal %s "
+                    "as a group member of %s." % (uri, self.principalURL(),)
                 ))
             principals.append(principal)
             newUIDs.add(principal.principalUID())
 
         # Get the old set of UIDs
         oldUIDs = (yield self._index().getMembers(self.uid))
-        
+
         # Change membership
         yield self.setGroupMemberSetPrincipals(principals)
-        
+
         # Invalidate the primary principal's cache, and any principal's whose
         # membership status changed
         yield self.parent.cacheNotifier.changed()
-        
+
         changedUIDs = newUIDs.symmetric_difference(oldUIDs)
         for uid in changedUIDs:
             principal = self.pcollection.principalForUID(uid)
             if principal:
                 yield principal.cacheNotifier.changed()
-            
+
         returnValue(True)
 
-    @inlineCallbacks
+
     def setGroupMemberSetPrincipals(self, principals):
         # Map the principals to UIDs.
-        uids = [p.principalUID() for p in principals]
-        yield self._index().setGroupMembers(self.uid, uids)
+        return self._index().setGroupMembers(
+            self.uid,
+            [p.principalUID() for p in principals],
+        )
 
     ##
     # HTTP
     ##
 
-    def renderDirectoryBody(self, request):
-        # FIXME: Too much code duplication here from principal.py
-        from twistedcaldav.directory.principal import format_list, format_principals, format_link
+    def htmlElement(self):
+        """
+        Customize HTML display of proxy groups.
+        """
+        return ProxyPrincipalElement(self)
 
-        closure = {}
-
-        d = super(CalendarUserProxyPrincipalResource, self).renderDirectoryBody(request)
-        d.addCallback(lambda output: closure.setdefault("output", output))
-
-        d.addCallback(lambda _: self.groupMembers())
-        d.addCallback(lambda members: closure.setdefault("members", members))
-
-        d.addCallback(lambda _: self.groupMemberships())
-        d.addCallback(lambda memberships: closure.setdefault("memberships", memberships))
-        
-        d.addCallback(
-            lambda _: "".join((
-                """<div class="directory-listing">"""
-                """<h1>Principal Details</h1>"""
-                """<pre><blockquote>"""
-                """Directory Information\n"""
-                """---------------------\n"""
-                """Directory GUID: %s\n"""         % (self.parent.record.service.guid,),
-                """Realm: %s\n"""                  % (self.parent.record.service.realmName,),
-                """\n"""
-                """Parent Principal Information\n"""
-                """---------------------\n"""
-                """GUID: %s\n"""                   % (self.parent.record.guid,),
-                """Record type: %s\n"""            % (self.parent.record.recordType,),
-                """Short names: %s\n"""            % (",".join(self.parent.record.shortNames,)),
-                """Full name: %s\n"""              % (self.parent.record.fullName,),
-                """Principal UID: %s\n"""          % (self.parent.principalUID(),),
-                """Principal URL: %s\n"""          % (format_link(self.parent.principalURL()),),
-                """\n"""
-                """Proxy Principal Information\n"""
-                """---------------------\n"""
-               #"""GUID: %s\n"""                   % (self.guid,),
-                """Principal UID: %s\n"""          % (self.principalUID(),),
-                """Principal URL: %s\n"""          % (format_link(self.principalURL()),),
-                """\nAlternate URIs:\n"""          , format_list(format_link(u) for u in self.alternateURIs()),
-                """\nGroup members:\n"""           , format_principals(closure["members"]),
-                """\nGroup memberships:\n"""       , format_principals(closure["memberships"]),
-                """</pre></blockquote></div>""",
-                closure["output"]
-            ))
-        )
-
-        return d
 
     ##
     # DAV
@@ -277,6 +318,7 @@ class CalendarUserProxyPrincipalResource (CalDAVComplianceMixIn, PermissionsMixI
     def displayName(self):
         return self.proxyType
 
+
     ##
     # ACL
     ##
@@ -328,39 +370,40 @@ class CalendarUserProxyPrincipalResource (CalDAVComplianceMixIn, PermissionsMixI
         # Get member UIDs from database and map to principal resources
         members = yield self._index().getMembers(self.uid)
         found = []
-        missing = []
         for uid in members:
             p = self.pcollection.principalForUID(uid)
             if p:
                 found.append(p)
                 # Make sure any outstanding deletion timer entries for
                 # existing principals are removed
-                yield self._index()._memcacher.clearDeletionTimer(uid)
+                yield self._index().refreshPrincipal(uid)
             else:
-                missing.append(uid)
-
-        # Clean-up ones that are missing
-        for uid in missing:
-            cacheTimeout = config.DirectoryService.params.get("cacheTimeout", 30) * 60 # in seconds
-
-            yield self._index().removePrincipal(uid,
-                delay=cacheTimeout*2)
+                self.log_warn("Delegate is missing from directory: %s" % (uid,))
 
         returnValue(found)
 
     def groupMembers(self):
         return self._expandMemberUIDs()
 
-    def expandedGroupMembers(self):
-        return self._expandMemberUIDs(infinity=True)
-
     @inlineCallbacks
+    def expandedGroupMembers(self):
+        """
+        Return the complete, flattened set of principals belonging to this
+        group.
+        """
+        returnValue((yield self._expandMemberUIDs(infinity=True)))
+
     def groupMemberships(self):
         # Get membership UIDs and map to principal resources
-        memberships = yield self._index().getMemberships(self.uid)
-        returnValue([p for p in [self.pcollection.principalForUID(uid) for uid in memberships] if p])
+        d = self._index().getMemberships(self.uid)
+        d.addCallback(lambda memberships: [
+            p for p
+            in [self.pcollection.principalForUID(uid) for uid in memberships]
+            if p
+        ])
+        return d
 
-class CalendarUserProxyDatabase(AbstractSQLDatabase, LoggingMixIn):
+class ProxyDB(AbstractADBAPIDatabase, LoggingMixIn):
     """
     A database to maintain calendar user proxy group memberships.
 
@@ -372,13 +415,14 @@ class CalendarUserProxyDatabase(AbstractSQLDatabase, LoggingMixIn):
 
     """
 
-    dbType = "CALENDARUSERPROXY"
-    dbFilename = "calendaruserproxy.sqlite"
-    dbOldFilename = db_prefix + "calendaruserproxy"
-    dbFormatVersion = "4"
-
+    schema_version = "4"
+    schema_type    = "CALENDARUSERPROXY"
+    
     class ProxyDBMemcacher(Memcacher):
         
+        def __init__(self, namespace):
+            super(ProxyDB.ProxyDBMemcacher, self).__init__(namespace, key_normalization=config.Memcached.ProxyDBKeyNormalization)
+
         def setMembers(self, guid, members):
             return self.set("members:%s" % (str(guid),), str(",".join(members)))
 
@@ -442,11 +486,10 @@ class CalendarUserProxyDatabase(AbstractSQLDatabase, LoggingMixIn):
                 theTime = int(time.time())
             return theTime
 
-    def __init__(self, path):
-        path = os.path.join(path, CalendarUserProxyDatabase.dbFilename)
-        super(CalendarUserProxyDatabase, self).__init__(path, True)
+    def __init__(self, dbID, dbapiName, dbapiArgs, **kwargs):
+        AbstractADBAPIDatabase.__init__(self, dbID, dbapiName, dbapiArgs, True, **kwargs)
         
-        self._memcacher = CalendarUserProxyDatabase.ProxyDBMemcacher("proxyDB")
+        self._memcacher = ProxyDB.ProxyDBMemcacher("ProxyDB")
 
     @inlineCallbacks
     def setGroupMembers(self, principalUID, members):
@@ -463,17 +506,19 @@ class CalendarUserProxyDatabase(AbstractSQLDatabase, LoggingMixIn):
             current_members = ()
         current_members = set(current_members)
 
-        self.setGroupMembersInDatabase(principalUID, members)
-
-        # Update cache
+        # Find changes
         update_members = set(members)
-        
         remove_members = current_members.difference(update_members)
         add_members = update_members.difference(current_members)
-        for member in itertools.chain(remove_members, add_members,):
-            _ignore = yield self._memcacher.deleteMembership(member)
-        _ignore = yield self._memcacher.deleteMember(principalUID)
 
+        yield self.changeGroupMembersInDatabase(principalUID, add_members, remove_members)
+
+        # Update cache
+        for member in itertools.chain(remove_members, add_members,):
+            yield self._memcacher.deleteMembership(member)
+        yield self._memcacher.deleteMember(principalUID)
+
+    @inlineCallbacks
     def setGroupMembersInDatabase(self, principalUID, members):
         """
         A blocking call to add a group membership record in the database.
@@ -482,9 +527,23 @@ class CalendarUserProxyDatabase(AbstractSQLDatabase, LoggingMixIn):
         @param members: a list UIDs of principals that are members of this group.
         """
         # Remove what is there, then add it back.
-        self._delete_from_db(principalUID)
-        self._add_to_db(principalUID, members)
-        self._db_commit()
+        yield self._delete_from_db(principalUID)
+        yield self._add_to_db(principalUID, members)
+        
+    @inlineCallbacks
+    def changeGroupMembersInDatabase(self, principalUID, addMembers, removeMembers):
+        """
+        A blocking call to add a group membership record in the database.
+
+        @param principalUID: the UID of the group principal to add.
+        @param addMembers: a list UIDs of principals to be added as members of this group.
+        @param removeMembers: a list UIDs of principals to be removed as members of this group.
+        """
+        # Remove what is there, then add it back.
+        for member in removeMembers:
+            yield self._delete_from_db_one(principalUID, member)
+        for member in addMembers:
+            yield self._add_to_db_one(principalUID, member)
         
     @inlineCallbacks
     def removeGroup(self, principalUID):
@@ -497,8 +556,7 @@ class CalendarUserProxyDatabase(AbstractSQLDatabase, LoggingMixIn):
         # Need to get the members before we do the delete
         members = yield self.getMembers(principalUID)
 
-        self._delete_from_db(principalUID)
-        self._db_commit()
+        yield self._delete_from_db(principalUID)
         
         # Update cache
         if members:
@@ -530,17 +588,17 @@ class CalendarUserProxyDatabase(AbstractSQLDatabase, LoggingMixIn):
 
             elif overdue is None:
                 # No timer was previously set
-                self.log_debug("Delaying removal of missing proxy principal '%s'" %
-                    (principalUID,))
-                self._memcacher.setDeletionTimer(principalUID, delay=delay)
+                self.log_debug("Delaying removal of missing proxy principal '%s'"
+                               % (principalUID,))
+                yield self._memcacher.setDeletionTimer(principalUID, delay=delay)
                 returnValue(None)
 
-        self.log_warn("Removing missing proxy principal for '%s'" %
-            (principalUID,))
+        self.log_warn("Removing missing proxy principal for '%s'"
+                      % (principalUID,))
 
         for suffix in ("calendar-proxy-read", "calendar-proxy-write",):
             groupUID = "%s#%s" % (principalUID, suffix,)
-            self._delete_from_db(groupUID)
+            yield self._delete_from_db(groupUID)
 
             # Update cache
             members = yield self.getMembers(groupUID)
@@ -553,47 +611,71 @@ class CalendarUserProxyDatabase(AbstractSQLDatabase, LoggingMixIn):
         for groupUID in memberships:
             yield self._memcacher.deleteMember(groupUID)
 
-        self._delete_from_db_member(principalUID)
+        yield self._delete_from_db_member(principalUID)
         yield self._memcacher.deleteMembership(principalUID)
-        self._db_commit()
-        self._memcacher.clearDeletionTimer(principalUID)
+        yield self._memcacher.clearDeletionTimer(principalUID)
+
+    def refreshPrincipal(self, principalUID):
+        """
+        Bring back to life a principal that was previously deleted.
+
+        @param principalUID:
+        @type principalUID:
+        """
+        
+        return self._memcacher.clearDeletionTimer(principalUID)
 
-    @inlineCallbacks
     def getMembers(self, principalUID):
         """
         Return the list of group member UIDs for the specified principal.
 
         @return: a deferred returning a C{set} of members.
         """
+        def gotCachedMembers(members):
+            if members is not None:
+                return members
 
-        def _members():
-            return set([row[0] for row in self._db_execute("select MEMBER from GROUPS where GROUPNAME = :1", principalUID)])
+            # Cache miss; compute members and update cache
+            def gotMembersFromDB(dbmembers):
+                members = set([row[0].encode("utf-8") for row in dbmembers])
+                d = self._memcacher.setMembers(principalUID, members)
+                d.addCallback(lambda _: members)
+                return d
 
-        # Pull from cache
-        result = yield self._memcacher.getMembers(principalUID)
-        if result is None:
-            result = _members()
-            yield self._memcacher.setMembers(principalUID, result)
-        returnValue(result)
+            d =  self.query("select MEMBER from GROUPS where GROUPNAME = :1", (principalUID.decode("utf-8"),))
+            d.addCallback(gotMembersFromDB)
+            return d
+
+        d = self._memcacher.getMembers(principalUID)
+        d.addCallback(gotCachedMembers)
+        return d
 
-    @inlineCallbacks
     def getMemberships(self, principalUID):
         """
         Return the list of group principal UIDs the specified principal is a member of.
         
         @return: a deferred returning a C{set} of memberships.
         """
+        def gotCachedMemberships(memberships):
+            if memberships is not None:
+                return memberships
 
-        def _members():
-            return set([row[0] for row in self._db_execute("select GROUPNAME from GROUPS where MEMBER = :1", principalUID)])
+            # Cache miss; compute memberships and update cache
+            def gotMembershipsFromDB(dbmemberships):
+                memberships = set([row[0].encode("utf-8") for row in dbmemberships])
+                d = self._memcacher.setMemberships(principalUID, memberships)
+                d.addCallback(lambda _: memberships)
+                return d
 
-        # Pull from cache
-        result = yield self._memcacher.getMemberships(principalUID)
-        if result is None:
-            result = _members()
-            yield self._memcacher.setMemberships(principalUID, result)
-        returnValue(result)
+            d =  self.query("select GROUPNAME from GROUPS where MEMBER = :1", (principalUID.decode("utf-8"),))
+            d.addCallback(gotMembershipsFromDB)
+            return d
 
+        d = self._memcacher.getMemberships(principalUID)
+        d.addCallback(gotCachedMemberships)
+        return d
+
+    @inlineCallbacks
     def _add_to_db(self, principalUID, members):
         """
         Insert the specified entry into the database.
@@ -602,20 +684,43 @@ class CalendarUserProxyDatabase(AbstractSQLDatabase, LoggingMixIn):
         @param members: a list of UIDs or principals that are members of this group.
         """
         for member in members:
-            self._db_execute(
+            yield self.execute(
                 """
                 insert into GROUPS (GROUPNAME, MEMBER)
                 values (:1, :2)
-                """, principalUID, member
+                """, (principalUID.decode("utf-8"), member,)
             )
 
+    def _add_to_db_one(self, principalUID, memberUID):
+        """
+        Insert the specified entry into the database.
+
+        @param principalUID: the UID of the group principal to add.
+        @param memberUID: the UID of the principal that is being added as a member of this group.
+        """
+        return self.execute(
+            """
+            insert into GROUPS (GROUPNAME, MEMBER)
+            values (:1, :2)
+            """, (principalUID.decode("utf-8"), memberUID.decode("utf-8"),)
+        )
+
     def _delete_from_db(self, principalUID):
         """
         Deletes the specified entry from the database.
 
         @param principalUID: the UID of the group principal to remove.
         """
-        self._db_execute("delete from GROUPS where GROUPNAME = :1", principalUID)
+        return self.execute("delete from GROUPS where GROUPNAME = :1", (principalUID.decode("utf-8"),))
+
+    def _delete_from_db_one(self, principalUID, memberUID):
+        """
+        Deletes the specified entry from the database.
+
+        @param principalUID: the UID of the group principal to remove.
+        @param memberUID: the UID of the principal that is being removed as a member of this group.
+        """
+        return self.execute("delete from GROUPS where GROUPNAME = :1 and MEMBER = :2", (principalUID.decode("utf-8"), memberUID.decode("utf-8"),))
 
     def _delete_from_db_member(self, principalUID):
         """
@@ -623,21 +728,22 @@ class CalendarUserProxyDatabase(AbstractSQLDatabase, LoggingMixIn):
 
         @param principalUID: the UID of the member principal to remove.
         """
-        self._db_execute("delete from GROUPS where MEMBER = :1", principalUID)
+        return self.execute("delete from GROUPS where MEMBER = :1", (principalUID.decode("utf-8"),))
 
     def _db_version(self):
         """
         @return: the schema version assigned to this index.
         """
-        return CalendarUserProxyDatabase.dbFormatVersion
+        return ProxyDB.schema_version
 
     def _db_type(self):
         """
         @return: the collection type assigned to this index.
         """
-        return CalendarUserProxyDatabase.dbType
+        return ProxyDB.schema_type
 
-    def _db_init_data_tables(self, q):
+    @inlineCallbacks
+    def _db_init_data_tables(self):
         """
         Initialise the underlying database tables.
         @param q:           a database cursor to use.
@@ -646,47 +752,104 @@ class CalendarUserProxyDatabase(AbstractSQLDatabase, LoggingMixIn):
         #
         # GROUPS table
         #
-        q.execute(
-            """
-            create table GROUPS (
-                GROUPNAME   text,
-                MEMBER      text
-            )
-            """
-        )
-        q.execute(
-            """
-            create index GROUPNAMES on GROUPS (GROUPNAME)
-            """
-        )
-        q.execute(
-            """
-            create index MEMBERS on GROUPS (MEMBER)
-            """
+        yield self._create_table(
+            "GROUPS",
+            (
+                ("GROUPNAME", "text"),
+                ("MEMBER",    "text"),
+            ),
+            ifnotexists=True,
         )
 
-    def _db_upgrade_data_tables(self, q, old_version):
+        yield self._create_index(
+            "GROUPNAMES",
+            "GROUPS",
+            ("GROUPNAME",),
+            ifnotexists=True,
+        )
+        yield self._create_index(
+            "MEMBERS",
+            "GROUPS",
+            ("MEMBER",),
+            ifnotexists=True,
+        )
+
+    @inlineCallbacks
+    def _db_upgrade_data_tables(self, old_version):
         """
         Upgrade the data from an older version of the DB.
-        @param q: a database cursor to use.
         @param old_version: existing DB's version number
         @type old_version: str
         """
 
         # Add index if old version is less than "4"
         if int(old_version) < 4:
-            q.execute(
-                """
-                create index GROUPNAMES on GROUPS (GROUPNAME)
-                """
+            yield self._create_index(
+                "GROUPNAMES",
+                "GROUPS",
+                ("GROUPNAME",),
+                ifnotexists=True,
             )
-            q.execute(
-                """
-                create index MEMBERS on GROUPS (MEMBER)
-                """
+            yield self._create_index(
+                "MEMBERS",
+                "GROUPS",
+                ("MEMBER",),
+                ifnotexists=True,
             )
 
+    def _db_empty_data_tables(self):
+        """
+        Empty the underlying database tables.
+        @param q:           a database cursor to use.
+        """
 
+        #
+        # GROUPS table
+        #
+        return self._db_execute("delete from GROUPS")
+
+    @inlineCallbacks
+    def clean(self):
+        
+        if not self.initialized:
+            yield self.open()
+
+        for group in [row[0] for row in (yield self.query("select GROUPNAME from GROUPS"))]:
+            self.removeGroup(group)
+        
+        yield super(ProxyDB, self).clean()
+
+    @inlineCallbacks
+    def getAllMembers(self):
+        """
+        Retrieve all members that have been directly delegated to
+        """
+        returnValue([row[0] for row in (yield self.query("select DISTINCT MEMBER from GROUPS"))])
+
+ProxyDBService = None   # Global proxyDB service
+
+
+class ProxySqliteDB(ADBAPISqliteMixin, ProxyDB):
+    """
+    Sqlite based proxy database implementation.
+    """
+
+    def __init__(self, dbpath):
+        
+        ADBAPISqliteMixin.__init__(self)
+        ProxyDB.__init__(self, "Proxies", "sqlite3", (fullServerPath(config.DataRoot, dbpath),))
+
+class ProxyPostgreSQLDB(ADBAPIPostgreSQLMixin, ProxyDB):
+    """
+    PostgreSQL based augment database implementation.
+    """
+
+    def __init__(self, host, database, user=None, password=None, dbtype=None):
+        
+        ADBAPIPostgreSQLMixin.__init__(self, )
+        ProxyDB.__init__(self, "Proxies", "pgdb", (), host=host, database=database, user=user, password=password,)
+        if dbtype:
+            ProxyDB.schema_type = dbtype
 
 ##
 # Utilities
