@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2011 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -70,6 +70,8 @@ class IDirectoryService(Interface):
     def recordWithCalendarUserAddress(address):
         """
         @param address: the calendar user address of the record to look up.
+        @type address: C{str}
+
         @return: an L{IDirectoryRecord} with the given calendar user
             address, or C{None} if no such record is found.  Note that
             some directory services may not be able to locate records
@@ -85,6 +87,12 @@ class IDirectoryService(Interface):
             match the given fields.
         """
 
+    def setRealm(realmName):
+        """
+        Set a new realm name for this (and nested services if any)
+
+        @param realmName: the realm name this service should use.
+        """
 
 class IDirectoryRecord(Interface):
     """
@@ -94,14 +102,29 @@ class IDirectoryRecord(Interface):
     recordType            = Attribute("The type of this record.")
     guid                  = Attribute("The GUID of this record.")
     uid                   = Attribute("The UID of this record.")
+    enabled               = Attribute("Determines whether this record should allow a principal to be created.")
+    serverID              = Attribute("Identifies the server that actually hosts data for the record.")
+    partitionID           = Attribute("Identifies the partition node that actually hosts data for the record.")
     shortNames            = Attribute("The names for this record.")
     authIDs               = Attribute("Alternative security identities for this record.")
     fullName              = Attribute("The full name of this record.")
     firstName             = Attribute("The first name of this record.")
     lastName              = Attribute("The last name of this record.")
     emailAddress          = Attribute("The email address of this record.")
-    calendarUserAddresses = Attribute("A set of calendar user addresses for this record.")
-    enabledForCalendaring = Attribute("Determines whether this record should be provisioned with a calendar home.")
+    enabledForCalendaring = Attribute("Determines whether this record creates a principal with a calendar home.")
+    enabledForAddressBooks = Attribute("Determines whether this record creates a principal with an address book home.")
+    calendarUserAddresses = Attribute(
+        """
+        An iterable of C{str}s representing calendar user addresses for this
+        L{IDirectoryRecord}.
+
+        A "calendar user address", as defined by U{RFC 2445 section
+        4.3.3<http://xml.resource.org/public/rfc/html/rfc2445.html#anchor50>},
+        is simply an URI which identifies this user.  Some of these URIs are
+        relative references to URLs from the root of the calendar server's HTTP
+        hierarchy.
+        """
+    )
 
     def members():
         """
