@@ -1,5 +1,6 @@
+# -*- test-case-name: twistedcaldav.directory.test.test_principal -*-
 ##
-# Copyright (c) 2006-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2011 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,12 +26,8 @@ __all__ = [
     "DirectoryPrincipalUIDProvisioningResource",
     "DirectoryPrincipalResource",
     "DirectoryCalendarPrincipalResource",
-    "format_list",
-    "format_principals",
-    "format_link",
 ]
 
-from cgi import escape
 from urllib import unquote
 from urlparse import urlparse
 
@@ -38,48 +35,58 @@ from twisted.cred.credentials import UsernamePassword
 from twisted.python.failure import Failure
 from twisted.internet.defer import inlineCallbacks, returnValue
 from twisted.internet.defer import succeed
-from twisted.web2.auth.digest import DigestedCredentials
-from twisted.web2 import responsecode
-from twisted.web2.http import HTTPError
-from twisted.web2.dav import davxml
-from twisted.web2.dav.util import joinURL
-from twisted.web2.dav.noneprops import NonePropertyStore
+from twisted.web.template import XMLFile, Element, renderer, tags
 
-from twistedcaldav.authkerb import NegotiateCredentials
+from twext.web2.auth.digest import DigestedCredentials
+from twext.web2 import responsecode
+from twext.web2.http import HTTPError
+from twext.web2.dav import davxml
+from twext.web2.dav.util import joinURL
+from twext.web2.dav.noneprops import NonePropertyStore
+
+from twext.python.log import Logger
+
+try:
+    from twistedcaldav.authkerb import NegotiateCredentials
+    NegotiateCredentials # sigh, pyflakes
+except ImportError:
+    NegotiateCredentials = None
+from twisted.python.modules import getModule
 from twistedcaldav.config import config
 from twistedcaldav.cache import DisabledCacheNotifier, PropfindCacheMixin
 
-from twistedcaldav.directory.calendaruserproxy import CalendarUserProxyDatabase
-from twistedcaldav.directory.resourceinfo import ResourceInfoDatabase
-from twistedcaldav.directory.calendaruserproxy import CalendarUserProxyPrincipalResource
+from twistedcaldav.extensions import DirectoryElement
+
+from twistedcaldav.directory.common import uidsResourceName
 from twistedcaldav.directory.directory import DirectoryService, DirectoryRecord
-from twistedcaldav.directory.util import NotFilePath
-from twistedcaldav.extensions import ReadOnlyResourceMixIn, DAVFile, DAVPrincipalResource
-from twistedcaldav.resource import CalendarPrincipalCollectionResource, CalendarPrincipalResource
+from twistedcaldav.extensions import ReadOnlyResourceMixIn, DAVPrincipalResource,\
+    DAVResourceWithChildrenMixin
+from twistedcaldav.resource import (
+    CalendarPrincipalCollectionResource, CalendarPrincipalResource
+)
 from twistedcaldav.directory.idirectory import IDirectoryService
-from twistedcaldav.log import Logger
 from twistedcaldav import caldavxml, customxml
 from twistedcaldav.customxml import calendarserver_namespace
-from twistedcaldav.directory.wiki import getWikiACL
 from twistedcaldav.scheduling.cuaddress import normalizeCUAddr
+from twistedcaldav.directory.wiki import getWikiACL
 
+thisModule = getModule(__name__)
 log = Logger()
 
-# Use __underbars__ convention to avoid conflicts with directory resource types.
-uidsResourceName = "__uids__"
-
-# FIXME: These should not be tied to DAVFile
-# The reason that they is that web2.dav only implements DAV methods on
-# DAVFile instead of DAVResource.  That should change.
 
 class PermissionsMixIn (ReadOnlyResourceMixIn):
     def defaultAccessControlList(self):
         return authReadACL
 
+
     @inlineCallbacks
     def accessControlList(self, request, inheritance=True, expanding=False, inherited_aces=None):
 
-        wikiACL = (yield getWikiACL(self, request))
+        try:
+            wikiACL = (yield getWikiACL(self, request))
+        except HTTPError:
+            wikiACL = None
+
         if wikiACL is not None:
             # ACL depends on wiki server...
             log.debug("Wiki ACL: %s" % (wikiACL.toxml(),))
@@ -90,7 +97,6 @@ class PermissionsMixIn (ReadOnlyResourceMixIn):
             returnValue(self.defaultAccessControlList())
 
 
-
 # Converter methods for recordsMatchingFields()
 #
 # A DAV property can be associated with one of these converter methods,
@@ -129,7 +135,6 @@ def cuAddressConverter(origCUAddr):
 class DirectoryProvisioningResource (
     PermissionsMixIn,
     CalendarPrincipalCollectionResource,
-    DAVFile,
 ):
     def __init__(self, url, directory):
         """
@@ -139,7 +144,7 @@ class DirectoryProvisioningResource (
         assert url.endswith("/"), "Collection URL must end in '/'"
 
         CalendarPrincipalCollectionResource.__init__(self, url)
-        DAVFile.__init__(self, NotFilePath(isdir=True))
+        DAVResourceWithChildrenMixin.__init__(self)
 
         self.directory = IDirectoryService(directory)
 
@@ -170,7 +175,7 @@ class DirectoryProvisioningResource (
         # Basic/Digest creds -> just lookup user name
         if isinstance(user, UsernamePassword) or isinstance(user, DigestedCredentials):
             return self.principalForUser(user.username)
-        elif isinstance(user, NegotiateCredentials):
+        elif NegotiateCredentials is not None and isinstance(user, NegotiateCredentials):
             authID = "Kerberos:%s" % (user.principal,)
             principal = self.principalForRecord(self.directory.recordWithAuthID(authID))
             if principal:
@@ -183,14 +188,14 @@ class DirectoryProvisioningResource (
     def principalForUID(self, uid):
         raise NotImplementedError("Subclass must implement principalForUID()")
 
-    def principalForRecord(self, record):
-        if record is None:
-            return None
-        return self.principalForUID(record.uid)
-
     def principalForCalendarUserAddress(self, address):
         raise NotImplementedError("Subclass must implement principalForCalendarUserAddress()")
 
+    def principalForRecord(self, record):
+        if record is None or not record.enabled:
+            return None
+        return self.principalForUID(record.uid)
+
     ##
     # DAV-property-to-record-field mapping
     ##
@@ -219,7 +224,7 @@ class DirectoryProvisioningResource (
         If property is a DAV property that maps to a directory field, return
         that field's name, otherwise return None
         """
-        field, converter, description, xmlClass = self._fieldMap.get(
+        field, converter, _ignore_description, _ignore_xmlClass = self._fieldMap.get(
             property.qname(), (None, None, None, None))
         if field is None:
             return (None, None)
@@ -229,7 +234,7 @@ class DirectoryProvisioningResource (
 
     def principalSearchPropertySet(self):
         props = []
-        for field, converter, description, xmlClass in self._fieldMap.itervalues():
+        for _ignore_field, _ignore_converter, description, xmlClass in self._fieldMap.itervalues():
             props.append(
                 davxml.PrincipalSearchProperty(
                     davxml.PropertyContainer(
@@ -321,17 +326,19 @@ class DirectoryPrincipalProvisioningResource (DirectoryProvisioningResource):
         # First see if the address is a principal URI
         principal = self._principalForURI(address)
         if principal:
-            if isinstance(principal, DirectoryCalendarPrincipalResource):
+            if isinstance(principal, DirectoryCalendarPrincipalResource) and principal.record.enabledForCalendaring:
                 return principal
         else:
             # Next try looking it up in the directory
             record = self.directory.recordWithCalendarUserAddress(address)
-            if record is not None:
+            if record is not None and record.enabled and record.enabledForCalendaring:
                 return self.principalForRecord(record)
 
         log.debug("No principal for calendar user address: %r" % (address,))
         return None
 
+    def principalForRecord(self, record):
+        return self.getChild(uidsResourceName).principalForRecord(record)
 
     ##
     # Static
@@ -383,6 +390,9 @@ class DirectoryPrincipalTypeProvisioningResource (DirectoryProvisioningResource)
     def principalForCalendarUserAddress(self, address):
         return self.parent.principalForCalendarUserAddress(address)
 
+    def principalForRecord(self, record):
+        return self.parent.principalForRecord(record)
+
     ##
     # Static
     ##
@@ -402,8 +412,9 @@ class DirectoryPrincipalTypeProvisioningResource (DirectoryProvisioningResource)
 
             def _recordShortnameExpand():
                 for record in self.directory.listRecords(self.recordType):
-                    for shortName in record.shortNames:
-                        yield shortName
+                    if record.enabled:
+                        for shortName in record.shortNames:
+                            yield shortName
 
             return _recordShortnameExpand()
         else:
@@ -442,6 +453,18 @@ class DirectoryPrincipalUIDProvisioningResource (DirectoryProvisioningResource):
     def principalForCalendarUserAddress(self, address):
         return self.parent.principalForCalendarUserAddress(address)
 
+    def principalForRecord(self, record):
+        if record is None or not record.enabled:
+            return None
+
+        if record.enabledForCalendaring or record.enabledForAddressBooks:
+            # XXX these are different features and one should not automatically
+            # imply the other...
+            principal = DirectoryCalendarPrincipalResource(self, record)
+        else:
+            principal = DirectoryPrincipalResource(self, record)
+        return principal
+
     ##
     # Static
     ##
@@ -462,16 +485,11 @@ class DirectoryPrincipalUIDProvisioningResource (DirectoryProvisioningResource):
             subType = None
 
         record = self.directory.recordWithUID(primaryUID)
-
-        if record is None:
-            log.err("No principal found for UID: %s" % (name,))
+        primaryPrincipal = self.principalForRecord(record)
+        if primaryPrincipal is None:
+            log.info("No principal found for UID: %s" % (name,))
             return None
 
-        if record.enabledForCalendaring:
-            primaryPrincipal = DirectoryCalendarPrincipalResource(self, record)
-        else:
-            primaryPrincipal = DirectoryPrincipalResource(self, record)
-
         if subType is None:
             return primaryPrincipal
         else:
@@ -488,16 +506,183 @@ class DirectoryPrincipalUIDProvisioningResource (DirectoryProvisioningResource):
     def principalCollections(self):
         return self.parent.principalCollections()
 
-class DirectoryPrincipalResource (PropfindCacheMixin, PermissionsMixIn, DAVPrincipalResource, DAVFile):
+
+
+class DirectoryPrincipalDetailElement(Element):
+    """
+    Element that can render the details of a
+    L{CalendarUserDirectoryPrincipalResource}.
+    """
+
+    loader = XMLFile(thisModule.filePath.sibling(
+        "directory-principal-resource.html").open()
+    )
+
+    def __init__(self, resource):
+        super(DirectoryPrincipalDetailElement, self).__init__()
+        self.resource = resource
+
+
+    @renderer
+    def serversEnabled(self, request, tag):
+        """
+        Renderer for when servers are enabled.
+        """
+        if not config.Servers.Enabled:
+            return ""
+        record = self.resource.record
+        return tag.fillSlots(
+            hostedAt=str(record.serverURI()),
+            partition=str(record.effectivePartitionID()),
+        )
+
+
+    @renderer
+    def principal(self, request, tag):
+        """
+        Top-level renderer in the template.
+        """
+        record = self.resource.record
+        return tag.fillSlots(
+            directoryGUID=str(record.service.guid),
+            realm=str(record.service.realmName),
+            principalGUID=str(record.guid),
+            recordType=str(record.recordType),
+            shortNames=",".join(record.shortNames),
+            securityIDs=",".join(record.authIDs),
+            fullName=str(record.fullName),
+            firstName=str(record.firstName),
+            lastName=str(record.lastName),
+            emailAddresses=formatList(record.emailAddresses),
+            principalUID=str(self.resource.principalUID()),
+            principalURL=formatLink(self.resource.principalURL()),
+            alternateURIs=formatLinks(self.resource.alternateURIs()),
+            groupMembers=self.resource.groupMembers().addCallback(
+                formatPrincipals
+            ),
+            groupMemberships=self.resource.groupMemberships().addCallback(
+                formatPrincipals
+            ),
+            readWriteProxyFor=self.resource.proxyFor(True).addCallback(
+                formatPrincipals
+            ),
+            readOnlyProxyFor=self.resource.proxyFor(False).addCallback(
+                formatPrincipals
+            ),
+        )
+
+
+    @renderer
+    def extra(self, request, tag):
+        """
+        No-op; implemented in subclass.
+        """
+        return ''
+
+
+    @renderer
+    def enabledForCalendaring(self, request, tag):
+        """
+        No-op; implemented in subclass.
+        """
+        return ''
+
+
+    @renderer
+    def enabledForAddressBooks(self, request, tag):
+        """
+        No-op; implemented in subclass.
+        """
+        return ''
+
+
+
+class DirectoryPrincipalElement(DirectoryElement):
+    """
+    L{DirectoryPrincipalElement} is a renderer for directory details.
+    """
+
+    @renderer
+    def resourceDetail(self, request, tag):
+        """
+        Render the directory principal's details.
+        """
+        return DirectoryPrincipalDetailElement(self.resource)
+
+
+class DirectoryCalendarPrincipalDetailElement(DirectoryPrincipalDetailElement):
+
+    @renderer
+    def extra(self, request, tag):
+        """
+        Renderer for extra directory body items for calendar/addressbook
+        principals.
+        """
+        return tag
+
+
+    @renderer
+    def enabledForCalendaring(self, request, tag):
+        """
+        Renderer which returns its tag when the wrapped record is enabled for
+        calendaring.
+        """
+        resource = self.resource
+        record = resource.record
+        if record.enabledForCalendaring:
+            return tag.fillSlots(
+                calendarUserAddresses=formatLinks(
+                    resource.calendarUserAddresses()
+                ),
+                calendarHomes=formatLinks(resource.calendarHomeURLs())
+            )
+        return ''
+
+
+    @renderer
+    def enabledForAddressBooks(self, request, tag):
+        """
+        Renderer which returnst its tag when the wrapped record is enabled for
+        addressbooks.
+        """
+        resource = self.resource
+        record = resource.record
+        if record.enabledForAddressBooks:
+            return tag.fillSlots(
+                addressBookHomes=formatLinks(resource.addressBookHomeURLs())
+            )
+        return ''
+
+
+
+class DirectoryCalendarPrincipalElement(DirectoryPrincipalElement):
+    """
+    L{DirectoryPrincipalElement} is a renderer for directory details, with
+    calendaring additions.
+    """
+
+    @renderer
+    def resourceDetail(self, request, tag):
+        """
+        Render the directory calendar principal's details.
+        """
+        return DirectoryCalendarPrincipalDetailElement(self.resource)
+
+
+class DirectoryPrincipalResource (
+        PropfindCacheMixin, PermissionsMixIn, DAVPrincipalResource):
     """
     Directory principal resource.
     """
 
-    liveProperties = tuple(DAVPrincipalResource.liveProperties) + (
-        (calendarserver_namespace, "first-name"       ),
-        (calendarserver_namespace, "last-name"        ),
-        (calendarserver_namespace, "email-address-set"),
-    )
+    def liveProperties(self):
+        
+        return super(DirectoryPrincipalResource, self).liveProperties() + (
+            (calendarserver_namespace, "first-name"       ),
+            (calendarserver_namespace, "last-name"        ),
+            (calendarserver_namespace, "email-address-set"),
+            davxml.ResourceID.qname(),
+        )
 
     cacheNotifierFactory = DisabledCacheNotifier
 
@@ -506,9 +691,9 @@ class DirectoryPrincipalResource (PropfindCacheMixin, PermissionsMixIn, DAVPrinc
         @param parent: the parent of this resource.
         @param record: the L{IDirectoryRecord} that this resource represents.
         """
-        super(DirectoryPrincipalResource, self).__init__(NotFilePath(isdir=True))
+        super(DirectoryPrincipalResource, self).__init__()
 
-        self.cacheNotifier = self.cacheNotifierFactory(self)
+        self.cacheNotifier = self.cacheNotifierFactory(self, cacheHandle="PrincipalToken")
 
         if self.isCollection():
             slash = "/"
@@ -517,10 +702,11 @@ class DirectoryPrincipalResource (PropfindCacheMixin, PermissionsMixIn, DAVPrinc
 
         assert record is not None, "Principal must have a directory record"
 
-        url = joinURL(parent.principalCollectionURL(), record.guid) + slash
 
         self.record = record
         self.parent = parent
+
+        url = joinURL(parent.principalCollectionURL(), self.principalUID()) + slash
         self._url   = url
 
         self._alternate_urls = tuple([
@@ -539,17 +725,19 @@ class DirectoryPrincipalResource (PropfindCacheMixin, PermissionsMixIn, DAVPrinc
 
         namespace, name = qname
 
-        if namespace == calendarserver_namespace:
+        if qname == davxml.ResourceID.qname():
+            returnValue(davxml.ResourceID(davxml.HRef.fromString("urn:uuid:%s" % (self.record.guid,))))
+        elif namespace == calendarserver_namespace:
             if name == "first-name":
                 firstName = self.record.firstName
-                if firstName:
+                if firstName is not None:
                     returnValue(customxml.FirstNameProperty(firstName))
                 else:
                     returnValue(None)
 
             elif name == "last-name":
                 lastName = self.record.lastName
-                if lastName:
+                if lastName is not None:
                     returnValue(customxml.LastNameProperty(lastName))
                 else:
                     returnValue(None)
@@ -574,51 +762,11 @@ class DirectoryPrincipalResource (PropfindCacheMixin, PermissionsMixIn, DAVPrinc
     # HTTP
     ##
 
-    @inlineCallbacks
-    def renderDirectoryBody(self, request):
-
-        extras = self.extraDirectoryBodyItems(request)
-        output = (yield super(DirectoryPrincipalResource, self).renderDirectoryBody(request))
-
-        members = (yield self.groupMembers())
-        
-        memberships = (yield self.groupMemberships())
-
-        proxyFor = (yield self.proxyFor(True))
-        readOnlyProxyFor = (yield self.proxyFor(False))
-        
-        returnValue("".join((
-            """<div class="directory-listing">"""
-            """<h1>Principal Details</h1>"""
-            """<pre><blockquote>"""
-            """Directory Information\n"""
-            """---------------------\n"""
-            """Directory GUID: %s\n"""         % (self.record.service.guid,),
-            """Realm: %s\n"""                  % (self.record.service.realmName,),
-            """\n"""
-            """Principal Information\n"""
-            """---------------------\n"""
-            """GUID: %s\n"""                   % (self.record.guid,),
-            """Record type: %s\n"""            % (self.record.recordType,),
-            """Short names: %s\n"""            % (",".join(self.record.shortNames),),
-            """Security Identities: %s\n"""    % (",".join(self.record.authIDs),),
-            """Full name: %s\n"""              % (self.record.fullName,),
-            """First name: %s\n"""             % (self.record.firstName,),
-            """Last name: %s\n"""              % (self.record.lastName,),
-            """Email addresses:\n"""           , format_list(self.record.emailAddresses),
-            """Principal UID: %s\n"""          % (self.principalUID(),),
-            """Principal URL: %s\n"""          % (format_link(self.principalURL()),),
-            """\nAlternate URIs:\n"""          , format_list(format_link(u) for u in self.alternateURIs()),
-            """\nGroup members:\n"""           , format_principals(members),
-            """\nGroup memberships:\n"""       , format_principals(memberships),
-            """\nRead-write Proxy For:\n"""    , format_principals(proxyFor),
-            """\nRead-only Proxy For:\n"""     , format_principals(readOnlyProxyFor),
-            """%s</pre></blockquote></div>"""  % extras,
-            output
-        )))
-
-    def extraDirectoryBodyItems(self, request):
-        return ""
+    def htmlElement(self):
+        """
+        Customize HTML rendering for directory principals.
+        """
+        return DirectoryPrincipalElement(self)
 
     ##
     # DAV
@@ -641,16 +789,13 @@ class DirectoryPrincipalResource (PropfindCacheMixin, PermissionsMixIn, DAVPrinc
         """
         Return the SQL database for calendar user proxies.
 
-        @return: the L{CalendarUserProxyDatabase} for the principal collection.
+        @return: the L{ProxyDB} for the principal collection.
         """
 
-        # Get the principal collection we are contained in
-        pcollection = self.parent.parent
-
         # The db is located in the principal collection root
-        if not hasattr(pcollection, "calendar_user_proxy_db"):
-            setattr(pcollection, "calendar_user_proxy_db", CalendarUserProxyDatabase(config.DataRoot))
-        return pcollection.calendar_user_proxy_db
+        from twistedcaldav.directory.calendaruserproxy import ProxyDBService
+        return ProxyDBService
+
 
     def alternateURIs(self):
         # FIXME: Add API to IDirectoryRecord for getting a record URI?
@@ -664,10 +809,22 @@ class DirectoryPrincipalResource (PropfindCacheMixin, PermissionsMixIn, DAVPrinc
 
     @inlineCallbacks
     def proxyFor(self, read_write, resolve_memberships=True):
+
         proxyFors = set()
 
         if resolve_memberships:
-            memberships = self._getRelatives("groups", infinity=True)
+            cache = getattr(self.record.service, "groupMembershipCache", None)
+            if cache:
+                log.debug("proxyFor is using groupMembershipCache")
+                guids = (yield self.record.cachedGroups())
+                memberships = set()
+                for guid in guids:
+                    principal = self.parent.principalForUID(guid)
+                    if principal:
+                        memberships.add(principal)
+            else:
+                memberships = self._getRelatives("groups", infinity=True)
+
             for membership in memberships:
                 results = (yield membership.proxyFor(read_write, False))
                 proxyFors.update(results)
@@ -732,7 +889,18 @@ class DirectoryPrincipalResource (PropfindCacheMixin, PermissionsMixIn, DAVPrinc
 
     @inlineCallbacks
     def groupMemberships(self, infinity=False):
-        groups = self._getRelatives("groups", infinity=infinity)
+
+        cache = getattr(self.record.service, "groupMembershipCache", None)
+        if cache:
+            log.debug("groupMemberships is using groupMembershipCache")
+            guids = (yield self.record.cachedGroups())
+            groups = set()
+            for guid in guids:
+                principal = self.parent.principalForUID(guid)
+                if principal:
+                    groups.add(principal)
+        else:
+            groups = self._getRelatives("groups", infinity=infinity)
 
         if config.EnableProxyPrincipals:
             # Get proxy group UIDs and map to principal resources
@@ -758,44 +926,37 @@ class DirectoryPrincipalResource (PropfindCacheMixin, PermissionsMixIn, DAVPrinc
     def principalUID(self):
         return self.record.uid
 
+    def serverURI(self):
+        return self.record.serverURI()
 
+    def server(self):
+        return self.record.server()
+
+    def partitionURI(self):
+        return self.record.partitionURI()
+
+    def locallyHosted(self):
+        return self.record.locallyHosted()
+    
+    def thisServer(self):
+        return self.record.thisServer()
+    
     ##
     # Extra resource info
     ##
 
-    def setAutoSchedule(self, autoSchedule):
-        self._resource_info_index().setAutoSchedule(self.record.guid, autoSchedule)
-
     @inlineCallbacks
+    def setAutoSchedule(self, autoSchedule):
+        self.record.autoSchedule = autoSchedule
+        augmentRecord = (yield self.record.service.augmentService.getAugmentRecord(self.record.guid, self.record.recordType))
+        augmentRecord.autoSchedule = autoSchedule
+        (yield self.record.service.augmentService.addAugmentRecords([augmentRecord]))
+
     def getAutoSchedule(self):
-        value = (yield self._resource_info_index().getAutoSchedule(self.record.guid))
-        if value is None:
-            # No value has been explicitly set yet.  If this is a user/group
-            # the default should be False.  If resource/location, True.
-            if self.record.recordType in (DirectoryService.recordType_locations,
-                DirectoryService.recordType_resources):
-                yield self.setAutoSchedule(True)
-                returnValue(True)
-            else:
-                yield self.setAutoSchedule(False)
-                returnValue(False)
-        else:
-            returnValue(value)
-
-
-    def _resource_info_index(self):
-        """
-        Return the resource info SQL database for this calendar principal.
-
-        @return: the L{ResourceInfoDatabase} for the calendar principal.
-        """
-
-        # The db is located in the data root
-        self.pcollection = self.parent.parent
-        if not hasattr(self.pcollection, "resource_info_db"):
-            setattr(self.pcollection, "resource_info_db", ResourceInfoDatabase(config.DataRoot))
-        return self.pcollection.resource_info_db
+        return self.record.autoSchedule
 
+    def getCUType(self):
+        return self.record.getCUType()
 
     ##
     # Static
@@ -821,21 +982,21 @@ class DirectoryPrincipalResource (PropfindCacheMixin, PermissionsMixIn, DAVPrinc
         return ()
 
 
-class DirectoryCalendarPrincipalResource (DirectoryPrincipalResource, CalendarPrincipalResource):
+class DirectoryCalendarPrincipalResource(DirectoryPrincipalResource,
+                                         CalendarPrincipalResource):
     """
     Directory calendar principal resource.
     """
 
-    @property
     def liveProperties(self):
-        # This needs to be a dynamic property because CalendarPrincipalResource
-        # liveProperties changes on the fly (drop box enabling)
-        return (
-            tuple(DirectoryPrincipalResource.liveProperties) +
-            tuple(CalendarPrincipalResource.liveProperties)
-        )
-
+        return DirectoryPrincipalResource.liveProperties(self) + CalendarPrincipalResource.liveProperties(self)
 
+    def calendarsEnabled(self):
+        return config.EnableCalDAV and self.record.enabledForCalendaring
+    
+    def addressBooksEnabled(self):
+        return config.EnableCardDAV and self.record.enabledForAddressBooks
+    
     @inlineCallbacks
     def readProperty(self, property, request):
         # Ouch, multiple inheritance.
@@ -844,17 +1005,17 @@ class DirectoryCalendarPrincipalResource (DirectoryPrincipalResource, CalendarPr
             result = (yield CalendarPrincipalResource.readProperty(self, property, request))
         returnValue(result)
 
-    def extraDirectoryBodyItems(self, request):
-        return "".join((
-            """\nCalendar homes:\n"""          , format_list(format_link(u) for u in self.calendarHomeURLs()),
-            """\nCalendar user addresses:\n""" , format_list(format_link(a) for a in self.calendarUserAddresses()),
-        ))
 
     ##
     # CalDAV
     ##
 
     def calendarUserAddresses(self):
+
+        # No CUAs if not enabledForCalendaring.
+        if not self.record.enabledForCalendaring:
+            return set()
+
         # Get any CUAs defined by the directory implementation.
         addresses = set(self.record.calendarUserAddresses)
 
@@ -863,14 +1024,42 @@ class DirectoryCalendarPrincipalResource (DirectoryPrincipalResource, CalendarPr
             addresses.add(uri)
             if config.HTTPPort:
                 addresses.add("http://%s:%s%s" % (config.ServerHostName, config.HTTPPort, uri))
-            if config.SSLPort:
+            if config.EnableSSL and config.SSLPort:
                 addresses.add("https://%s:%s%s" % (config.ServerHostName, config.SSLPort, uri))
 
-        # Add a UUID URI based on the record's GUID to the list.
-        addresses.add("urn:uuid:%s" % (self.record.guid,))
-
         return addresses
 
+
+    def htmlElement(self):
+        """
+        Customize HTML generation for calendar principals.
+        """
+        return DirectoryCalendarPrincipalElement(self)
+
+
+    def canonicalCalendarUserAddress(self):
+        """
+        Return a CUA for this principal, preferring in this order:
+            urn:uuid: form
+            mailto: form
+            first in calendarUserAddresses( ) list
+        """
+
+        cua = ""
+        for candidate in self.calendarUserAddresses():
+            # Pick the first one, but urn:uuid: and mailto: can override
+            if not cua:
+                cua = candidate
+            # But always immediately choose the urn:uuid: form
+            if candidate.startswith("urn:uuid:"):
+                cua = candidate
+                break
+            # Prefer mailto: if no urn:uuid:
+            elif candidate.startswith("mailto:"):
+                cua = candidate
+        return cua
+
+
     def enabledAsOrganizer(self):
         if self.record.recordType == DirectoryService.recordType_users:
             return True
@@ -883,23 +1072,34 @@ class DirectoryCalendarPrincipalResource (DirectoryPrincipalResource, CalendarPr
         else:
             return False
 
+    @inlineCallbacks
     def scheduleInbox(self, request):
-        home = self.calendarHome()
+        home = yield self.calendarHome(request)
         if home is None:
-            return succeed(None)
+            returnValue(None)
 
-        inbox = home.getChild("inbox")
+        inbox = yield home.getChild("inbox")
         if inbox is None:
-            return succeed(None)
+            returnValue(None)
 
-        return succeed(inbox)
+        returnValue(inbox)
+
+    @inlineCallbacks
+    def notificationCollection(self, request):
+
+        notification = None
+        if config.Sharing.Enabled:
+            home = yield self.calendarHome(request)
+            if home is not None:    
+                notification = yield home.getChild("notification")
+        returnValue(notification)
 
     def calendarHomeURLs(self):
-        home = self.calendarHome()
-        if home is None:
-            return ()
+        if self.record.enabledForCalendaring:
+            homeURL = self._homeChildURL(None)
         else:
-            return (home.url(),)
+            homeURL = ""
+        return (homeURL,) if homeURL else ()
 
     def scheduleInboxURL(self):
         return self._homeChildURL("inbox/")
@@ -913,23 +1113,77 @@ class DirectoryCalendarPrincipalResource (DirectoryPrincipalResource, CalendarPr
         else:
             return None
 
+    def notificationURL(self):
+        if config.Sharing.Enabled:
+            return self._homeChildURL("notification/")
+        else:
+            return None
+
+    def addressBookHomeURLs(self):
+        if self.record.enabledForAddressBooks:
+            homeURL = self._addressBookHomeChildURL(None)
+        else:
+            homeURL = ""
+        return (homeURL,) if homeURL else ()
+
     def _homeChildURL(self, name):
-        home = self.calendarHome()
-        if home is None:
+        if not hasattr(self, "calendarHomeURL"):
+            if not hasattr(self.record.service, "calendarHomesCollection"):
+                return None
+            self.calendarHomeURL = joinURL(
+                self.record.service.calendarHomesCollection.url(),
+                uidsResourceName,
+                self.record.uid
+            ) + "/"
+            
+            # Prefix with other server if needed
+            if not self.thisServer():
+                self.calendarHomeURL = joinURL(self.serverURI(), self.calendarHomeURL)
+
+        url = self.calendarHomeURL
+        if url is None:
             return None
         else:
-            return joinURL(home.url(), name)
+            return joinURL(url, name) if name else url
 
-    def calendarHome(self):
+
+    def calendarHome(self, request):
         # FIXME: self.record.service.calendarHomesCollection smells like a hack
-        # See CalendarHomeProvisioningFile.__init__()
         service = self.record.service
         if hasattr(service, "calendarHomesCollection"):
-            return service.calendarHomesCollection.homeForDirectoryRecord(self.record)
+            return service.calendarHomesCollection.homeForDirectoryRecord(self.record, request)
         else:
-            return None
+            return succeed(None)
 
 
+    def _addressBookHomeChildURL(self, name):
+        if not hasattr(self, "addressBookHomeURL"):
+            if not hasattr(self.record.service, "addressBookHomesCollection"):
+                return None
+            self.addressBookHomeURL = joinURL(
+                self.record.service.addressBookHomesCollection.url(),
+                uidsResourceName,
+                self.record.uid
+            ) + "/"
+            
+            # Prefix with other server if needed
+            if not self.thisServer():
+                self.addressBookHomeURL = joinURL(self.serverURI(), self.addressBookHomeURL)
+
+        url = self.addressBookHomeURL
+        if url is None:
+            return None
+        else:
+            return joinURL(url, name) if name else url
+
+    def addressBookHome(self, request):
+        # FIXME: self.record.service.addressBookHomesCollection smells like a hack
+        service = self.record.service
+        if hasattr(service, "addressBookHomesCollection"):
+            return service.addressBookHomesCollection.homeForDirectoryRecord(self.record, request)
+        else:
+            return succeed(None)
+
     ##
     # Static
     ##
@@ -938,8 +1192,13 @@ class DirectoryCalendarPrincipalResource (DirectoryPrincipalResource, CalendarPr
         if name == "":
             return self
 
-        if config.EnableProxyPrincipals and name in ("calendar-proxy-read", "calendar-proxy-write"):
-            return CalendarUserProxyPrincipalResource(self, name)
+        if config.EnableProxyPrincipals and name in ("calendar-proxy-read",
+                                                     "calendar-proxy-write"):
+            # name is required to be str
+            from twistedcaldav.directory.calendaruserproxy import (
+                CalendarUserProxyPrincipalResource
+            )
+            return CalendarUserProxyPrincipalResource(self, str(name))
         else:
             return None
 
@@ -962,21 +1221,11 @@ authReadACL = davxml.ACL(
     ),
 )
 
-def format_list(items, *args):
-    def genlist():
-        try:
-            item = None
-            for item in items:
-                yield " -> %s\n" % (item,)
-            if item is None:
-                yield " '()\n"
-        except Exception, e:
-            log.err("Exception while rendering: %s" % (e,))
-            Failure().printTraceback()
-            yield "  ** %s **: %s\n" % (e.__class__.__name__, e)
-    return "".join(genlist())
 
-def format_principals(principals):
+def formatPrincipals(principals):
+    """
+    Format a list of principals into some twisted.web.template DOM objects.
+    """
     def recordKey(principal):
         try:
             record = principal.record
@@ -985,7 +1234,6 @@ def format_principals(principals):
                 record = principal.parent.record
             except:
                 return None
-
         return (record.recordType, record.shortNames[0])
 
     def describe(principal):
@@ -994,11 +1242,49 @@ def format_principals(principals):
         else:
             return ""
 
-    return format_list(
-        """<a href="%s">%s%s</a>"""
-        % (principal.principalURL(), escape(str(principal)), describe(principal))
+    return formatList(
+        tags.a(href=principal.principalURL())(
+            str(principal), describe(principal)
+        )
         for principal in sorted(principals, key=recordKey)
     )
 
-def format_link(url):
-    return """<a href="%s">%s</a>""" % (url, url)
+
+def formatList(iterable):
+    """
+    Format a list of stuff as an interable.
+    """
+    thereAreAny = False
+    try:
+        item = None
+        for item in iterable:
+            thereAreAny = True
+            yield " -> "
+            if item is None:
+                yield "None"
+            else:
+                yield item
+            yield "\n"
+    except Exception, e:
+        log.err("Exception while rendering: %s" % (e,))
+        Failure().printTraceback()
+        yield "  ** %s **: %s\n" % (e.__class__.__name__, e)
+    if not thereAreAny:
+        yield " '()\n"
+
+
+
+def formatLink(url):
+    """
+    Convert a URL string into some twisted.web.template DOM objects for
+    rendering as a link to itself.
+    """
+    return tags.a(href=url)(url)
+
+
+def formatLinks(urls):
+    """
+    Format a list of URL strings as a list of twisted.web.template DOM links.
+    """
+    return formatList(formatLink(link) for link in urls)
+
