@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2007 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,53 +14,45 @@
 # limitations under the License.
 ##
 
-import os
-
 from twisted.internet.defer import inlineCallbacks
-from twisted.web2.dav import davxml
-from twisted.web2.test.test_server import SimpleRequest
+from twext.web2.dav import davxml
 
 from twistedcaldav import caldavxml
-from twistedcaldav.directory.principal import DirectoryPrincipalProvisioningResource
-from twistedcaldav.directory.test.test_xmlfile import xmlFile
-from twistedcaldav.directory.xmlfile import XMLDirectoryService
-from twistedcaldav.static import CalendarHomeProvisioningFile
 
-import twistedcaldav.test.util
+from twistedcaldav.test.util import TestCase
+from twext.web2.test.test_server import SimpleRequest
+from twistedcaldav.directory.util import transactionFromRequest
 
-class ProvisionedCalendars (twistedcaldav.test.util.TestCase):
+class ProvisionedCalendars (TestCase):
     """
     Directory service provisioned principals.
     """
+
+    @inlineCallbacks
     def setUp(self):
-        super(ProvisionedCalendars, self).setUp()
-        
-        # Setup the initial directory
-        self.xmlfile = self.mktemp()
-        fd = open(self.xmlfile, "w")
-        fd.write(open(xmlFile.path, "r").read())
-        fd.close()
-        self.directoryService = XMLDirectoryService({'xmlFile' : self.xmlfile})
-        
-        # Set up a principals hierarchy for each service we're testing with
-        name = "principals"
-        url = "/" + name + "/"
-
-        provisioningResource = DirectoryPrincipalProvisioningResource(url, self.directoryService)
-
-        self.site.resource.putChild("principals", provisioningResource)
+        yield super(ProvisionedCalendars, self).setUp()
 
+        self.createStockDirectoryService()
         self.setupCalendars()
 
-        self.site.resource.setAccessControlList(davxml.ACL())
 
-    def setupCalendars(self):
-        calendarCollection = CalendarHomeProvisioningFile(
-            os.path.join(self.docroot, "calendars"),
-            self.directoryService,
-            "/calendars/"
-        )
-        self.site.resource.putChild("calendars", calendarCollection)
+    def oneRequest(self, uri):
+        req = self._cleanupRequest = SimpleRequest(self.site, "GET", uri)
+        return req
+
+
+    def tearDown(self):
+        """
+        If the request started by this test has a transaction, commit it.
+        Otherwise, don't bother.
+        """
+        class JustForCleanup(object):
+            def newTransaction(self, *whatever):
+                return self
+            def commit(self):
+                return
+        return transactionFromRequest(self._cleanupRequest, JustForCleanup()).commit()
+
 
     def test_NonExistentCalendarHome(self):
 
@@ -68,7 +60,7 @@ class ProvisionedCalendars (twistedcaldav.test.util.TestCase):
             if resource is not None:
                 self.fail("Incorrect response to GET on non-existent calendar home.")
 
-        request = SimpleRequest(self.site, "GET", "/calendars/users/12345/")
+        request = self.oneRequest("/calendars/users/12345/")
         d = request.locateResource(request.uri)
         d.addCallback(_response)
 
@@ -78,7 +70,7 @@ class ProvisionedCalendars (twistedcaldav.test.util.TestCase):
             if resource is None:
                 self.fail("Incorrect response to GET on existent calendar home.")
 
-        request = SimpleRequest(self.site, "GET", "/calendars/users/wsanchez/")
+        request = self.oneRequest("/calendars/users/wsanchez/")
         d = request.locateResource(request.uri)
         d.addCallback(_response)
 
@@ -88,7 +80,7 @@ class ProvisionedCalendars (twistedcaldav.test.util.TestCase):
             if resource is None:
                 self.fail("Incorrect response to GET on existent calendar.")
 
-        request = SimpleRequest(self.site, "GET", "/calendars/users/wsanchez/calendar/")
+        request = self.oneRequest("/calendars/users/wsanchez/calendar/")
         d = request.locateResource(request.uri)
         d.addCallback(_response)
 
@@ -98,14 +90,14 @@ class ProvisionedCalendars (twistedcaldav.test.util.TestCase):
             if resource is None:
                 self.fail("Incorrect response to GET on existent inbox.")
 
-        request = SimpleRequest(self.site, "GET", "/calendars/users/wsanchez/inbox/")
+        request = self.oneRequest("/calendars/users/wsanchez/inbox/")
         d = request.locateResource(request.uri)
         d.addCallback(_response)
 
     @inlineCallbacks
     def test_CalendarTranspProperty(self):
 
-        request = SimpleRequest(self.site, "GET", "/calendars/users/wsanchez/calendar/")
+        request = self.oneRequest("/calendars/users/wsanchez/calendar/")
 
         # Get calendar first
         calendar = (yield request.locateResource("/calendars/users/wsanchez/calendar/"))
@@ -158,3 +150,23 @@ class ProvisionedCalendars (twistedcaldav.test.util.TestCase):
         transp = (yield calendar.readProperty(caldavxml.ScheduleCalendarTransp, request))
         self.assertEqual(transp, caldavxml.ScheduleCalendarTransp(caldavxml.Transparent()))
 
+        # Force trailing slash on fbset
+        inbox.writeDeadProperty(caldavxml.CalendarFreeBusySet(
+            davxml.HRef.fromString("/calendars/__uids__/6423F94A-6B76-4A3A-815B-D52CFD77935D/calendar/"),
+        ))
+
+        # Now remove the dead property to simulate the old calendar server state with
+        # a calendar listed in the fbset
+        yield calendar.removeDeadProperty(caldavxml.ScheduleCalendarTransp)
+        fbset = (yield inbox.readProperty(caldavxml.CalendarFreeBusySet, request))
+        self.assertEqual(fbset, caldavxml.CalendarFreeBusySet(
+            davxml.HRef.fromString("/calendars/__uids__/6423F94A-6B76-4A3A-815B-D52CFD77935D/calendar/"),
+        ))
+
+        # Calendar has opaque property derived from inbox
+        transp = (yield calendar.hasProperty(caldavxml.ScheduleCalendarTransp, request))
+        self.assertTrue(transp)
+
+        transp = (yield calendar.readProperty(caldavxml.ScheduleCalendarTransp, request))
+        self.assertEqual(transp, caldavxml.ScheduleCalendarTransp(caldavxml.Opaque()))
+
