@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2007 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,60 +13,34 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 ##
-from twistedcaldav.directory.principal import DirectoryPrincipalProvisioningResource
-from twistedcaldav.static import CalendarHomeProvisioningFile
 from twistedcaldav.directory.directory import DirectoryService
 
-import os
+from twext.web2.dav import davxml
+from twext.web2.dav.resource import AccessDeniedError
+from twext.web2.test.test_server import SimpleRequest
 
-from twisted.web2.dav import davxml
-from twisted.web2.dav.resource import AccessDeniedError
-from twisted.web2.test.test_server import SimpleRequest
-
-from twistedcaldav.directory.xmlfile import XMLDirectoryService
-from twistedcaldav.directory.test.test_xmlfile import xmlFile
-
-import twistedcaldav.test.util
+from twistedcaldav.test.util import TestCase, xmlFile
 
 
-class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
+class ProvisionedPrincipals(TestCase):
     """
     Directory service provisioned principals.
     """
     def setUp(self):
         super(ProvisionedPrincipals, self).setUp()
-        
+
         # Setup the initial directory
-        self.xmlfile = self.mktemp()
-        fd = open(self.xmlfile, "w")
-        fd.write(open(xmlFile.path, "r").read())
-        fd.close()
-        self.directoryService = XMLDirectoryService({'xmlFile' : self.xmlfile})
-        
-        # Set up a principals hierarchy for each service we're testing with
-        name = "principals"
-        url = "/" + name + "/"
-
-        provisioningResource = DirectoryPrincipalProvisioningResource(url, self.directoryService)
-
-        self.site.resource.putChild("principals", provisioningResource)
-
+        self.createStockDirectoryService()
         self.setupCalendars()
 
         self.site.resource.setAccessControlList(davxml.ACL())
 
-    def setupCalendars(self):
-        calendarCollection = CalendarHomeProvisioningFile(
-            os.path.join(self.docroot, "calendars"),
-            self.directoryService,
-            "/calendars/"
-        )
-        self.site.resource.putChild("calendars", calendarCollection)
 
     def resetCalendars(self):
         del self.site.resource.putChildren["calendars"]
         self.setupCalendars()
 
+
     def test_guidchange(self):
         """
         DirectoryPrincipalResource.proxies()
@@ -78,10 +52,9 @@ class ProvisionedPrincipals (twistedcaldav.test.util.TestCase):
         
         def privs1(result):
             # Change GUID in record
-            fd = open(self.xmlfile, "w")
-            fd.write(open(xmlFile.path, "r").read().replace(oldUID, newUID))
-            fd.close()
-            fd = None
+            self.xmlFile.setContent(
+                xmlFile.getContent().replace(oldUID, newUID)
+            )
 
             # Force re-read of records (not sure why _fileInfo has to be wiped here...)
             self.directoryService._fileInfo = (0, 0)
