@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,28 +14,37 @@
 # limitations under the License.
 ##
 
-from twisted.internet.defer import DeferredList, inlineCallbacks, returnValue
-from twisted.web2.dav import davxml
+from twisted.internet.defer import DeferredList, inlineCallbacks, succeed
+from twext.web2.dav import davxml
 
 from twistedcaldav.directory.directory import DirectoryService
-from twistedcaldav.directory.test.test_xmlfile import xmlFile
-from twistedcaldav.directory.principal import DirectoryPrincipalProvisioningResource
-from twistedcaldav.directory.principal import DirectoryPrincipalResource
-from twistedcaldav.directory.xmlaccountsparser import XMLAccountsParser
+from twistedcaldav.test.util import xmlFile, augmentsFile, proxiesFile
+from twistedcaldav.directory.principal import DirectoryPrincipalProvisioningResource,\
+    DirectoryPrincipalResource
 from twistedcaldav.directory.xmlfile import XMLDirectoryService
 
 import twistedcaldav.test.util
-from twistedcaldav.config import config
+from twistedcaldav.directory import augment, calendaruserproxy
+from twistedcaldav.directory.calendaruserproxyloader import XMLCalendarUserProxyLoader
 
 
 class ProxyPrincipals (twistedcaldav.test.util.TestCase):
     """
     Directory service provisioned principals.
     """
+
+    @inlineCallbacks
     def setUp(self):
         super(ProxyPrincipals, self).setUp()
 
-        self.directoryService = XMLDirectoryService({'xmlFile' : xmlFile})
+        self.directoryService = XMLDirectoryService(
+            {
+                'xmlFile' : xmlFile,
+                'augmentService' :
+                    augment.AugmentXMLDB(xmlFiles=(augmentsFile.path,)),
+            }
+        )
+        calendaruserproxy.ProxyDBService = calendaruserproxy.ProxySqliteDB("proxies.sqlite")
 
         # Set up a principals hierarchy for each service we're testing with
         self.principalRootResources = {}
@@ -48,6 +57,12 @@ class ProxyPrincipals (twistedcaldav.test.util.TestCase):
 
         self.principalRootResources[self.directoryService.__class__.__name__] = provisioningResource
 
+        yield XMLCalendarUserProxyLoader(proxiesFile.path).updateProxyDB()
+
+    def tearDown(self):
+        """ Empty the proxy db between tests """
+        return calendaruserproxy.ProxyDBService.clean()
+
     def _getPrincipalByShortName(self, type, name):
         provisioningResource = self.principalRootResources[self.directoryService.__class__.__name__]
         return provisioningResource.principalForShortName(type, name)
@@ -90,7 +105,7 @@ class ProxyPrincipals (twistedcaldav.test.util.TestCase):
             proxyPrincipal = self._getPrincipalByShortName(proxyPrincipal[0], proxyPrincipal[1])
         members.add(proxyPrincipal)
         
-        principal.setGroupMemberSetPrincipals(members)
+        yield principal.setGroupMemberSetPrincipals(members)
 
     @inlineCallbacks
     def _removeProxy(self, recordType, recordName, subPrincipalName, proxyRecordType, proxyRecordName):
@@ -105,14 +120,15 @@ class ProxyPrincipals (twistedcaldav.test.util.TestCase):
                 members.remove(p)
                 break
         
-        principal.setGroupMemberSetPrincipals(members)
+        yield principal.setGroupMemberSetPrincipals(members)
 
+    @inlineCallbacks
     def _clearProxy(self, principal, subPrincipalName):
 
         if isinstance(principal, tuple):
             principal = self._getPrincipalByShortName(principal[0], principal[1])
         principal = principal.getChild(subPrincipalName)
-        principal.setGroupMemberSetPrincipals(set())
+        yield principal.setGroupMemberSetPrincipals(set())
 
     @inlineCallbacks
     def _proxyForTest(self, recordType, recordName, expectedProxies, read_write):
@@ -121,6 +137,19 @@ class ProxyPrincipals (twistedcaldav.test.util.TestCase):
         proxies = sorted([principal.displayName() for principal in proxies])
         self.assertEquals(proxies, sorted(expectedProxies))
 
+    @inlineCallbacks
+    def test_multipleProxyAssignmentsAtOnce(self):
+        yield self._proxyForTest(
+            DirectoryService.recordType_users, "userb",
+            ('a',),
+            True
+        )
+        yield self._proxyForTest(
+            DirectoryService.recordType_users, "userc",
+            ('a',),
+            True
+        )
+
     def test_groupMembersRegular(self):
         """
         DirectoryPrincipalResource.expandedGroupMembers()
@@ -240,6 +269,7 @@ class ProxyPrincipals (twistedcaldav.test.util.TestCase):
         d.addCallback(check)
         return d
 
+    @inlineCallbacks
     def test_setGroupMemberSet(self):
         class StubMemberDB(object):
             def __init__(self):
@@ -247,9 +277,10 @@ class ProxyPrincipals (twistedcaldav.test.util.TestCase):
 
             def setGroupMembers(self, uid, members):
                 self.members = members
+                return succeed(None)
 
             def getMembers(self, uid):
-                return self.members
+                return succeed(self.members)
 
 
         user = self._getPrincipalByShortName(self.directoryService.recordType_users,
@@ -267,19 +298,20 @@ class ProxyPrincipals (twistedcaldav.test.util.TestCase):
             davxml.HRef.fromString(
                 "/XMLDirectoryService/__uids__/5FF60DAD-0BDE-4508-8C77-15F0CA5C8DD1/"))
 
-        proxyGroup.setGroupMemberSet(new_members, None)
+        yield proxyGroup.setGroupMemberSet(new_members, None)
 
         self.assertEquals(
             set([str(p) for p in memberdb.members]),
             set(["5FF60DAD-0BDE-4508-8C77-15F0CA5C8DD1",
                  "8B4288F6-CC82-491D-8EF9-642EF4F3E7D0"]))
 
-
+    @inlineCallbacks
     def test_setGroupMemberSetNotifiesPrincipalCaches(self):
         class StubCacheNotifier(object):
             changedCount = 0
             def changed(self):
                 self.changedCount += 1
+                return succeed(None)
 
         user = self._getPrincipalByShortName(self.directoryService.recordType_users, "cdaboo")
 
@@ -290,11 +322,11 @@ class ProxyPrincipals (twistedcaldav.test.util.TestCase):
         oldCacheNotifier = DirectoryPrincipalResource.cacheNotifierFactory
 
         try:
-            DirectoryPrincipalResource.cacheNotifierFactory = (lambda _1, _2: notifier)
+            DirectoryPrincipalResource.cacheNotifierFactory = (lambda _1, _2, **kwargs: notifier)
 
             self.assertEquals(notifier.changedCount, 0)
 
-            proxyGroup.setGroupMemberSet(
+            yield proxyGroup.setGroupMemberSet(
                 davxml.GroupMemberSet(
                     davxml.HRef.fromString(
                         "/XMLDirectoryService/__uids__/5FF60DAD-0BDE-4508-8C77-15F0CA5C8DD1/")),
@@ -308,7 +340,7 @@ class ProxyPrincipals (twistedcaldav.test.util.TestCase):
 
         return self._proxyForTest(
             DirectoryService.recordType_users, "wsanchez", 
-            ("Mecury Seven", "Gemini Twelve", "Apollo Eleven", "Orion", ),
+            ("Mercury Seven", "Gemini Twelve", "Apollo Eleven", "Orion", ),
             True
         )
 
@@ -323,7 +355,7 @@ class ProxyPrincipals (twistedcaldav.test.util.TestCase):
 
         yield self._proxyForTest(
             DirectoryService.recordType_users, "wsanchez", 
-            ("Mecury Seven", "Gemini Twelve", "Apollo Eleven", "Orion", ),
+            ("Mercury Seven", "Gemini Twelve", "Apollo Eleven", "Orion", ),
             True
         )
 
@@ -376,156 +408,36 @@ class ProxyPrincipals (twistedcaldav.test.util.TestCase):
                 ("Chris Lecroy",),
             )
 
+
     @inlineCallbacks
-    def test_InvalidUserProxy(self):
+    def test_NonAsciiProxy(self):
+        """
+        Ensure that principalURLs with non-ascii don't cause problems
+        within CalendarUserProxyPrincipalResource
+        """
+
+        recordType = DirectoryService.recordType_users
+        proxyType = "calendar-proxy-read"
+
+        record = self.directoryService.recordWithGUID("320B73A1-46E2-4180-9563-782DFDBE1F63")
+        provisioningResource = self.principalRootResources[self.directoryService.__class__.__name__]
+        principal =  provisioningResource.principalForRecord(record)
+        proxyPrincipal = provisioningResource.principalForShortName(recordType,
+            "wsanchez")
+
+        yield self._addProxy(principal, proxyType, proxyPrincipal)
+        memberships = yield proxyPrincipal._calendar_user_proxy_index().getMemberships(proxyPrincipal.principalUID())
+        for uid in memberships:
+            provisioningResource.principalForUID(uid)
 
 
-        # Set up the in-memory (non-null) memcacher:
-        config.ProcessType = "Single"
-        principal = self._getPrincipalByShortName(
-            DirectoryService.recordType_users, "wsanchez")
-        db = principal._calendar_user_proxy_index()
-
-        # Set the clock to the epoch:
-        theTime = 0
-        db._memcacher.theTime = theTime
-
-
-        for doMembershipFirst in (True, False):
-            for proxyType in ("calendar-proxy-read", "calendar-proxy-write"):
-
-                principal = self._getPrincipalByShortName(DirectoryService.recordType_users, "wsanchez")
-                proxyGroup = principal.getChild(proxyType)
-
-                testPrincipal = self._getPrincipalByShortName(DirectoryService.recordType_users, "cdaboo")
-
-                fakePrincipal = self._getPrincipalByShortName(DirectoryService.recordType_users, "dreid")
-                fakeProxyGroup = fakePrincipal.getChild(proxyType)
-
-                yield self._addProxy(
-                    principal,
-                    proxyType,
-                    testPrincipal,
-                )
-
-                members = yield proxyGroup._index().getMembers(proxyGroup.uid)
-                self.assertEquals(len(members), 1)
-
-                yield self._addProxy(
-                    fakePrincipal,
-                    proxyType,
-                    testPrincipal,
-                )
-                members = yield fakeProxyGroup._index().getMembers(fakeProxyGroup.uid)
-                self.assertEquals(len(members), 1)
-
-                uids = [p.principalUID() for p in (yield testPrincipal.groupMemberships())]
-                self.assertTrue("5FF60DAD-0BDE-4508-8C77-15F0CA5C8DD1#%s" % (proxyType,) in uids)
-
-                memberships = yield testPrincipal._calendar_user_proxy_index().getMemberships(testPrincipal.principalUID())
-                self.assertEquals(len(memberships), 2)
-
-                yield self._addProxy(
-                    principal,
-                    proxyType,
-                    fakePrincipal,
-                )
-                members = yield proxyGroup._index().getMembers(proxyGroup.uid)
-                self.assertEquals(len(members), 2)
-
-                # Remove the dreid user from the directory service
-
-                delRec = self.directoryService.recordWithShortName(
-                    DirectoryService.recordType_users, "dreid")
-                for cache in self.directoryService._recordCaches.itervalues():
-                   cache.removeRecord(delRec)
-                del self.directoryService._accounts()[
-                    DirectoryService.recordType_users]["dreid"]
-
-
-                cacheTimeout = config.DirectoryService.params.get("cacheTimeout", 30) * 60 * 2
-
-                @inlineCallbacks
-                def _membershipTest():
-
-                    uids = [p.principalUID() for p in (yield testPrincipal.groupMemberships())]
-                    self.assertTrue("5FF60DAD-0BDE-4508-8C77-15F0CA5C8DD1#%s" % (proxyType,) not in uids)
-
-                    memberships = yield testPrincipal._calendar_user_proxy_index().getMemberships(testPrincipal.principalUID())
-                    self.assertEquals(len(memberships), 1)
-
-                @inlineCallbacks
-                def _membersTest(theTime):
-                    yield self._groupMembersTest(
-                        DirectoryService.recordType_users, "wsanchez",
-                        proxyType,
-                        ("Cyrus Daboo",),
-                    )
-
-                    # Trigger the proxy DB clean up, which won't actually
-                    # remove anything because we haven't exceeded the timeout
-                    yield proxyGroup.groupMembers()
-
-                    # Advance 10 seconds
-                    theTime += 10
-                    db._memcacher.theTime = theTime
-
-                    # When we first examine the members, we have not exceeded
-                    # the clean-up timeout, so we'll still have 2:
-                    members = yield proxyGroup._index().getMembers(proxyGroup.uid)
-                    self.assertEquals(len(members), 2)
-
-                    # Restore removed user
-                    parser = XMLAccountsParser(self.directoryService.xmlFile)
-                    self.directoryService._parsedAccounts = parser.items
-                    self.directoryService.recordWithShortName(
-                        DirectoryService.recordType_users, "dreid")
-
-                    # Trigger the proxy DB clean up, which will actually
-                    # remove the deletion timer because the principal has been
-                    # restored
-                    yield proxyGroup.groupMembers()
-
-                    # Verify the deletion timer has been removed
-                    result = yield db._memcacher.checkDeletionTimer("5FF60DAD-0BDE-4508-8C77-15F0CA5C8DD1")
-                    self.assertEquals(result, None)
-
-                    # Remove the dreid user from the directory service
-                    delRec = self.directoryService.recordWithShortName(
-                        DirectoryService.recordType_users, "dreid")
-                    for cache in self.directoryService._recordCaches.itervalues():
-                       cache.removeRecord(delRec)
-                    del self.directoryService._accounts()[
-                        DirectoryService.recordType_users]["dreid"]
-
-                    # Trigger the proxy DB clean up, which won't actually
-                    # remove anything because we haven't exceeded the timeout
-                    yield proxyGroup.groupMembers()
-
-                    # Advance beyond the timeout
-                    theTime += cacheTimeout
-                    db._memcacher.theTime = theTime
-
-                    # Trigger the proxy DB clean up
-                    yield proxyGroup.groupMembers()
-
-                    # The missing principal has now been cleaned out of the
-                    # proxy DB
-                    members = yield proxyGroup._index().getMembers(proxyGroup.uid)
-                    self.assertEquals(len(members), 1)
-                    returnValue(theTime)
-
-
-                if doMembershipFirst:
-                    yield _membershipTest()
-                    theTime = yield _membersTest(theTime)
-                else:
-                    theTime = yield _membersTest(theTime)
-                    yield _membershipTest()
-
-                # Restore removed user
-                parser = XMLAccountsParser(self.directoryService.xmlFile)
-                self.directoryService._parsedAccounts = parser.items
-
-                self._clearProxy(principal, proxyType)
-                self._clearProxy(fakePrincipal, proxyType)
+    @inlineCallbacks
+    def test_getAllMembers(self):
+        """
+        getAllMembers( ) returns the unique set of guids that have been
+        delegated-to directly
+        """
+        self.assertEquals(
+            set((yield calendaruserproxy.ProxyDBService.getAllMembers())),
+            set([u'6423F94A-6B76-4A3A-815B-D52CFD77935D', u'8A985493-EE2C-4665-94CF-4DFEA3A89500', u'9FF60DAD-0BDE-4508-8C77-15F0CA5C8DD2', u'both_coasts', u'left_coast', u'non_calendar_group', u'recursive1_coasts', u'recursive2_coasts'])
+        )
