@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,19 +14,13 @@
 # limitations under the License.
 ##
 
-import os
+from twext.python.filepath import CachingFilePath as FilePath
 
-from twisted.python.filepath import FilePath
-from twisted.internet.defer import inlineCallbacks
-
-from twistedcaldav.directory.calendaruserproxy import CalendarUserProxyDatabase
+from twistedcaldav.directory import augment
 from twistedcaldav.directory.directory import DirectoryService
 import twistedcaldav.directory.test.util
 from twistedcaldav.directory.xmlfile import XMLDirectoryService
-from twistedcaldav.directory.resourceinfo import ResourceInfoDatabase
-from twistedcaldav.config import config
-
-xmlFile = FilePath(os.path.join(os.path.dirname(__file__), "accounts.xml"))
+from twistedcaldav.test.util import TestCase, xmlFile, augmentsFile
 
 # FIXME: Add tests for GUID hooey, once we figure out what that means here
 
@@ -45,8 +39,8 @@ class XMLFileBase(object):
         "lecroy"     : { "password": "yorcel",     "guid": "8B4288F6-CC82-491D-8EF9-642EF4F3E7D0", "addresses": ("mailto:lecroy@example.com",)   },
         "dreid"      : { "password": "dierd",      "guid": "5FF60DAD-0BDE-4508-8C77-15F0CA5C8DD1", "addresses": ("mailto:dreid@example.com",)    },
         "nocalendar" : { "password": "radnelacon", "guid": "543D28BA-F74F-4D5F-9243-B3E3A61171E5", "addresses": () },
-        "user01"     : { "password": "01user",     "guid": None                                  , "addresses": () },
-        "user02"     : { "password": "02user",     "guid": None                                  , "addresses": () },
+        "user01"     : { "password": "01user",     "guid": None                                  , "addresses": ("mailto:c4ca4238a0@example.com",) },
+        "user02"     : { "password": "02user",     "guid": None                                  , "addresses": ("mailto:c81e728d9d@example.com",) },
     }
 
     groups = {
@@ -88,6 +82,12 @@ class XMLFileBase(object):
             xmlFile.copyTo(self._xmlFile)
         return self._xmlFile
 
+    def augmentsFile(self):
+        if not hasattr(self, "_augmentsFile"):
+            self._augmentsFile = FilePath(self.mktemp())
+            augmentsFile.copyTo(self._augmentsFile)
+        return self._augmentsFile
+
 class XMLFile (
     XMLFileBase,
     twistedcaldav.directory.test.util.BasicTestCase,
@@ -97,7 +97,15 @@ class XMLFile (
     Test XML file based directory implementation.
     """
     def service(self):
-        return XMLDirectoryService({'xmlFile' : self.xmlFile()}, alwaysStat=True)
+        directory = XMLDirectoryService(
+            {
+                'xmlFile' : self.xmlFile(),
+                'augmentService' :
+                   augment.AugmentXMLDB(xmlFiles=(self.augmentsFile().path,)),
+            },
+            alwaysStat=True
+        )
+        return directory
 
     def test_changedXML(self):
         service = self.service()
@@ -130,7 +138,6 @@ class XMLFile (
                 set(expectedRecords)
             )
 
-    @inlineCallbacks
     def test_okAutoSchedule(self):
         service = self.service()
 
@@ -143,11 +150,25 @@ class XMLFile (
     <guid>myoffice</guid>
     <password>nimda</password>
     <name>Super User</name>
-    <auto-schedule/>
   </location>
 </accounts>
 """
         )
+        self.augmentsFile().open("w").write(
+"""<?xml version="1.0" encoding="utf-8"?>
+<!DOCTYPE accounts SYSTEM "accounts.dtd">
+<augments>
+  <record>
+    <uid>myoffice</uid>
+    <enable>true</enable>
+    <enable-calendar>true</enable-calendar>
+    <auto-schedule>true</auto-schedule>
+  </record>
+</augments>
+"""
+        )
+        service.augmentService.refresh()
+
         for recordType, expectedRecords in (
             ( DirectoryService.recordType_users     , ()             ),
             ( DirectoryService.recordType_groups    , ()             ),
@@ -162,8 +183,7 @@ class XMLFile (
                 set(r.shortNames[0] for r in service.listRecords(recordType)),
                 set(expectedRecords)
             )
-        resourceInfoDatabase = ResourceInfoDatabase(config.DataRoot)
-        self.assertTrue((yield resourceInfoDatabase.getAutoSchedule(service.recordWithShortName(DirectoryService.recordType_locations, "my office").guid)))
+        self.assertTrue(service.recordWithShortName(DirectoryService.recordType_locations, "my office").autoSchedule)
 
 
     def test_okDisableCalendar(self):
@@ -175,18 +195,20 @@ class XMLFile (
 <accounts realm="Test Realm">
   <group>
     <uid>enabled</uid>
+    <guid>enabled</guid>
     <password>enabled</password>
     <name>Enabled</name>
   </group>
   <group>
     <uid>disabled</uid>
+    <guid>disabled</guid>
     <password>disabled</password>
     <name>Disabled</name>
-    <disable-calendar/>
   </group>
 </accounts>
 """
         )
+        
         for recordType, expectedRecords in (
             ( DirectoryService.recordType_users     , ()                       ),
             ( DirectoryService.recordType_groups    , ("enabled", "disabled")  ),
@@ -206,50 +228,111 @@ class XMLFile (
         self.assertFalse(service.recordWithShortName(DirectoryService.recordType_groups, "enabled").enabledForCalendaring)
         self.assertFalse(service.recordWithShortName(DirectoryService.recordType_groups, "disabled").enabledForCalendaring)
 
-    @inlineCallbacks
-    def test_okProxies(self):
+
+    def test_readExtras(self):
         service = self.service()
 
         self.xmlFile().open("w").write(
 """<?xml version="1.0" encoding="utf-8"?>
 <!DOCTYPE accounts SYSTEM "accounts.dtd">
 <accounts realm="Test Realm">
-  <user>
-    <uid>test</uid>
-    <guid>test</guid>
-    <password>nimda</password>
-    <name>Test</name>
-  </user>
   <location>
     <uid>my office</uid>
     <guid>myoffice</guid>
-    <password>nimda</password>
-    <name>Super User</name>
-    <auto-schedule/>
-    <proxies>
-        <member>test</member>
-    </proxies>
+    <name>My Office</name>
+    <extras>
+        <comment>This is the comment</comment>
+        <capacity>40</capacity>
+    </extras>
   </location>
 </accounts>
 """
         )
-        for recordType, expectedRecords in (
-            ( DirectoryService.recordType_users     , ("test",)      ),
-            ( DirectoryService.recordType_groups    , ()             ),
-            ( DirectoryService.recordType_locations , ("my office",) ),
-            ( DirectoryService.recordType_resources , ()             ),
-        ):
-            # Fault records in
-            for name in expectedRecords:
-                service.recordWithShortName(recordType, name)
+        
+        record = service.recordWithShortName(
+            DirectoryService.recordType_locations, "my office")
+        self.assertEquals(record.guid, "myoffice")
+        self.assertEquals(record.extras["comment"], "This is the comment")
+        self.assertEquals(record.extras["capacity"], "40")
 
-            self.assertEquals(
-                set(r.shortNames[0] for r in service.listRecords(recordType)),
-                set(expectedRecords)
-            )
-        calendarUserProxyDatabase = CalendarUserProxyDatabase(config.DataRoot)
-        members = (yield calendarUserProxyDatabase.getMembers("myoffice#calendar-proxy-write"))
-        self.assertTrue("test" in members)
-        members = (yield calendarUserProxyDatabase.getMemberships("test"))
-        self.assertTrue("myoffice#calendar-proxy-write" in members)
+    def test_writeExtras(self):
+        service = self.service()
 
+        service.createRecord(DirectoryService.recordType_locations, "newguid",
+            shortNames=("New office",),
+            fullName="My New Office",
+            address="1 Infinite Loop, Cupertino, CA",
+            capacity="10",
+            comment="Test comment",
+        )
+
+        record = service.recordWithShortName(
+            DirectoryService.recordType_locations, "New office")
+        self.assertEquals(record.extras["comment"], "Test comment")
+        self.assertEquals(record.extras["capacity"], "10")
+
+
+        service.updateRecord(DirectoryService.recordType_locations, "newguid",
+            shortNames=("New office",),
+            fullName="My Newer Office",
+            address="2 Infinite Loop, Cupertino, CA",
+            capacity="20",
+            comment="Test comment updated",
+        )
+
+        record = service.recordWithShortName(
+            DirectoryService.recordType_locations, "New office")
+        self.assertEquals(record.fullName, "My Newer Office")
+        self.assertEquals(record.extras["address"], "2 Infinite Loop, Cupertino, CA")
+        self.assertEquals(record.extras["comment"], "Test comment updated")
+        self.assertEquals(record.extras["capacity"], "20")
+
+        service.destroyRecord(DirectoryService.recordType_locations, "newguid")
+
+        record = service.recordWithShortName(
+            DirectoryService.recordType_locations, "New office")
+        self.assertEquals(record, None)
+
+
+    def test_indexing(self):
+        service = self.service()
+        self.assertNotEquals(None, service._lookupInIndex(service.recordType_users, service.INDEX_TYPE_SHORTNAME, "usera"))
+        self.assertNotEquals(None, service._lookupInIndex(service.recordType_users, service.INDEX_TYPE_CUA, "mailto:wsanchez@example.com"))
+        self.assertNotEquals(None, service._lookupInIndex(service.recordType_users, service.INDEX_TYPE_GUID, "9FF60DAD-0BDE-4508-8C77-15F0CA5C8DD2"))
+        self.assertNotEquals(None, service._lookupInIndex(service.recordType_locations, service.INDEX_TYPE_SHORTNAME, "orion"))
+        self.assertEquals(None, service._lookupInIndex(service.recordType_users, service.INDEX_TYPE_CUA, "mailto:nobody@example.com"))
+
+    def test_repeat(self):
+        service = self.service()
+        record = service.recordWithShortName(
+            DirectoryService.recordType_users, "user01")
+        self.assertEquals(record.fullName, "c4ca4238a0b923820dcc509a6f75849bc4c User 01")
+        self.assertEquals(record.firstName, "c4ca4")
+        self.assertEquals(record.lastName, "c4ca4238a User 01")
+        self.assertEquals(record.emailAddresses, set(['c4ca4238a0@example.com']))
+
+class XMLFileSubset (XMLFileBase, TestCase):
+    """
+    Test the recordTypes subset feature of XMLFile service.
+    """
+    recordTypes = set((
+        DirectoryService.recordType_users,
+        DirectoryService.recordType_groups,
+    ))
+
+    def test_recordTypesSubset(self):
+        directory = XMLDirectoryService(
+            {
+                'xmlFile' : self.xmlFile(),
+                'augmentService' :
+                    augment.AugmentXMLDB(xmlFiles=(self.augmentsFile().path,)),
+                'recordTypes' :
+                    (
+                        DirectoryService.recordType_users,
+                        DirectoryService.recordType_groups
+                    ),
+            },
+            alwaysStat=True
+        )
+        self.assertEquals(set(("users", "groups")), set(directory.recordTypes()))
+    
