@@ -16,7 +16,7 @@
 
 from twisted.trial.unittest import SkipTest
 from twisted.cred.credentials import UsernamePassword
-from twisted.web2.auth.digest import DigestedCredentials, calcResponse, calcHA1
+from twext.web2.auth.digest import DigestedCredentials, calcResponse, calcHA1
 
 from twistedcaldav.directory.directory import DirectoryService
 from twistedcaldav.directory.directory import UnknownRecordTypeError
@@ -37,6 +37,9 @@ class DirectoryTestCase (TestCase):
     # Subclass should init this to a dict of groupname keys and dict values.
     groups = {}
 
+    # Subclass should init this to a dict of locationnames keys and dict values.
+    locations = {}
+
     # Subclass should init this to a dict of resourcenames keys and dict values.
     resources = {}
 
