@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2010 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,15 +25,14 @@ __all__ = [
 
 import xml.dom.minidom
 
-from twisted.python.filepath import FilePath
+from twext.python.filepath import CachingFilePath as FilePath
+
+from twext.python.log import Logger
 
-from twistedcaldav.config import config
 from twistedcaldav.directory.directory import DirectoryService
-from twistedcaldav.log import Logger
-from twistedcaldav.sql import DatabaseError
-from twistedcaldav.directory.directory import DirectoryError
-from twistedcaldav.directory.resourceinfo import ResourceInfoDatabase
-from twistedcaldav.directory.calendaruserproxy import CalendarUserProxyDatabase
+
+import re
+import hashlib
 
 log = Logger()
 
@@ -52,16 +51,15 @@ ELEMENT_LAST_NAME         = "last-name"
 ELEMENT_EMAIL_ADDRESS     = "email-address"
 ELEMENT_MEMBERS           = "members"
 ELEMENT_MEMBER            = "member"
-ELEMENT_CUADDR            = "cuaddr"
-ELEMENT_AUTOSCHEDULE      = "auto-schedule"
-ELEMENT_DISABLECALENDAR   = "disable-calendar"
-ELEMENT_PROXIES           = "proxies"
-ELEMENT_READ_ONLY_PROXIES = "read-only-proxies"
+ELEMENT_EXTRAS            = "extras"
 
 ATTRIBUTE_REALM           = "realm"
 ATTRIBUTE_REPEAT          = "repeat"
 ATTRIBUTE_RECORDTYPE      = "type"
 
+VALUE_TRUE                = "true"
+VALUE_FALSE               = "false"
+
 RECORD_TYPES = {
     ELEMENT_USER     : DirectoryService.recordType_users,
     ELEMENT_GROUP    : DirectoryService.recordType_groups,
@@ -76,7 +74,7 @@ class XMLAccountsParser(object):
     def __repr__(self):
         return "<%s %r>" % (self.__class__.__name__, self.xmlFile)
 
-    def __init__(self, xmlFile):
+    def __init__(self, xmlFile, externalUpdate=True):
 
         if type(xmlFile) is str:
             xmlFile = FilePath(xmlFile)
@@ -99,45 +97,6 @@ class XMLAccountsParser(object):
             log.error("Ignoring file %r because it is not a repository builder file" % (self.xmlFile,))
             return
         self._parseXML(accounts_node)
-        try:
-            self._updateExternalDatabases()
-        except DatabaseError, e:
-            raise DirectoryError(e)
-
-    def _updateExternalDatabases(self):
-        resourceInfoDatabase = ResourceInfoDatabase(config.DataRoot)
-
-        calendarUserProxyDatabase = CalendarUserProxyDatabase(config.DataRoot)
-
-        for records in self.items.itervalues():
-            for principal in records.itervalues():
-
-                resourceInfoDatabase.setAutoScheduleInDatabase(principal.guid,
-                    principal.autoSchedule)
-
-                if principal.proxies:
-                    proxies = []
-                    for recordType, uid in principal.proxies:
-                        record = self.items[recordType].get(uid)
-                        if record is not None:
-                            proxies.append(record.guid)
-
-                    calendarUserProxyDatabase.setGroupMembersInDatabase(
-                        "%s#calendar-proxy-write" % (principal.guid,),
-                        proxies
-                    )
-
-                if principal.readOnlyProxies:
-                    readOnlyProxies = []
-                    for recordType, uid in principal.readOnlyProxies:
-                        record = self.items[recordType].get(uid)
-                        if record is not None:
-                            readOnlyProxies.append(record.guid)
-
-                    calendarUserProxyDatabase.setGroupMembersInDatabase(
-                        "%s#calendar-proxy-read" % (principal.guid,),
-                        readOnlyProxies
-                    )
 
     def _parseXML(self, node):
         """
@@ -154,19 +113,6 @@ class XMLAccountsParser(object):
                 if item is not None:
                     item.groups.add(group.shortNames[0])
 
-        def updateProxyFor(proxier):
-            # Update proxy membership
-            for recordType, shortName in proxier.proxies:
-                item = self.items[recordType].get(shortName)
-                if item is not None:
-                    item.proxyFor.add((proxier.recordType, proxier.shortNames[0]))
-
-            # Update read-only proxy membership
-            for recordType, shortName in proxier.readOnlyProxies:
-                item = self.items[recordType].get(shortName)
-                if item is not None:
-                    item.readOnlyProxyFor.add((proxier.recordType, proxier.shortNames[0]))
-
         for child in node._get_childNodes():
             child_name = child._get_localName()
             if child_name is None:
@@ -195,7 +141,6 @@ class XMLAccountsParser(object):
         for records in self.items.itervalues():
             for principal in records.itervalues():
                 updateMembership(principal)
-                updateProxyFor(principal)
                 
 class XMLAccountRecord (object):
     """
@@ -215,16 +160,7 @@ class XMLAccountRecord (object):
         self.emailAddresses = set()
         self.members = set()
         self.groups = set()
-        self.calendarUserAddresses = set()
-        self.autoSchedule = False
-        if recordType == DirectoryService.recordType_groups:
-            self.enabledForCalendaring = False
-        else:
-            self.enabledForCalendaring = True
-        self.proxies = set()
-        self.proxyFor = set()
-        self.readOnlyProxies = set()
-        self.readOnlyProxyFor = set()
+        self.extras = {}
 
     def repeat(self, ctr):
         """
@@ -232,6 +168,36 @@ class XMLAccountRecord (object):
         done on them with the numeric value provided.
         @param ctr: an integer to substitute into text.
         """
+
+        # Regular expression which matches ~ followed by a number
+        matchNumber = re.compile(r"(~\d+)")
+
+        def expand(text, ctr):
+            """
+            Returns a string where ~<number> is replaced by the first <number>
+            characters from the md5 hexdigest of str(ctr), e.g.:
+
+                expand("~9 foo", 1)
+
+            returns:
+
+                "c4ca4238a foo"
+
+            ...since "c4ca4238a" is the first 9 characters of:
+
+                hashlib.md5(str(1)).hexdigest()
+
+            If <number> is larger than 32, the hash will repeat as needed.
+            """
+            if text:
+                m = matchNumber.search(text)
+                if m:
+                    length = int(m.group(0)[1:])
+                    hash = hashlib.md5(str(ctr)).hexdigest()
+                    string = (hash*((length/32)+1))[:-(32-(length%32))]
+                    return text.replace(m.group(0), string)
+            return text
+
         shortNames = []
         for shortName in self.shortNames:
             if shortName.find("%") != -1:
@@ -250,26 +216,24 @@ class XMLAccountRecord (object):
             fullName = self.fullName % ctr
         else:
             fullName = self.fullName
+        fullName = expand(fullName, ctr)
         if self.firstName and self.firstName.find("%") != -1:
             firstName = self.firstName % ctr
         else:
             firstName = self.firstName
+        firstName = expand(firstName, ctr)
         if self.lastName and self.lastName.find("%") != -1:
             lastName = self.lastName % ctr
         else:
             lastName = self.lastName
+        lastName = expand(lastName, ctr)
         emailAddresses = set()
         for emailAddr in self.emailAddresses:
+            emailAddr = expand(emailAddr, ctr)
             if emailAddr.find("%") != -1:
                 emailAddresses.add(emailAddr % ctr)
             else:
                 emailAddresses.add(emailAddr)
-        calendarUserAddresses = set()
-        for cuaddr in self.calendarUserAddresses:
-            if cuaddr.find("%") != -1:
-                calendarUserAddresses.add(cuaddr % ctr)
-            else:
-                calendarUserAddresses.add(cuaddr)
         
         result = XMLAccountRecord(self.recordType)
         result.shortNames = shortNames
@@ -280,11 +244,7 @@ class XMLAccountRecord (object):
         result.lastName = lastName
         result.emailAddresses = emailAddresses
         result.members = self.members
-        result.calendarUserAddresses = calendarUserAddresses
-        result.autoSchedule = self.autoSchedule
-        result.enabledForCalendaring = self.enabledForCalendaring
-        result.proxies = self.proxies
-        result.readOnlyProxies = self.readOnlyProxies
+        result.extras = self.extras
         return result
 
     def parseXML(self, node):
@@ -317,27 +277,13 @@ class XMLAccountRecord (object):
                     self.emailAddresses.add(child.firstChild.data.encode("utf-8").lower())
             elif child_name == ELEMENT_MEMBERS:
                 self._parseMembers(child, self.members)
-            elif child_name == ELEMENT_CUADDR:
-                if child.firstChild is not None:
-                    self.calendarUserAddresses.add(child.firstChild.data.encode("utf-8"))
-            elif child_name == ELEMENT_AUTOSCHEDULE:
-                self.autoSchedule = True
-            elif child_name == ELEMENT_DISABLECALENDAR:
-                # FIXME: Not sure I see why this restriction is needed. --wsanchez
-                ## Only Users or Groups
-                #if self.recordType != DirectoryService.recordType_users:
-                #    raise ValueError("<disable-calendar> element only allowed for Users: %s" % (child_name,))
-                self.enabledForCalendaring = False
-            elif child_name == ELEMENT_PROXIES:
-                self._parseMembers(child, self.proxies)
-            elif child_name == ELEMENT_READ_ONLY_PROXIES:
-                self._parseMembers(child, self.readOnlyProxies)
+            elif child_name == ELEMENT_EXTRAS:
+                self._parseExtras(child, self.extras)
             else:
                 raise RuntimeError("Unknown account attribute: %s" % (child_name,))
 
-        if self.enabledForCalendaring:
-            for email in self.emailAddresses:
-                self.calendarUserAddresses.add("mailto:%s" % (email,))
+        if not self.shortNames:
+            self.shortNames.append(self.guid)
 
     def _parseMembers(self, node, addto):
         for child in node._get_childNodes():
@@ -348,3 +294,10 @@ class XMLAccountRecord (object):
                     recordType = DirectoryService.recordType_users
                 if child.firstChild is not None:
                     addto.add((recordType, child.firstChild.data.encode("utf-8")))
+
+    def _parseExtras(self, node, addto):
+        for child in node._get_childNodes():
+            key = child._get_localName()
+            if key:
+                value = child.firstChild.data.encode("utf-8")
+                addto[key.encode("utf-8")] = value
