@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2007 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2010 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,16 +21,19 @@ Implements drop-box functionality. A drop box is an external attachment store.
 __all__ = [
     "DropBoxHomeResource",
     "DropBoxCollectionResource",
-    "DropBoxChildResource",
 ]
 
-from twext.web2.dav.davxml import ErrorResponse
-from twisted.web2 import responsecode
-from twisted.web2.dav import davxml
-from twisted.web2.dav.resource import DAVResource, TwistedACLInheritable
+from twext.python.log import Logger
+from twext.web2 import responsecode
+from twext.web2.dav import davxml
+from twext.web2.dav.http import ErrorResponse
+from twext.web2.dav.resource import DAVResource, TwistedACLInheritable
+from twext.web2.dav.util import joinURL
 
+from twisted.internet.defer import inlineCallbacks, returnValue
+
+from twistedcaldav.config import config
 from twistedcaldav.customxml import calendarserver_namespace
-from twistedcaldav.log import Logger
 
 log = Logger()
 
@@ -39,7 +42,7 @@ class DropBoxHomeResource (DAVResource):
     Drop box collection resource.
     """
     def resourceType(self):
-        return davxml.ResourceType.dropboxhome
+        return davxml.ResourceType.dropboxhome #@UndefinedVariable
 
     def isCollection(self):
         return True
@@ -47,16 +50,48 @@ class DropBoxHomeResource (DAVResource):
     def http_PUT(self, request):
         return responsecode.FORBIDDEN
 
+    @inlineCallbacks
+    def accessControlList(self, request, *args, **kwargs):
+        """
+        Override this to give write proxies DAV:write-acl privilege so they can add attachments too.
+        """
+
+        acl = (yield super(DropBoxHomeResource, self).accessControlList(request, *args, **kwargs))
+        
+        if config.EnableProxyPrincipals:
+            owner = (yield self.ownerPrincipal(request))
+
+            newaces = tuple(acl.children)
+            newaces += (
+                # DAV:write-acl access for this principal's calendar-proxy-write users.
+                davxml.ACE(
+                    davxml.Principal(davxml.HRef(joinURL(owner.principalURL(), "calendar-proxy-write/"))),
+                    davxml.Grant(
+                        davxml.Privilege(davxml.WriteACL()),
+                    ),
+                    davxml.Protected(),
+                    TwistedACLInheritable(),
+                ),
+            )
+
+            returnValue(davxml.ACL(*newaces))
+        
+        else:
+            returnValue(acl)
+
 class DropBoxCollectionResource (DAVResource):
     """
     Drop box resource.
     """
+
     def resourceType(self):
-        return davxml.ResourceType.dropbox
+        return davxml.ResourceType.dropbox #@UndefinedVariable
+
 
     def isCollection(self):
         return True
 
+
     def writeNewACEs(self, newaces):
         """
         Write a new ACL to the resource's property store. We override this for calendar collections
@@ -76,12 +111,14 @@ class DropBoxCollectionResource (DAVResource):
                 edited_aces.append(davxml.ACE(*children))
             else:
                 edited_aces.append(ace)
-        
+
         # Do inherited with possibly modified set of aces
-        super(DropBoxCollectionResource, self).writeNewACEs(edited_aces)
+        return super(DropBoxCollectionResource, self).writeNewACEs(edited_aces)
+
 
     def http_PUT(self, request):
         return ErrorResponse(
             responsecode.FORBIDDEN,
-            (calendarserver_namespace, "valid-drop-box")
+            (calendarserver_namespace, "valid-drop-box"),
+            "Cannot store resources in dropbox",
         )
