@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2011 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,20 +18,18 @@
 iCalendar Recurrence Expansion Utilities
 """
 
-import datetime
+from twistedcaldav.config import config
+from twistedcaldav.dateops import normalizeForIndex, differenceDateTime
 
-from twistedcaldav.dateops import normalizeForIndex, compareDateTime, differenceDateTime, periodEnd
-
-from vobject.icalendar import utc
-
-# The maximum number of instances we will expand out to.
-# Raise a TooManyInstancesError exception if we exceed this.
-max_allowed_instances = 1000
+from pycalendar.datetime import PyCalendarDateTime
+from pycalendar.duration import PyCalendarDuration
+from pycalendar.period import PyCalendarPeriod
+from pycalendar.timezone import PyCalendarTimezone
 
 class TooManyInstancesError(Exception):
     def __init__(self):
         Exception.__init__(self)
-        self.max_allowed = max_allowed_instances
+        self.max_allowed = config.MaxAllowedInstances
 
     def __repr__(self):
         return "<%s max:%s>" % (self.__class__.__name__, self.max_allowed)
@@ -46,22 +44,11 @@ class InvalidOverriddenInstanceError(Exception):
 
 class Instance(object):
     
-    __slots__ = ["component", "start", "end", "rid", "overridden", "future"]
-    
     def __init__(self, component, start = None, end = None, rid = None, overridden = False, future = False):
         self.component = component
-        if start is None:
-            self.start = component.getStartDateUTC()
-        else:
-            self.start = start
-        if end is None:
-            self.end = component.getEndDateUTC()
-        else:
-            self.end = end
-        if rid is None:
-            self.rid = self.start
-        else:
-            self.rid = rid
+        self.start = component.getStartDateUTC() if start is None else start
+        self.end = component.getEndDateUTC() if end is None else end
+        self.rid = self.start if rid is None else rid
         self.overridden = overridden
         self.future = future
         
@@ -76,28 +63,28 @@ class Instance(object):
             (trigger, related, repeat, duration)  = alarm.getTriggerDetails()
             
             # Handle relative vs absolute triggers
-            if isinstance(trigger, datetime.date):
+            if isinstance(trigger, PyCalendarDateTime):
                 # Absolute trigger
                 start = trigger
             else:
                 # Relative trigger
-                if related:
-                    start = self.start + trigger
-                else:
-                    start = self.end + trigger
+                start = (self.start if related else self.end) + trigger
             triggers.add(start)
             
             # Handle repeats
             if repeat > 0:
-                for i in xrange(1, repeat+1):
-                    triggers.add(start + (duration * i))
+                tstart = start.duplicate()
+                for _ignore in xrange(1, repeat+1):
+                    tstart += duration
+                    triggers.add(tstart)
         
         return triggers
     
+    def isMasterInstance(self):
+        return not self.overridden and self.start == self.component.getStartDateUTC()
+
 class InstanceList(object):
     
-    __slots__ = ["instances", "limit", "ignoreInvalidInstances",]
-    
     def __init__(self, ignoreInvalidInstances=False):
         self.instances = {}
         self.limit = None
@@ -120,7 +107,7 @@ class InstanceList(object):
         @param componentSet: the set of components that are to make up the
                 recurrence set. These MUST all be components with the same UID
                 and type, forming a proper recurring set.
-        @param limit: datetime.date value representing the end of the expansion.
+        @param limit: L{PyCalendarDateTime} value representing the end of the expansion.
         """
         
         # Look at each component type
@@ -175,34 +162,48 @@ class InstanceList(object):
         self.instances[str(instance.rid)] = instance
         
         # Check for too many instances
-        if len(self.instances) > max_allowed_instances:
+        if config.MaxAllowedInstances and len(self.instances) > config.MaxAllowedInstances:
             raise TooManyInstancesError()
 
+    def _getMasterEventDetails(self, component):
+        """
+        Logic here comes from RFC4791 Section 9.9
+        """
+
+        start = component.getStartDateUTC()
+        if start is None:
+            return None
+        rulestart = component.propertyValue("DTSTART")
+
+        end = component.getEndDateUTC()
+        duration = None
+        if end is None:
+            if not start.isDateOnly():
+                # Timed event with zero duration
+                duration = PyCalendarDuration(days=0)
+            else:
+                # All day event default duration is one day
+                duration = PyCalendarDuration(days=1)
+            end = start + duration
+        else:
+            duration = differenceDateTime(start, end)
+        
+        return (rulestart, start, end, duration,)
+
     def _addMasterEventComponent(self, component, limit):
         """
         Add the specified master VEVENT Component to the instance list, expanding it
         within the supplied time range.
         @param component: the Component to expand
-        @param limit: the end datetime.datetime for expansion
+        @param limit: the end L{PyCalendarDateTime} for expansion
         """
-        start = component.getStartDateUTC()
-        if start is None:
+        
+        details = self._getMasterEventDetails(component)
+        if details is None:
             return
+        rulestart, start, end, duration = details
 
-        end = component.getEndDateUTC()
-        duration = None
-        if end is None:
-            if isinstance(start, datetime.datetime):
-                # Timed event with zero duration
-                duration = datetime.timedelta(days=0)
-            else:
-                # All day event default duration is one day
-                duration = datetime.timedelta(days=1)
-            end = start + duration
-        else:
-            duration = differenceDateTime(start, end)
-
-        self._addMasterComponent(component, limit, start, end, duration)
+        self._addMasterComponent(component, limit, rulestart, start, end, duration)
 
     def _addOverrideEventComponent(self, component, limit, got_master):
         """
@@ -214,45 +215,71 @@ class InstanceList(object):
         
         #TODO: This does not take into account THISANDPRIOR - only THISANDFUTURE
         
-        start = component.getStartDateUTC()
-        if start is None:
+        details = self._getMasterEventDetails(component)
+        if details is None:
             return
-
-        end = component.getEndDateUTC()
-        duration = None
-        if end is None:
-            if isinstance(start, datetime.datetime):
-                # Timed event with zero duration
-                duration = datetime.timedelta(days=0)
-            else:
-                # All day event default duration is one day
-                duration = datetime.timedelta(days=1)
-            end = start + duration
-        else:
-            duration = differenceDateTime(start, end)
+        _ignore_rulestart, start, end, _ignore_duration = details
 
         self._addOverrideComponent(component, limit, start, end, got_master)
 
+    def _getMasterToDoDetails(self, component):
+        """
+        Logic here comes from RFC4791 Section 9.9
+        """
+
+        dtstart = component.getStartDateUTC()
+        dtend = component.getEndDateUTC()
+        dtdue = component.getDueDateUTC()
+
+        # DTSTART and DURATION or DUE case
+        if dtstart is not None:
+            rulestart = component.propertyValue("DTSTART")
+            start = dtstart
+            if dtend is not None:
+                end = dtend
+            elif dtdue is not None:
+                end = dtdue
+            else:
+                end = dtstart
+        
+        # DUE case
+        elif dtdue is not None:
+            rulestart = component.propertyValue("DUE")
+            start = end = dtdue
+        
+        # Fall back to COMPLETED or CREATED - cannot be recurring
+        else:
+            rulestart = None
+            from twistedcaldav.ical import maxDateTime, minDateTime
+            dtcreated = component.getCreatedDateUTC()
+            dtcompleted = component.getCompletedDateUTC()
+            if dtcompleted:
+                end = dtcompleted
+                start = dtcreated if dtcreated else end
+            elif dtcreated:
+                start = dtcreated
+                end = maxDateTime
+            else:
+                start = minDateTime
+                end = maxDateTime
+
+        duration = differenceDateTime(start, end)
+
+        return (rulestart, start, end, duration,)
+
     def _addMasterToDoComponent(self, component, limit):
         """
         Add the specified master VTODO Component to the instance list, expanding it
         within the supplied time range.
         @param component: the Component to expand
-        @param limit: the end datetime.datetime for expansion
+        @param limit: the end L{PyCalendarDateTime} for expansion
         """
-        start = component.getStartDateUTC()
-        due = component.getDueDateUTC()
-
-        if start is None and due is None:
+        details = self._getMasterToDoDetails(component)
+        if details is None:
             return
+        rulestart, start, end, duration = details
 
-        if start is None:
-            start = due
-        elif due is None:
-            due = start
-        duration = differenceDateTime(start, due)
-
-        self._addMasterComponent(component, limit, start, due, duration)
+        self._addMasterComponent(component, limit, rulestart, start, end, duration)
 
     def _addOverrideToDoComponent(self, component, limit, got_master):
         """
@@ -264,48 +291,34 @@ class InstanceList(object):
         
         #TODO: This does not take into account THISANDPRIOR - only THISANDFUTURE
         
-        start = component.getStartDateUTC()
-        due = component.getDueDateUTC()
-
-        if start is None and due is None:
+        details = self._getMasterToDoDetails(component)
+        if details is None:
             return
+        _ignore_rulestart, start, end, _ignore_duration = details
 
-        if start is None:
-            start = due
-        elif due is None:
-            due = start
+        self._addOverrideComponent(component, limit, start, end, got_master)
 
-        self._addOverrideComponent(component, limit, start, due, got_master)
-
-    def _addMasterComponent(self, component, limit, start, end, duration):
-        # Always add first instance if included in range.
-        if compareDateTime(start, limit) < 0:
-            # dateutils does not do all-day - so convert to datetime.datetime
-            start = normalizeForIndex(start)
-            end = normalizeForIndex(end)
-            
-            # Do not add if in EXDATEs
-            exdates = []
-            for prop in component.properties("EXDATE"):
-                exdates.extend(prop.value())
-            exdates = [normalizeForIndex(exdate) for exdate in exdates]
-            if start not in exdates:
-                self.addInstance(Instance(component, start, end))
-        else:
-            self.limit = limit
+    def _addMasterComponent(self, component, limit, rulestart, start, end, duration):
         
-        # Now expand recurrence
-        # FIXME: Current Python implementation fails when RDATEs are PERIODs
-        recur = component.getRRuleSet(True)
-        if recur is not None:
-            for startDate in recur:
-                if compareDateTime(startDate, limit) >= 0:
-                    self.limit = limit
-                    break
-                endDate = startDate + duration
+        rrules = component.getRecurrenceSet()
+        if rrules is not None and rulestart is not None:
+            # Do recurrence set expansion
+            expanded = []
+            limited = rrules.expand(rulestart, PyCalendarPeriod(start, limit), expanded)
+            for startDate in expanded:
                 startDate = normalizeForIndex(startDate)
-                endDate = normalizeForIndex(endDate)
+                endDate = startDate + duration
                 self.addInstance(Instance(component, startDate, endDate))
+            if limited:
+                self.limit = limit
+        else:
+            # Always add main instance if included in range.
+            if start < limit:
+                start = normalizeForIndex(start)
+                end = normalizeForIndex(end)
+                self.addInstance(Instance(component, start, end))
+            else:
+                self.limit = limit
     
     def _addOverrideComponent(self, component, limit, start, end, got_master):
 
@@ -319,12 +332,12 @@ class InstanceList(object):
         rid = normalizeForIndex(rid)
 
         # Make sure start is within the limit
-        if compareDateTime(start, limit) > 0 and compareDateTime(rid, limit) > 0:
+        if start > limit and rid > limit:
             return
 
         # Make sure override RECURRENCE-ID is a valid instance of the master
         if got_master:
-            if str(rid) not in self.instances and compareDateTime(rid, limit) <= 0:
+            if str(rid) not in self.instances and rid < limit:
                 if self.ignoreInvalidInstances:
                     return
                 else:
@@ -369,11 +382,11 @@ class InstanceList(object):
         Add the specified master VFREEBUSY Component to the instance list, expanding it
         within the supplied time range.
         @param component: the Component to expand
-        @param limit: the end datetime.datetime for expansion
+        @param limit: the end L{PyCalendarDateTime} for expansion
         """
 
         start = component.getStartDateUTC()
-        if start is not None and (compareDateTime(start, limit) >= 0):
+        if start is not None and start >= limit:
             # If the free busy is beyond the end of the range we want, ignore it
             return
 
@@ -387,10 +400,11 @@ class InstanceList(object):
             assert isinstance(fb.value(), list), "FREEBUSY property does not contain a list of values: %r" % (fb,)
             for period in fb.value():
                 # Ignore if period starts after limit
-                if compareDateTime(period[0], limit) >= 0:
+                period = period.getValue()
+                if period.getStart() >= limit:
                     continue
-                start = normalizeForIndex(period[0])
-                end = normalizeForIndex(periodEnd(period))
+                start = normalizeForIndex(period.getStart())
+                end = normalizeForIndex(period.getEnd())
                 self.addInstance(Instance(component, start, end))
 
     def _addAvailabilityComponent(self, component, limit):
@@ -401,20 +415,20 @@ class InstanceList(object):
         depending on the presence of the properties. If unbounded at one or both ends, we will
         set the time to 1/1/1900 in the past and 1/1/3000 in the future.
         @param component: the Component to expand
-        @param limit: the end datetime.datetime for expansion
+        @param limit: the end L{PyCalendarDateTime} for expansion
         """
 
         start = component.getStartDateUTC()
-        if start is not None and (compareDateTime(start, limit) >= 0):
+        if start is not None and start >= limit:
             # If the free busy is beyond the end of the range we want, ignore it
             return
         if start is None:
-            start = datetime.datetime(1900, 1, 1, 0, 0, 0, tzinfo=utc)
+            start = PyCalendarDateTime(1900, 1, 1, 0, 0, 0, tzid=PyCalendarTimezone(utc=True))
         start = normalizeForIndex(start)
 
         end = component.getEndDateUTC()
         if end is None:
-            end = datetime.datetime(3000, 1, 1, 0, 0, 0, tzinfo=utc)
+            end = PyCalendarDateTime(2100, 1, 1, 0, 0, 0, tzid=PyCalendarTimezone(utc=True))
         end = normalizeForIndex(end)
 
         self.addInstance(Instance(component, start, end))
