@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2007 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,20 +17,25 @@
 """
 CalDAV methods.
 
-Modules in this package are imported by twistedcaldav.static in order to
-bind methods to CalDAVFile.
+Modules in this package are imported by twistedcaldav.resource in order to
+bind methods to CalDAVResource.
 """
 
 __all__ = [
+    "acl",
     "copymove",
     "delete",
     "get",
     "mkcalendar",
     "mkcol",
+    "post",
     "propfind",
     "put",
     "report",
-    "report_calquery",
     "report_freebusy",
-    "report_multiget",
+    "report_calendar_multiget",
+    "report_calendar_query",
+    "report_addressbook_multiget",
+    "report_addressbook_query",
+    "report_sync_collection",
 ]
