@@ -23,18 +23,24 @@ __all__ = ["http_COPY", "http_MOVE"]
 from urlparse import urlsplit
 
 from twisted.internet.defer import inlineCallbacks, returnValue
-from twisted.web2 import responsecode
-from twisted.web2.filter.location import addLocation
-from twisted.web2.dav import davxml
-from twext.web2.dav.davxml import ErrorResponse
-from twisted.web2.dav.util import parentForURL
-from twisted.web2.http import StatusResponse, HTTPError
+from twext.web2 import responsecode
+from twext.web2.filter.location import addLocation
+from twext.web2.dav import davxml
+from twext.web2.dav.http import ErrorResponse
+from twext.web2.dav.util import parentForURL
+from twext.web2.http import StatusResponse, HTTPError
+
+from twext.python.log import Logger
 
 from twistedcaldav.caldavxml import caldav_namespace
 from twistedcaldav.method.put_common import StoreCalendarObjectResource
+from twistedcaldav.method.copymove_contact import (
+    maybeCOPYContact, maybeMOVEContact, KEEP_GOING
+)
+
 from twistedcaldav.resource import isCalendarCollectionResource,\
-    isPseudoCalendarCollectionResource
-from twistedcaldav.log import Logger
+    isPseudoCalendarCollectionResource, CalDAVResource,\
+    isAddressBookCollectionResource
 
 log = Logger()
 
@@ -53,8 +59,10 @@ def http_COPY(self, request):
 
     result, sourcecal, sourceparent, destination_uri, destination, destinationcal, destinationparent = (yield checkForCalendarAction(self, request))
     if not result or not destinationcal:
-        # Do default WebDAV action
-        result = (yield super(CalDAVFile, self).http_COPY(request))
+        # Check with CardDAV first (XXX might want to check EnableCardDAV switch?)
+        result = yield maybeCOPYContact(self, request)
+        if result is KEEP_GOING:
+            result = yield super(CalDAVResource, self).http_COPY(request)
         returnValue(result)
 
     #
@@ -71,8 +79,8 @@ def http_COPY(self, request):
     # Check for existing destination resource
     overwrite = request.headers.getHeader("overwrite", True)
     if destination.exists() and not overwrite:
-        log.err("Attempt to copy onto existing file without overwrite flag enabled: %s"
-                % (destination.fp.path,))
+        log.err("Attempt to copy onto existing resource without overwrite flag enabled: %s"
+                % (destination,))
         raise HTTPError(StatusResponse(
             responsecode.PRECONDITION_FAILED,
             "Destination %s already exists." % (destination_uri,))
@@ -81,7 +89,11 @@ def http_COPY(self, request):
     # Checks for copying a calendar collection
     if self.isCalendarCollection():
         log.err("Attempt to copy a calendar collection into another calendar collection %s" % destination)
-        raise HTTPError(ErrorResponse(responsecode.FORBIDDEN, (caldav_namespace, "calendar-collection-location-ok")))
+        raise HTTPError(ErrorResponse(
+            responsecode.FORBIDDEN,
+            (caldav_namespace, "calendar-collection-location-ok"),
+            "Cannot copy calendar collection inside another calendar collection",
+        ))
 
     # We also do not allow regular collections in calendar collections
     if self.isCollection():
@@ -120,13 +132,24 @@ def http_MOVE(self, request):
     if not result:
         is_calendar_collection = isPseudoCalendarCollectionResource(self)
         defaultCalendar = (yield self.isDefaultCalendar(request)) if is_calendar_collection else False
+        is_addressbook_collection = isAddressBookCollectionResource(self)
+        defaultAddressBook = (yield self.isDefaultAddressBook(request)) if is_addressbook_collection else False
+
+        if not is_calendar_collection:
+            result = yield maybeMOVEContact(self, request)
+            if result is not KEEP_GOING:
+                returnValue(result)
 
         # Do default WebDAV action
-        result = (yield super(CalDAVFile, self).http_MOVE(request))
+        result = (yield super(CalDAVResource, self).http_MOVE(request))
         
-        if is_calendar_collection:
-            # Do some clean up
-            yield self.movedCalendar(request, defaultCalendar, destination, destination_uri)
+        if result == responsecode.NO_CONTENT:
+            if is_calendar_collection:
+                # Do some clean up
+                yield self.movedCalendar(request, defaultCalendar, destination, destination_uri)
+            elif is_addressbook_collection:
+                # Do some clean up
+                yield self.movedAddressBook(request, defaultAddressBook, destination, destination_uri)
 
         returnValue(result)
         
@@ -145,8 +168,8 @@ def http_MOVE(self, request):
     # Check for existing destination resource
     overwrite = request.headers.getHeader("overwrite", True)
     if destination.exists() and not overwrite:
-        log.err("Attempt to copy onto existing file without overwrite flag enabled: %s"
-                % (destination.fp.path,))
+        log.err("Attempt to copy onto existing resource without overwrite flag enabled: %s"
+                % (destination,))
         raise HTTPError(StatusResponse(
             responsecode.PRECONDITION_FAILED,
             "Destination %s already exists." % (destination_uri,)
@@ -156,7 +179,11 @@ def http_MOVE(self, request):
         # Checks for copying a calendar collection
         if self.isCalendarCollection():
             log.err("Attempt to move a calendar collection into another calendar collection %s" % destination)
-            raise HTTPError(ErrorResponse(responsecode.FORBIDDEN, (caldav_namespace, "calendar-collection-location-ok")))
+            raise HTTPError(ErrorResponse(
+                responsecode.FORBIDDEN,
+                (caldav_namespace, "calendar-collection-location-ok"),
+                "Cannot move calendar collection inside another calendar collection",
+            ))
     
         # We also do not allow regular collections in calendar collections
         if self.isCollection():
@@ -197,7 +224,7 @@ def checkForCalendarAction(self, request):
         sourcecal:        True if source is in a calendar collection, False otherwise
         sourceparent:     The parent resource for the source
         destination_uri:  The URI of the destination resource
-        destination:      CalDAVFile of destination if special proccesing required,
+        destination:      CalDAVResource of destination if special processing required,
         None otherwise
         destinationcal:   True if the destination is in a calendar collection,
             False otherwise
@@ -210,8 +237,8 @@ def checkForCalendarAction(self, request):
     destinationcal = False
     
     # Check the source path first
-    if not self.fp.exists():
-        log.err("File not found: %s" % (self.fp.path,))
+    if not self.exists():
+        log.err("Resource not found: %s" % (self,))
         raise HTTPError(StatusResponse(
             responsecode.NOT_FOUND,
             "Source resource %s not found." % (request.uri,)
