@@ -20,14 +20,15 @@ CalDAV DELETE method.
 
 __all__ = ["http_DELETE"]
 
+from twext.python.log import Logger
+
 from twisted.internet.defer import inlineCallbacks, returnValue
-from twisted.web2 import responsecode
-from twisted.web2.dav import davxml
-from twisted.web2.dav.util import parentForURL
-from twisted.web2.http import HTTPError
+from twext.web2 import responsecode
+from twext.web2.dav import davxml
+from twext.web2.dav.util import parentForURL
+from twext.web2.http import HTTPError
 
 from twistedcaldav.method.delete_common import DeleteResource
-from twistedcaldav.log import Logger
 
 log = Logger()
 
@@ -38,8 +39,8 @@ def http_DELETE(self, request):
     # index file has the entry for the deleted calendar component removed.
     #
 
-    if not self.fp.exists():
-        log.err("File not found: %s" % (self.fp.path,))
+    if not self.exists():
+        log.err("Resource not found: %s" % (self,))
         raise HTTPError(responsecode.NOT_FOUND)
 
     depth = request.headers.getHeader("depth", "infinity")
