@@ -20,19 +20,19 @@ CalDAV MKCALENDAR method.
 
 __all__ = ["http_MKCALENDAR"]
 
-from twext.web2.dav.davxml import ErrorResponse
-
 from twisted.internet.defer import inlineCallbacks, returnValue
 from twisted.python.failure import Failure
-from twisted.web2 import responsecode
-from twisted.web2.dav import davxml
-from twisted.web2.dav.http import MultiStatusResponse, PropertyStatusResponseQueue
-from twisted.web2.dav.util import davXMLFromStream
-from twisted.web2.dav.util import parentForURL
-from twisted.web2.http import HTTPError, StatusResponse
+from twext.web2 import responsecode
+from twext.web2.dav import davxml
+from twext.web2.dav.http import MultiStatusResponse, PropertyStatusResponseQueue
+from twext.web2.dav.util import davXMLFromStream
+from twext.web2.dav.util import parentForURL
+from twext.web2.http import HTTPError, StatusResponse
+
+from twext.python.log import Logger
+from twext.web2.dav.http import ErrorResponse
 
 from twistedcaldav import caldavxml
-from twistedcaldav.log import Logger
 
 log = Logger()
 
@@ -50,20 +50,21 @@ def http_MKCALENDAR(self, request):
     yield parent.authorize(request, (davxml.Bind(),))
 
     if self.exists():
-        log.err("Attempt to create collection where file exists: %s"
-                % (self.fp.path,))
+        log.err("Attempt to create collection where resource exists: %s"  % (self,))
         raise HTTPError(ErrorResponse(
             responsecode.FORBIDDEN,
-            (davxml.dav_namespace, "resource-must-be-null"))
-        )
+            (davxml.dav_namespace, "resource-must-be-null"),
+            "Resource already exists",
+        ))
 
     if not parent.isCollection():
         log.err("Attempt to create collection with non-collection parent: %s"
-                % (self.fp.path,))
+                % (self,))
         raise HTTPError(ErrorResponse(
             responsecode.CONFLICT,
-            (caldavxml.caldav_namespace, "calendar-collection-location-ok"))
-        )
+            (caldavxml.caldav_namespace, "calendar-collection-location-ok"),
+            "Cannot create calendar inside another calendar",
+        ))
 
     #
     # Read request body
@@ -78,9 +79,6 @@ def http_MKCALENDAR(self, request):
     if doc is not None:
         makecalendar = doc.root_element
         if not isinstance(makecalendar, caldavxml.MakeCalendar):
-            # Clean up
-            if self.fp.exists(): self.fp.remove()
-
             error = ("Non-%s element in MKCALENDAR request body: %s"
                      % (caldavxml.MakeCalendar.name, makecalendar))
             log.err(error)
@@ -104,9 +102,8 @@ def http_MKCALENDAR(self, request):
                     errors.add(responsecode.OK, property)
     
         if got_an_error:
-            # Clean up
-            if self.fp.exists(): self.fp.remove()
-
+            # Force a transaction error and proper clean-up
+            self.transactionError()
             errors.error()
             raise HTTPError(MultiStatusResponse([errors.response()]))
 
