@@ -28,17 +28,17 @@ __all__ = ["http_REPORT"]
 
 import string
 
-from twext.web2.dav.davxml import ErrorResponse
-
 from twisted.internet.defer import inlineCallbacks, returnValue
-from twisted.web2 import responsecode
-from twisted.web2.http import HTTPError, StatusResponse
-from twisted.web2.dav import davxml
-from twisted.web2.dav.element.parser import lookupElement
-from twisted.web2.dav.util import davXMLFromStream
+from twext.web2 import responsecode
+from twext.web2.http import HTTPError, StatusResponse
+from twext.web2.dav import davxml
+from twext.web2.dav.element.parser import lookupElement
+from twext.web2.dav.util import davXMLFromStream
+
+from twext.python.log import Logger
+from twext.web2.dav.http import ErrorResponse
 
 from twistedcaldav import caldavxml
-from twistedcaldav.log import Logger
 
 log = Logger()
 
@@ -47,8 +47,8 @@ def http_REPORT(self, request):
     """
     Respond to a REPORT request. (RFC 3253, section 3.6)
     """
-    if not self.fp.exists():
-        log.err("File not found: %s" % (self.fp.path,))
+    if not self.exists():
+        log.err("Resource not found: %s" % (self,))
         raise HTTPError(responsecode.NOT_FOUND)
 
     #
@@ -119,7 +119,8 @@ def http_REPORT(self, request):
 
         raise HTTPError(ErrorResponse(
             responsecode.FORBIDDEN,
-            davxml.SupportedReport()
+            davxml.SupportedReport(),
+            "Report not supported on this resource",
         ))
 
     #
