@@ -20,19 +20,19 @@ CalDAV freebusy report
 
 __all__ = ["report_urn_ietf_params_xml_ns_caldav_free_busy_query"]
 
-from twext.web2.dav.davxml import ErrorResponse
+from twext.python.log import Logger
+from twext.web2.dav.http import ErrorResponse
 
 from twisted.internet.defer import inlineCallbacks, returnValue
-from twisted.web2 import responsecode
-from twisted.web2.dav import davxml
-from twisted.web2.dav.method.report import NumberOfMatchesWithinLimits
-from twisted.web2.http import HTTPError, Response, StatusResponse
-from twisted.web2.http_headers import MimeType
-from twisted.web2.stream import MemoryStream
+from twext.web2 import responsecode
+from twext.web2.dav import davxml
+from twext.web2.dav.method.report import NumberOfMatchesWithinLimits
+from twext.web2.http import HTTPError, Response, StatusResponse
+from twext.web2.http_headers import MimeType
+from twext.web2.stream import MemoryStream
 
 from twistedcaldav import caldavxml
 from twistedcaldav.method import report_common
-from twistedcaldav.log import Logger
 
 log = Logger()
 
@@ -62,7 +62,7 @@ def report_urn_ietf_params_xml_ns_caldav_free_busy_query(self, request, freebusy
         """
         Run a free busy report on the specified calendar collection
         accumulating the free busy info for later processing.
-        @param calresource: the L{CalDAVFile} for a calendar collection.
+        @param calresource: the L{CalDAVResource} for a calendar collection.
         @param uri: the uri for the calendar collecton resource.
         """
         
@@ -80,7 +80,11 @@ def report_urn_ietf_params_xml_ns_caldav_free_busy_query(self, request, freebusy
         yield report_common.applyToCalendarCollections(self, request, request.uri, depth, generateFreeBusyInfo, (caldavxml.ReadFreeBusy(),))
     except NumberOfMatchesWithinLimits:
         log.err("Too many matching components in free-busy report")
-        raise HTTPError(ErrorResponse(responsecode.FORBIDDEN, davxml.NumberOfMatchesWithinLimits()))
+        raise HTTPError(ErrorResponse(
+            responsecode.FORBIDDEN,
+            davxml.NumberOfMatchesWithinLimits(),
+            "Too many components"
+        ))
     
     # Now build a new calendar object with the free busy info we have
     fbcalendar = report_common.buildFreeBusyResult(fbinfo, timerange)
