@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2008 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,18 +26,20 @@ The notification server process is implemented as a twistd plugin
 services -- one handling the internal channel between icalserver
 and notification server, the other handling the external channel
 between notification server and a remote consumer.
-
-The icalserver tap creates a NotificationClient object at startup;
-it deals with passing along notifications to the notification server.
-These notifications originate from cache.py:MemcacheChangeNotifier.changed().
 """
 
 # TODO: add CalDAVTester test for examining new xmpp-uri property
 
+import uuid
+from fnmatch import fnmatch
+
+from zope.interface import Interface, implements
+
+from twext.python.log import LoggingMixIn
+
 from twisted.internet.protocol import ReconnectingClientFactory, ServerFactory
-from twisted.internet.address import IPv4Address
 from twisted.internet.ssl import ClientContextFactory
-from twisted.internet.defer import inlineCallbacks, Deferred
+from twisted.internet.defer import inlineCallbacks, Deferred, returnValue
 from twisted.protocols.basic import LineReceiver
 from twisted.plugin import IPlugin
 from twisted.application import internet, service
@@ -48,73 +50,114 @@ from twisted.words.protocols.jabber.jid import JID
 from twisted.words.protocols.jabber.client import XMPPAuthenticator, IQAuthInitializer
 from twisted.words.protocols.jabber.xmlstream import IQ
 from twisted.words.xish import domish
-from twistedcaldav.log import LoggingMixIn
-from twistedcaldav.config import config, defaultConfig, defaultConfigFile
+from twistedcaldav.config import config
 from twistedcaldav.memcacher import Memcacher
+from twistedcaldav.stdconfig import DEFAULT_CONFIG, DEFAULT_CONFIG_FILE
 from twistedcaldav import memcachepool
-from zope.interface import Interface, implements
-from fnmatch import fnmatch
-import uuid
 
 __all__ = [
-    "ClientNotifier",
     "Coalescer",
-    "getNodeCacher",
-    "getNotificationClient",
-    "getPubSubConfiguration",
-    "getPubSubHeartbeatURI",
-    "getPubSubPath",
-    "getPubSubXMPPURI",
     "INotifier",
-    "installNotificationClient",
     "InternalNotificationFactory",
     "InternalNotificationProtocol",
-    "NotificationClient",
     "NotificationClientFactory",
     "NotificationClientLineProtocol",
     "NotificationServiceMaker",
+    "Notifier",
+    "NotifierFactory",
     "SimpleLineNotificationFactory",
     "SimpleLineNotificationProtocol",
     "SimpleLineNotifier",
     "SimpleLineNotifierService",
     "XMPPNotificationFactory",
     "XMPPNotifier",
+    "getNodeCacher",
+    "getPubSubAPSConfiguration",
+    "getPubSubConfiguration",
+    "getPubSubHeartbeatURI",
+    "getPubSubPath",
+    "getPubSubXMPPURI",
+    "getXMPPSettings",
 ]
 
+
 # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 # Classes used within calendarserver itself
 #
 
-class ClientNotifier(LoggingMixIn):
+class Notifier(LoggingMixIn):
     """
-    Provides a method to send change notifications to the L{NotificationClient}.
+    Provides a hook for sending change notifications to the
+    L{NotifierFactory}.
     """
 
-    def __init__(self, resource, configOverride=None):
-        self._resource = resource
+    def __init__(self, notifierFactory, label="default", id=None, prefix=None):
+        self._notifierFactory = notifierFactory
+        self._ids = { label : self.normalizeID(id) }
         self._notify = True
-        self.config = configOverride or config
+        self._prefix = prefix
+
+    def normalizeID(self, id):
+        urn = "urn:uuid:"
+        try:
+            if id.startswith(urn):
+                return id[len(urn):]
+        except AttributeError:
+            pass
+        return id
 
     def enableNotify(self, arg):
-        url = self._resource.url()
-        self.log_debug("enableNotify: %s" % (url,))
+        self.log_debug("enableNotify: %s" % (self._ids['default'][1],))
         self._notify = True
 
     def disableNotify(self):
-        url = self._resource.url()
-        self.log_debug("disableNotify: %s" % (url,))
+        self.log_debug("disableNotify: %s" % (self._ids['default'][1],))
         self._notify = False
 
     def notify(self, op="update"):
-        url = self._resource.url()
+        for label in self._ids.iterkeys():
+            id = self.getID(label=label)
+            if id is not None:
+                if self._notify:
+                    self.log_debug("Notifications are enabled: %s %s %s" %
+                        (op, label, id))
+                    self._notifierFactory.send(op, id)
+                else:
+                    self.log_debug("Skipping notification for: %s" % (id,))
 
-        if self.config.Notifications.Enabled:
-            if self._notify:
-                self.log_debug("Notifications are enabled: %s %s" % (op, url))
-                return getNotificationClient().send(op, url)
-            else:
-                self.log_debug("Skipping notification for: %s" % (url,))
+    def clone(self, label="default", id=None):
+        newNotifier = self.__class__(self._notifierFactory)
+        newNotifier._ids = self._ids.copy()
+        newNotifier._ids[label] = id
+        newNotifier._prefix = self._prefix
+        return newNotifier
 
+    def addID(self, label="default", id=None):
+        self._ids[label] = self.normalizeID(id)
+
+    def getID(self, label="default"):
+        id = self._ids.get(label, None)
+        if self._prefix is None:
+            return id
+        else:
+            return "%s|%s" % (self._prefix, id)
+
+    @inlineCallbacks
+    def nodeName(self, label="default"):
+        id = self.getID(label=label)
+        pubSubConfig = self._notifierFactory.pubSubConfig
+        name = getPubSubPath(id, pubSubConfig)
+        if pubSubConfig["enabled"]:
+            try:
+                if self._notifierFactory.nodeCacher:
+                    nodeCacher = self._notifierFactory.nodeCacher
+                else:
+                    nodeCacher = getNodeCacher()
+                (yield nodeCacher.waitForNode(self, name))
+            except NodeCreationException, e:
+                self.log_warn(e)
+                returnValue(None)
+        returnValue(name)
 
 class NotificationClientLineProtocol(LineReceiver, LoggingMixIn):
     """
@@ -170,19 +213,24 @@ class NotificationClientFactory(ReconnectingClientFactory,
         return p
 
 
-class NotificationClient(LoggingMixIn):
+class NotifierFactory(LoggingMixIn):
     """
-    Notification Client
+    Notifier Factory
 
-    Forwards on notifications from ClientNotifiers to the
-    notification server.  A NotificationClient is installed by the tap at
-    startup.
+    Creates Notifier instances and forwards notifications from them to the
+    gateway.
     """
 
-    def __init__(self, host, port, reactor=None):
+    def __init__(self, gatewayHost, gatewayPort, pubSubConfig=None,
+        nodeCacher=None, reactor=None):
+
         self.factory = None
-        self.host = host
-        self.port = port
+
+        self.gatewayHost = gatewayHost
+        self.gatewayPort = gatewayPort
+        self.pubSubConfig = pubSubConfig
+        self.nodeCacher = nodeCacher
+
         self.observers = set()
         self.queued = set()
 
@@ -190,13 +238,14 @@ class NotificationClient(LoggingMixIn):
             from twisted.internet import reactor
         self.reactor = reactor
 
-    def send(self, op, uri):
+    def send(self, op, id):
         if self.factory is None:
             self.factory = NotificationClientFactory(self)
-            self.reactor.connectTCP(self.host, self.port, self.factory)
+            self.reactor.connectTCP(self.gatewayHost, self.gatewayPort,
+                self.factory)
             self.log_debug("Creating factory")
 
-        msg = "%s %s" % (op, str(uri))
+        msg = "%s %s" % (op, str(id))
         if self.factory.isReady() and self.observers:
             for observer in self.observers:
                 self.log_debug("Sending to notification server: %s" % (msg,))
@@ -219,15 +268,9 @@ class NotificationClient(LoggingMixIn):
     def removeObserver(self, observer):
         self.observers.remove(observer)
 
+    def newNotifier(self, label="default", id=None, prefix=None):
+        return Notifier(self, label=label, id=id, prefix=prefix)
 
-_notificationClient = None
-
-def installNotificationClient(host, port, klass=NotificationClient, reactor=None):
-    global _notificationClient
-    _notificationClient = klass(host, port, reactor=reactor)
-
-def getNotificationClient():
-    return _notificationClient
 
 
 
@@ -274,6 +317,19 @@ class NodeCacher(Memcacher, LoggingMixIn):
             self.log_debug("Giving up!")
             raise NodeCreationException("Could not create node %s" % (nodeName,))
 
+    def createNode(self, notifier, nodeName):
+        """
+        Check with memcached to see if this node is known to exist, and if
+        not, request it be created (without waiting)
+        """
+        def _nodeExistenceChecked(result):
+            if not result:
+                notifier.notify(op="create")
+
+        d = self.nodeExists(nodeName)
+        d.addCallback(_nodeExistenceChecked)
+        return d
+
 
 _nodeCacher = None
 
@@ -303,8 +359,8 @@ class InternalNotificationProtocol(LineReceiver):
     """
 
     def lineReceived(self, line):
-        op, uri = line.strip().split()
-        self.factory.coalescer.add(op, uri)
+        op, id = line.strip().split()
+        self.factory.coalescer.add(op, id)
 
 
 class InternalNotificationFactory(ServerFactory):
@@ -325,14 +381,20 @@ class Coalescer(LoggingMixIn):
     """
     Coalescer
 
-    A queue which hangs on to incoming uris for some period of time before
+    A queue which hangs on to incoming ids for some period of time before
     passing them along to the external notifier listening for these updates.
     A chatty CalDAV client can make several changes in a short period of time,
     and the Coalescer buffers the external clients somewhat.
     """
 
     delaySeconds = 5
-    sendAnywayAfterCount = 5
+
+    # sendAnywayAfterCount can be used to control how many times a notification
+    # countdown timer is reset because of new changes.  Once a notification
+    # has been delayed 'sendAnywayAfterCount' times, it is sent anyway,
+    # otherwise a busy calendar might never have a notification sent out.
+    # Set this to 0 to disable the timer reset feature.
+    sendAnywayAfterCount = 0
 
     def __init__(self, notifiers, reactor=None, delaySeconds=None,
         sendAnywayAfterCount=None):
@@ -347,40 +409,40 @@ class Coalescer(LoggingMixIn):
             from twisted.internet import reactor
         self.reactor = reactor
 
-        self.uris = {}
+        self.ids = {}
         self.notifiers = notifiers
 
-    def add(self, op, uri):
+    def add(self, op, id):
 
         if op == "create":
             # we don't want to delay a "create" notification; this opcode
             # is meant for XMPP pubsub -- it means create and configure the
             # node but don't publish to it
             for notifier in self.notifiers:
-                notifier.enqueue(op, uri)
+                notifier.enqueue(op, id)
 
         else: # normal update notification
-            delayed, count = self.uris.get(uri, [None, 0])
+            delayed, count = self.ids.get(id, [None, 0])
 
             if delayed and delayed.active():
                 count += 1
                 if count < self.sendAnywayAfterCount:
                     # reschedule for delaySeconds in the future
                     delayed.reset(self.delaySeconds)
-                    self.uris[uri][1] = count
-                    self.log_debug("Delaying: %s" % (uri,))
+                    self.ids[id][1] = count
+                    self.log_debug("Delaying: %s" % (id,))
                 else:
-                    self.log_debug("Not delaying to avoid starvation: %s" % (uri,))
+                    self.log_debug("Not delaying to avoid starvation: %s" % (id,))
             else:
-                self.log_debug("Scheduling: %s" % (uri,))
-                self.uris[uri] = [self.reactor.callLater(self.delaySeconds,
-                    self.delayedEnqueue, op, uri), 0]
+                self.log_debug("Scheduling: %s" % (id,))
+                self.ids[id] = [self.reactor.callLater(self.delaySeconds,
+                    self.delayedEnqueue, op, id), 0]
 
-    def delayedEnqueue(self, op, uri):
-        self.log_debug("Time to send: %s" % (uri,))
-        self.uris[uri][1] = 0
+    def delayedEnqueue(self, op, id):
+        self.log_debug("Time to send: %s" % (id,))
+        self.ids[id][1] = 0
         for notifier in self.notifiers:
-            notifier.enqueue(op, uri)
+            notifier.enqueue(op, id)
 
 
 
@@ -395,13 +457,13 @@ class INotifier(Interface):
     Defines an enqueue method that Notifier classes need to implement.
     """
 
-    def enqueue(self, op, uri):
+    def enqueue(self, op, id):
         """
-        Let's the notifier object know that a change has been made for this
-        uri, and enough time has passed to allow for coalescence.
+        Let the notifier object know that a change has been made for this
+        id, and enough time has passed to allow for coalescence.
 
         @type op: C{str}
-        @type uri: C{str}
+        @type id: C{str}
         """
 
 
@@ -409,10 +471,10 @@ class SimpleLineNotifier(LoggingMixIn):
     """
     Simple Line Notifier
 
-    Listens for uris from the coalescer and writes them out to any
+    Listens for ids from the coalescer and writes them out to any
     connected clients.  Each line is simply a sequence number, a
-    space, and a uri string.  If the external client sends a sequence
-    number, this notifier will send notification lines for each uri
+    space, and an id string.  If the external client sends a sequence
+    number, this notifier will send notification lines for each id
     that was changed since that sequence number was originally sent.
     A history of such sequence numbers is stored in a python dict.
     If the external client sends a zero, then the history is cleared
@@ -434,32 +496,32 @@ class SimpleLineNotifier(LoggingMixIn):
         self.observers = set()
         self.sentReset = False
 
-    def enqueue(self, op, uri):
+    def enqueue(self, op, id):
 
         if op == "update":
 
             self.latestSeq += 1L
 
             # Update history
-            self.history[uri] = self.latestSeq
+            self.history[id] = self.latestSeq
 
             for observer in self.observers:
-                msg = "%d %s" % (self.latestSeq, uri)
+                msg = "%d %s" % (self.latestSeq, id)
                 self.log_debug("Sending %s" % (msg,))
                 observer.sendLine(msg)
 
     def reset(self):
         self.latestSeq = 0L
-        self.history = { } # keys=uri, values=sequenceNumber
+        self.history = { } # keys=id, values=sequenceNumber
 
     def playback(self, observer, oldSeq):
 
         hist = self.history
-        toSend = [(hist[uri], uri) for uri in hist if hist[uri] > oldSeq]
+        toSend = [(hist[id], id) for id in hist if hist[id] > oldSeq]
         toSend.sort() # sorts the tuples based on numeric sequence number
 
-        for seq, uri in toSend:
-            msg = "%d %s" % (seq, uri)
+        for seq, id in toSend:
+            msg = "%d %s" % (seq, id)
             self.log_debug("Sending %s" % (msg,))
             observer.sendLine(msg)
 
@@ -544,7 +606,7 @@ class XMPPNotifier(LoggingMixIn):
 
     Uses pubsub XMPP requests to let subscribers know when there
     has been a change made to a DAV resource (currently just
-    CalendarHomeFiles).  Uses XMPP login info from the config file
+    CalendarHomeResources).  Uses XMPP login info from the config file
     to determine which pubsub service to connect to.  When it's
     time to send a notification, XMPPNotifier computes a node path
     corresponding to the DAV resource and emits a publish request
@@ -602,10 +664,10 @@ class XMPPNotifier(LoggingMixIn):
             self.reactor.callLater(self.settings['HeartbeatMinutes'] * 60,
                 self.sendHeartbeat)
 
-    def enqueue(self, op, uri, lock=True):
+    def enqueue(self, op, id, lock=True):
         if self.xmlStream is not None:
-            # Convert uri to node
-            nodeName = self.uriToNodeName(uri)
+            # Convert id to node
+            nodeName = getPubSubPath(id, getPubSubConfiguration(self.config))
             if op == "create":
                 if not self.lockNode(nodeName):
                     # this node is busy, so it must already be created, or at
@@ -615,9 +677,6 @@ class XMPPNotifier(LoggingMixIn):
             else:
                 self.publishNode(nodeName, lock=lock)
 
-    def uriToNodeName(self, uri):
-        return getPubSubPath(uri, getPubSubConfiguration(self.config))
-
     def publishNode(self, nodeName, lock=True):
         if self.xmlStream is None:
             # We lost our connection
@@ -631,11 +690,10 @@ class XMPPNotifier(LoggingMixIn):
             iq = IQ(self.xmlStream)
             pubsubElement = iq.addElement('pubsub', defaultUri=self.pubsubNS)
             publishElement = pubsubElement.addElement('publish')
-            publishElement['node'] = nodeName
+            publishElement['node'] = nodeName.decode("utf-8")
             if self.settings["NodeConfiguration"]["pubsub#deliver_payloads"] == '1':
                 itemElement = publishElement.addElement('item')
-                payloadElement = itemElement.addElement('plistfrag',
-                    defaultUri='plist-apple')
+                itemElement.addElement('plistfrag', defaultUri='plist-apple')
 
             self.sendDebug("Publishing (%s)" % (nodeName,), iq)
             d = iq.send(to=self.settings['ServiceAddress'])
@@ -665,7 +723,13 @@ class XMPPNotifier(LoggingMixIn):
                 self.sendDebug("Node publish failed (%s)" % (nodeName,), iq)
                 # Don't know how to proceed
                 self.unlockNode(None, nodeName)
+        except AttributeError:
+            # We did not get an XML response; most likely it was a disconnection
+            self.unlockNode(None, nodeName)
+            # Don't re-raise, just unlock and ignore
         except:
+            # Note: this block is not a "finally" because in the case of a 404
+            # we don't want to unlock yet
             self.unlockNode(None, nodeName)
             raise
 
@@ -679,7 +743,7 @@ class XMPPNotifier(LoggingMixIn):
             iq = IQ(self.xmlStream)
             pubsubElement = iq.addElement('pubsub', defaultUri=self.pubsubNS)
             child = pubsubElement.addElement('create')
-            child['node'] = nodeName
+            child['node'] = nodeName.decode("utf-8")
             d = iq.send(to=self.settings['ServiceAddress'])
             d.addCallback(self.createNodeSuccess, nodeName, publish)
             d.addErrback(self.createNodeFailure, nodeName, publish)
@@ -709,7 +773,13 @@ class XMPPNotifier(LoggingMixIn):
                 self.log_error("PubSub node creation error: %s" %
                     (iq.toXml().encode('ascii', 'replace')),)
                 self.sendError("Node creation failed (%s)" % (nodeName,), iq)
+        except AttributeError:
+            # We did not get an XML response; most likely it was a disconnection
+            self.unlockNode(None, nodeName)
+            # Don't re-raise, just unlock and ignore
         except:
+            # Note: this block is not a "finally" because in the case of a 409
+            # we don't want to unlock yet
             self.unlockNode(None, nodeName)
             raise
 
@@ -725,7 +795,7 @@ class XMPPNotifier(LoggingMixIn):
             child = iq.addElement('pubsub',
                 defaultUri=self.pubsubNS+"#owner")
             child = child.addElement('configure')
-            child['node'] = nodeName
+            child['node'] = nodeName.decode("utf-8")
             d = iq.send(to=self.settings['ServiceAddress'])
             d.addCallback(self.requestConfigurationFormSuccess, nodeName,
                 publish)
@@ -760,7 +830,7 @@ class XMPPNotifier(LoggingMixIn):
                         filledPubSub = filledIq.addElement('pubsub',
                             defaultUri=self.pubsubNS+"#owner")
                         filledConfigure = filledPubSub.addElement('configure')
-                        filledConfigure['node'] = nodeName
+                        filledConfigure['node'] = nodeName.decode("utf-8")
                         filledForm = filledConfigure.addElement('x',
                             defaultUri='jabber:x:data')
                         filledForm['type'] = 'submit'
@@ -790,7 +860,7 @@ class XMPPNotifier(LoggingMixIn):
                             cancelPubSub = cancelIq.addElement('pubsub',
                                 defaultUri=self.pubsubNS+"#owner")
                             cancelConfig = cancelPubSub.addElement('configure')
-                            cancelConfig['node'] = nodeName
+                            cancelConfig['node'] = nodeName.decode("utf-8")
                             cancelX = cancelConfig.addElement('x',
                                 defaultUri='jabber:x:data')
                             cancelX['type'] = 'cancel'
@@ -851,6 +921,9 @@ class XMPPNotifier(LoggingMixIn):
             self.log_error("PubSub node configuration error: %s" %
                 (iq.toXml().encode('ascii', 'replace')),)
             self.sendError("Failed to configure node (%s)" % (nodeName,), iq)
+        except AttributeError:
+            # We did not get an XML response; most likely it was a disconnection
+            pass
         finally:
             self.unlockNode(None, nodeName)
 
@@ -868,7 +941,7 @@ class XMPPNotifier(LoggingMixIn):
             pubsubElement = iq.addElement('pubsub',
                 defaultUri=self.pubsubNS+"#owner")
             publishElement = pubsubElement.addElement('delete')
-            publishElement['node'] = nodeName
+            publishElement['node'] = nodeName.decode("utf-8")
             self.sendDebug("Deleting (%s)" % (nodeName,), iq)
             d = iq.send(to=self.settings['ServiceAddress'])
             d.addCallback(self.deleteNodeSuccess, nodeName)
@@ -887,6 +960,9 @@ class XMPPNotifier(LoggingMixIn):
             self.log_error("PubSub node delete error: %s" %
                 (iq.toXml().encode('ascii', 'replace')),)
             self.sendDebug("Node delete failed (%s)" % (nodeName,), iq)
+        except AttributeError:
+            # We did not get an XML response; most likely it was a disconnection
+            pass
         finally:
             self.unlockNode(None, nodeName)
 
@@ -900,6 +976,15 @@ class XMPPNotifier(LoggingMixIn):
             d.addCallback(self.handleRoster)
 
     def allowedInRoster(self, jid):
+        """ Returns True if jid matches any of the patterns in AllowedJIDs,
+            or is our own JID.  False otherwise. """
+
+        # Always allow our own JID (in case multiple servers are sharing it)
+        settings = self.settings
+        if settings is not None:
+            if settings["JID"] == jid:
+                return True
+
         for pattern in self.settings.get("AllowedJIDs", []):
             if fnmatch(jid, pattern):
                 return True
@@ -1160,34 +1245,107 @@ class XMPPNotificationFactory(xmlstream.XmlStreamFactory, LoggingMixIn):
         self.log_debug("SEND: %s" % unicode(buf, 'utf-8').encode('ascii',
             'replace'))
 
+def getXMPPSettings(config):
+    """ Return the XMPP settings if both overall notifications are enabled
+        and XMPP is enabled; None otherwise.
+    """
+    if config.Notifications.Enabled:
+        # return the first enabled xmpp service settings in the config file
+        for key, settings in config.Notifications.Services.iteritems():
+            if (settings["Service"] == "twistedcaldav.notify.XMPPNotifierService"
+                and settings["Enabled"]):
+                return settings
+    return None
 
 def getPubSubConfiguration(config):
     # TODO: Should probably cache this
-    results = { 'enabled' : False }
-
-    # return the first enabled xmpp service settings in the config file
-    for key, settings in config.Notifications.Services.iteritems():
-        if (settings["Service"] == "twistedcaldav.notify.XMPPNotifierService"
-            and settings["Enabled"]):
-            results['enabled'] = True
-            results['service'] = settings['ServiceAddress']
-            results['host'] = config.ServerHostName
-            results['port'] = config.SSLPort or config.HTTPPort
-            results['xmpp-server'] = settings['Host']
-            results['heartrate'] = settings['HeartbeatMinutes']
+    results = { 'enabled' : False, 'host' : config.ServerHostName }
+    settings = getXMPPSettings(config)
+    if settings is not None:
+        results['enabled'] = True
+        results['service'] = settings['ServiceAddress']
+        results['port'] = config.SSLPort or config.HTTPPort
+        results['xmpp-server'] = (
+            settings['Host'] if settings['Port'] == 5222
+            else "%s:%d" % (settings['Host'], settings['Port'])
+        )
+        results['heartrate'] = settings['HeartbeatMinutes']
 
     return results
 
-def getPubSubPath(uri, pubSubConfiguration):
-    path = "/Public/CalDAV/%s/%d/" % (pubSubConfiguration['host'],
-        pubSubConfiguration['port'])
-    if uri:
-        path += "%s/" % (uri.strip("/"),)
+def getPubSubAPSConfiguration(id, config):
+    """
+    Returns the Apple push notification settings specific to the notifier
+    ID, which includes a prefix that is either "CalDAV" or "CardDAV"
+    """
+    try:
+        prefix, id = id.split("|", 1)
+    except ValueError:
+        # id has no prefix, so we can't look up APS config
+        return None
+
+    # If we are directly talking to apple push, advertise those settings
+    applePushSettings = config.Notifications.Services.ApplePushNotifier
+    if applePushSettings.Enabled:
+        settings = {}
+        settings["APSBundleID"] = applePushSettings[prefix]["Topic"]
+        if config.EnableSSL:
+            url = "https://%s:%s/%s" % (config.ServerHostName, config.SSLPort,
+                applePushSettings.SubscriptionURL)
+        else:
+            url = "http://%s:%s/%s" % (config.ServerHostName, config.HTTPPort,
+                applePushSettings.SubscriptionURL)
+        settings["SubscriptionURL"] = url
+        settings["APSEnvironment"] = applePushSettings.Environment
+        return settings
+
+    # ...otherwise pick up the apple push settings we get via XMPP and
+    # apn bridge
+    settings = getXMPPSettings(config)
+    if settings is None:
+        return None
+
+    if (settings.has_key(prefix) and
+        settings[prefix]["APSBundleID"] and
+        settings[prefix]["SubscriptionURL"]):
+        return settings[prefix]
+
+    return None
+
+
+
+
+def getPubSubPath(id, pubSubConfiguration):
+    """
+    Generate a pubsub node path from an id and the pubsub configuration
+    @param id: a string identifying the resource that was modified.  If
+        the id has a "|" in it, what is to the left of the first "|" is
+        treated as a prefix and will be used for the root of the path.
+    @type id: C{str}
+
+    @param pubSubConfiguration: a dictionary containing various relevant
+        configuration data
+    @type pubSubConfiguration: C{dict}
+
+    """
+
+    path = "/"
+
+    try:
+        prefix, id = id.split("|", 1)
+        path += "%s/" % (prefix,)
+    except ValueError:
+        # id has no prefix
+        pass
+
+    path += "%s/" % (pubSubConfiguration['host'],)
+    if id:
+        path += "%s/" % (id,)
     return path
 
-def getPubSubXMPPURI(uri, pubSubConfiguration):
+def getPubSubXMPPURI(id, pubSubConfiguration):
     return "xmpp:%s?pubsub;node=%s" % (pubSubConfiguration['service'],
-        getPubSubPath(uri, pubSubConfiguration))
+        getPubSubPath(id, pubSubConfiguration))
 
 def getPubSubHeartbeatURI(pubSubConfiguration):
     return "xmpp:%s?pubsub;node=%s" % (pubSubConfiguration['service'],
@@ -1199,7 +1357,7 @@ def getPubSubHeartbeatURI(pubSubConfiguration):
 
 class NotificationOptions(Options):
     optParameters = [[
-        "config", "f", defaultConfigFile, "Path to configuration file."
+        "config", "f", DEFAULT_CONFIG_FILE, "Path to configuration file."
     ]]
 
     def __init__(self, *args, **kwargs):
@@ -1267,7 +1425,7 @@ class NotificationOptions(Options):
         if "=" in option:
             path, value = option.split('=')
             self._setOverride(
-                defaultConfig,
+                DEFAULT_CONFIG,
                 path.split('/'),
                 value,
                 self.overrides
@@ -1278,7 +1436,7 @@ class NotificationOptions(Options):
     opt_o = opt_option
 
     def postOptions(self):
-        config.loadConfig(self['config'])
+        config.load(self['config'])
         config.updateDefaults(self.overrides)
         self.parent['pidfile'] = None
 
@@ -1295,20 +1453,24 @@ class NotificationServiceMaker(object):
         #
         # Configure Memcached Client Pool
         #
-        if config.Memcached.ClientEnabled:
-            memcachepool.installPool(
-                IPv4Address(
-                    'TCP',
-                    config.Memcached.BindAddress,
-                    config.Memcached.Port),
-                config.Memcached.MaxClients)
+        memcachepool.installPools(
+            config.Memcached.Pools,
+            config.Memcached.MaxClients,
+        )
 
         multiService = service.MultiService()
 
+        from calendarserver.tap.util import storeFromConfig, getDBPool
+        pool, txnFactory = getDBPool(config)
+        if pool is not None:
+            pool.setServiceParent(multiService)
+        store = storeFromConfig(config, txnFactory)
+
         notifiers = []
         for key, settings in config.Notifications.Services.iteritems():
             if settings["Enabled"]:
-                notifier = namedClass(settings["Service"])(settings)
+                notifier = namedClass(settings["Service"]).makeService(settings,
+                    store)
                 notifier.setServiceParent(multiService)
                 notifiers.append(notifier)
 
@@ -1324,13 +1486,17 @@ class NotificationServiceMaker(object):
 
 class SimpleLineNotifierService(service.Service):
 
+    @classmethod
+    def makeService(cls, settings, store):
+        return cls(settings)
+
     def __init__(self, settings):
         self.notifier = SimpleLineNotifier(settings)
         self.server = internet.TCPServer(settings["Port"],
             SimpleLineNotificationFactory(self.notifier))
 
-    def enqueue(self, op, uri):
-        self.notifier.enqueue(op, uri)
+    def enqueue(self, op, id):
+        self.notifier.enqueue(op, id)
 
     def startService(self):
         self.server.startService()
@@ -1341,6 +1507,10 @@ class SimpleLineNotifierService(service.Service):
 
 class XMPPNotifierService(service.Service):
 
+    @classmethod
+    def makeService(cls, settings, store):
+        return cls(settings)
+
     def __init__(self, settings):
         self.notifier = XMPPNotifier(settings)
 
@@ -1353,12 +1523,11 @@ class XMPPNotifierService(service.Service):
             self.client = internet.TCPClient(settings["Host"], settings["Port"],
                 XMPPNotificationFactory(self.notifier, settings))
 
-    def enqueue(self, op, uri):
-        self.notifier.enqueue(op, uri)
+    def enqueue(self, op, id):
+        self.notifier.enqueue(op, id)
 
     def startService(self):
         self.client.startService()
 
     def stopService(self):
         self.client.stopService()
-
