@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2011 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -30,26 +30,53 @@ import cStringIO as StringIO
 
 class sqlgenerator(object):
     
-    FROM          =" from "
-    WHERE         =" where "
-    RESOURCEDB    = "RESOURCE"
-    TIMESPANDB    = "TIMESPAN"
-    NOTOP         = "NOT "
-    ANDOP         = " AND "
-    OROP          = " OR "
-    CONTAINSOP    = " GLOB "
-    NOTCONTAINSOP = " NOT GLOB "
-    ISOP          = " == "
-    ISNOTOP       = " != "
-    INOP          = " IN "
-    NOTINOP       = " NOT IN "
+    FROM             =" from "
+    WHERE            =" where "
+    RESOURCEDB       = "RESOURCE"
+    TIMESPANDB       = "TIMESPAN"
+    TRANSPARENCYDB   = "TRANSPARENCY"
+    PERUSERDB        = "PERUSER"
+    NOTOP            = "NOT "
+    ANDOP            = " AND "
+    OROP             = " OR "
+    CONTAINSOP       = " GLOB "
+    NOTCONTAINSOP    = " NOT GLOB "
+    ISOP             = " == "
+    ISNOTOP          = " != "
+    STARTSWITHOP     = " GLOB "
+    NOTSTARTSWITHOP  = " NOT GLOB "
+    ENDSWITHOP       = " GLOB "
+    NOTENDSWITHOP    = " NOT GLOB "
+    INOP             = " IN "
+    NOTINOP          = " NOT IN "
 
-    TIMESPANTEST         = "((TIMESPAN.FLOAT == 'N' AND TIMESPAN.START < %s AND TIMESPAN.END > %s) OR (TIMESPAN.FLOAT == 'Y' AND TIMESPAN.START < %s AND TIMESPAN.END > %s)) AND TIMESPAN.NAME == RESOURCE.NAME"
-    TIMESPANTEST_NOEND   = "((TIMESPAN.FLOAT == 'N' AND TIMESPAN.END > %s) OR (TIMESPAN.FLOAT == 'Y' AND TIMESPAN.END > %s)) AND TIMESPAN.NAME == RESOURCE.NAME"
-    TIMESPANTEST_NOSTART = "((TIMESPAN.FLOAT == 'N' AND TIMESPAN.START < %s) OR (TIMESPAN.FLOAT == 'Y' AND TIMESPAN.START < %s)) AND TIMESPAN.NAME == RESOURCE.NAME"
+    FIELDS         = {
+        "TYPE": "RESOURCE.TYPE",
+        "UID":  "RESOURCE.UID",
+    }
 
-    def __init__(self, expr):
+    TIMESPANTEST         = "((TIMESPAN.FLOAT == 'N' AND TIMESPAN.START < %s AND TIMESPAN.END > %s) OR (TIMESPAN.FLOAT == 'Y' AND TIMESPAN.START < %s AND TIMESPAN.END > %s))"
+    TIMESPANTEST_NOEND   = "((TIMESPAN.FLOAT == 'N' AND TIMESPAN.END > %s) OR (TIMESPAN.FLOAT == 'Y' AND TIMESPAN.END > %s))"
+    TIMESPANTEST_NOSTART = "((TIMESPAN.FLOAT == 'N' AND TIMESPAN.START < %s) OR (TIMESPAN.FLOAT == 'Y' AND TIMESPAN.START < %s))"
+    TIMESPANTEST_TAIL_PIECE = " AND TIMESPAN.RESOURCEID == RESOURCE.RESOURCEID"
+    TIMESPANTEST_JOIN_ON_PIECE = "TIMESPAN.INSTANCEID == TRANSPARENCY.INSTANCEID AND TRANSPARENCY.PERUSERID == %s"
+
+    def __init__(self, expr, calendarid, userid, freebusy=False):
+        """
+        
+        @param expr: the query expression object model
+        @type expr: L{twistedcaldav.query.calendarqueryfilter.Filter}
+        @param calendarid: resource ID - not used for file-based per-calendar indexes
+        @type calendarid: C{int}
+        @param userid: user for whom query is being done - query will be scoped to that user's privileges and their transparency
+        @type userid: C{str}
+        @param freebusy: whether or not a freebusy query is being done - if it is, additional time range and transparency information is returned
+        @type freebusy: C{bool}
+        """
         self.expression = expr
+        self.calendarid = calendarid
+        self.userid = userid if userid else ""
+        self.freebusy = freebusy
         self.usedtimespan = False
         
     def generate(self):
@@ -63,17 +90,39 @@ class sqlgenerator(object):
         # Init state
         self.sout = StringIO.StringIO()
         self.arguments = []
+        self.substitutions = []
         self.usedtimespan = False
         
         # Generate ' where ...' partial statement
-        self.sout.write(self.WHERE)
         self.generateExpression(self.expression)
-
+            
         # Prefix with ' from ...' partial statement
         select = self.FROM + self.RESOURCEDB
         if self.usedtimespan:
-            select += ", " + self.TIMESPANDB
+            
+            # Free busy needs transparency join
+            if self.freebusy:
+                self.frontArgument(self.userid)
+                select += ", %s LEFT OUTER JOIN %s ON (%s)" % (
+                    self.TIMESPANDB,
+                    self.TRANSPARENCYDB,
+                    self.TIMESPANTEST_JOIN_ON_PIECE
+                )
+            else:
+                select += ", %s" % (
+                    self.TIMESPANDB,
+                )
+        select += self.WHERE
+        if self.usedtimespan:
+            select += "("
         select += self.sout.getvalue()
+        if self.usedtimespan:
+            if self.calendarid:
+                self.setArgument(self.calendarid)
+            select += ")%s" % (self.TIMESPANTEST_TAIL_PIECE,)
+    
+        select = select % tuple(self.substitutions)
+
         return select, self.arguments
         
     def generateExpression(self, expr):
@@ -91,6 +140,7 @@ class sqlgenerator(object):
             # Wipe out the ' where ...' clause so everything is matched
             self.sout.truncate(0)
             self.arguments = []
+            self.substitutions = []
             self.usedtimespan = False
         
         # NOT
@@ -121,19 +171,20 @@ class sqlgenerator(object):
         # time-range
         elif isinstance(expr, expression.timerangeExpression):
             if expr.start and expr.end:
-                arg1 = self.setArgument(expr.end)
-                arg2 = self.setArgument(expr.start)
-                arg3 = self.setArgument(expr.endfloat)
-                arg4 = self.setArgument(expr.startfloat)
-                test = self.TIMESPANTEST % (arg1, arg2, arg3, arg4)
+                self.setArgument(expr.end)
+                self.setArgument(expr.start)
+                self.setArgument(expr.endfloat)
+                self.setArgument(expr.startfloat)
+                test = self.TIMESPANTEST
             elif expr.start and expr.end is None:
-                arg1 = self.setArgument(expr.start)
-                arg2 = self.setArgument(expr.startfloat)
-                test = self.TIMESPANTEST_NOEND % (arg1, arg2)
+                self.setArgument(expr.start)
+                self.setArgument(expr.startfloat)
+                test = self.TIMESPANTEST_NOEND
             elif not expr.start and expr.end:
-                arg1 = self.setArgument(expr.end)
-                arg2 = self.setArgument(expr.endfloat)
-                test = self.TIMESPANTEST_NOSTART % (arg1, arg2)
+                self.setArgument(expr.end)
+                self.setArgument(expr.endfloat)
+                test = self.TIMESPANTEST_NOSTART
+                
             self.sout.write(test)
             self.usedtimespan = True
         
@@ -141,13 +192,13 @@ class sqlgenerator(object):
         elif isinstance(expr, expression.containsExpression):
             self.sout.write(expr.field)
             self.sout.write(self.CONTAINSOP)
-            self.addArgument(expr.text)
+            self.addArgument(self.containsArgument(expr.text))
         
         # NOT CONTAINS
         elif isinstance(expr, expression.notcontainsExpression):
             self.sout.write(expr.field)
             self.sout.write(self.NOTCONTAINSOP)
-            self.addArgument(expr.text)
+            self.addArgument(self.containsArgument(expr.text))
         
         # IS
         elif isinstance(expr, expression.isExpression):
@@ -161,6 +212,30 @@ class sqlgenerator(object):
             self.sout.write(self.ISNOTOP)
             self.addArgument(expr.text)
         
+        # STARTSWITH
+        elif isinstance(expr, expression.startswithExpression):
+            self.sout.write(expr.field)
+            self.sout.write(self.STARTSWITHOP)
+            self.addArgument(self.startswithArgument(expr.text))
+        
+        # NOT STARTSWITH
+        elif isinstance(expr, expression.notstartswithExpression):
+            self.sout.write(expr.field)
+            self.sout.write(self.NOTSTARTSWITHOP)
+            self.addArgument(self.startswithArgument(expr.text))
+        
+        # ENDSWITH
+        elif isinstance(expr, expression.endswithExpression):
+            self.sout.write(expr.field)
+            self.sout.write(self.ENDSWITHOP)
+            self.addArgument(self.endswithArgument(expr.text))
+        
+        # NOT ENDSWITH
+        elif isinstance(expr, expression.notendswithExpression):
+            self.sout.write(expr.field)
+            self.sout.write(self.NOTENDSWITHOP)
+            self.addArgument(self.endswithArgument(expr.text))
+        
         # IN
         elif isinstance(expr, expression.inExpression):
             self.sout.write(expr.field)
@@ -185,7 +260,7 @@ class sqlgenerator(object):
 
     def generateSubExpression(self, expression):
         """
-        Generate an SQL expression possibly in paranethesis if its a compound expression.
+        Generate an SQL expression possibly in parenthesis if its a compound expression.
 
         @param expression: the L{baseExpression} to write out.
         @return: C{True} if the TIMESPAN table is used, C{False} otherwise.
@@ -203,9 +278,10 @@ class sqlgenerator(object):
         @param arg: the C{str} of the argument to add
         """
         
-        # Append argument to the list and add the appropriate substituion string to the output stream.
+        # Append argument to the list and add the appropriate substitution string to the output stream.
         self.arguments.append(arg)
-        self.sout.write(":" + str(len(self.arguments)))
+        self.substitutions.append(":" + str(len(self.arguments)))
+        self.sout.write("%s")
     
     def setArgument(self, arg):
         """
@@ -214,10 +290,29 @@ class sqlgenerator(object):
         @return: C{str} for argument substitution text
         """
         
-        # Append argument to the list and add the appropriate substituion string to the output stream.
+        # Append argument to the list and add the appropriate substitution string to the output stream.
         self.arguments.append(arg)
-        return ":" + str(len(self.arguments))
+        self.substitutions.append(":" + str(len(self.arguments)))
 
+    def frontArgument(self, arg):
+        """
+        
+        @param arg: the C{str} of the argument to add
+        @return: C{str} for argument substitution text
+        """
+        
+        # Append argument to the list and add the appropriate substitution string to the output stream.
+        self.arguments.insert(0, arg)
+        self.substitutions.append(":" + str(len(self.arguments)))
+
+    def containsArgument(self, arg):
+        return "*%s*" % (arg,)
+
+    def startswithArgument(self, arg):
+        return "%s*" % (arg,)
+
+    def endswithArgument(self, arg):
+        return "*%s" % (arg,)
 
 if __name__ == "__main__":
     
@@ -226,13 +321,13 @@ if __name__ == "__main__":
     e3 = expression.notcontainsExpression("SUMMARY", "help", True)
     e5 = expression.andExpression([e1, e2, e3])
     print e5
-    sql = sqlgenerator(e5)
+    sql = sqlgenerator(e5, 'dummy-cal', 'dummy-user')
     print sql.generate()
     e6 = expression.inExpression("TYPE", ("VEVENT", "VTODO",), False)
     print e6
-    sql = sqlgenerator(e6)
+    sql = sqlgenerator(e6, 'dummy-cal', 'dummy-user')
     print sql.generate()
     e7 = expression.notinExpression("TYPE", ("VEVENT", "VTODO",), False)
     print e7
-    sql = sqlgenerator(e7)
+    sql = sqlgenerator(e7, 'dummy-cal', 'dummy-user')
     print sql.generate()
