@@ -14,16 +14,17 @@
 # limitations under the License.
 ##
 
+from twext.python.log import Logger
+
 from twistedcaldav.config import config
-from twistedcaldav.dateops import normalizeToUTC, toString,\
-    normalizeStartEndDuration
 from twistedcaldav.ical import Component, Property
-from twistedcaldav.log import Logger
 from twistedcaldav.scheduling.cuaddress import normalizeCUAddr
 from twistedcaldav.scheduling.itip import iTipGenerator
 from twistedcaldav import accounting
 
 from difflib import unified_diff
+from pycalendar.period import PyCalendarPeriod
+from pycalendar.datetime import PyCalendarDateTime
 
 """
 Class that handles diff'ing two calendar objects.
@@ -70,7 +71,9 @@ class iCalDiff(object):
                 "DTSTAMP",
                 "LAST-MODIFIED",
             ))
-            calendar.removeXProperties()
+            calendar.removeXProperties(keep_properties=(
+                "X-APPLE-DROPBOX",
+            ))
             calendar.removePropertyParameters("ATTENDEE", ("RSVP", "SCHEDULE-STATUS", "SCHEDULE-FORCE-SEND",))
             calendar.normalizeAll()
             return calendar
@@ -207,7 +210,7 @@ class iCalDiff(object):
             
             # Whenever SCHEDULE-FORCE-SEND is explicitly set by the Organizer we assume the Organizer
             # is deliberately overwriting PARTSTAT
-            if new_attendee.params().get("SCHEDULE-FORCE-SEND", ["",])[0] == "REQUEST":
+            if new_attendee.parameterValue("SCHEDULE-FORCE-SEND", "") == "REQUEST":
                 continue
 
             # Transfer parameters from any old Attendees found
@@ -219,14 +222,14 @@ class iCalDiff(object):
                 self._transferParameter(old_attendee, new_attendee, "SCHEDULE-STATUS")
     
     def _transferParameter(self, old_property, new_property, parameter):
-        paramvalue = old_property.params().get(parameter)
+        paramvalue = old_property.parameterValue(parameter)
         if paramvalue is None:
             try:
-                del new_property.params()[parameter]
+                new_property.removeParameter(parameter)
             except KeyError:
                 pass
         else:
-            new_property.params()[parameter] = paramvalue
+            new_property.setParameter(parameter, paramvalue)
 
     def attendeeMerge(self, attendee):
         """
@@ -280,7 +283,7 @@ class iCalDiff(object):
                 # Get all EXDATEs in UTC
                 exdates = set()
                 for exdate in master.properties("EXDATE"):
-                    exdates.update([normalizeToUTC(value) for value in exdate.value()])
+                    exdates.update([value.getValue().duplicate().adjustToUTC() for value in exdate.value()])
                
             return exdates, map, master
         
@@ -329,7 +332,7 @@ class iCalDiff(object):
                     # Mark Attendee as DECLINED in the server instance
                     if self._attendeeDecline(self.newCalendar.overriddenComponent(rid)):
                         changeCausesReply = True
-                        changedRids.append(toString(rid) if rid else "")
+                        changedRids.append(rid.getText() if rid else "")
                 else:
                     # We used to generate a 403 here - but instead we now ignore this error and let the server data
                     # override the client
@@ -344,7 +347,9 @@ class iCalDiff(object):
                     overridden = self.newCalendar.overriddenComponent(rid)
                     self.newCalendar.removeComponent(overridden)
                     if self.newMaster:
-                        self.newMaster.addProperty(Property("EXDATE", [rid,]))
+                        # Use the original R-ID value so we preserve the timezone
+                        original_rid = component.propertyValue("RECURRENCE-ID")
+                        self.newMaster.addProperty(Property("EXDATE", [original_rid,]))
         
         # Derive a new component in the new calendar for each new one in setnew
         for key in setnew - setold:
@@ -405,7 +410,7 @@ class iCalDiff(object):
                 #return False, False, (), None
             changeCausesReply |= reply
             if reply:
-                changedRids.append(toString(rid) if rid else "")
+                changedRids.append(rid.getText() if rid else "")
 
         # We need to derive instances for any declined using an EXDATE
         for decline in sorted(declines):
@@ -416,7 +421,7 @@ class iCalDiff(object):
                     self.newCalendar.addComponent(overridden)
                     if self._attendeeDecline(overridden):
                         changeCausesReply = True
-                        changedRids.append(toString(decline) if decline else "")
+                        changedRids.append(decline.getText() if decline else "")
                 else:
                     self._logDiffError("attendeeMerge: Unable to override an instance to mark as DECLINED: %s" % (decline,))
                     return False, False, (), None
@@ -447,24 +452,24 @@ class iCalDiff(object):
         
         # Now look for items to transfer from one to the other.
         # We care about the ATTENDEE's PARTSTAT, TRANSP, VALARMS, X-APPLE-NEEDS-REPLY,
-        # DTSTAMP, LAST-MODIFIED, and ATTACH's referring to a dropbox
+        # DTSTAMP, LAST-MODIFIED, COMPLETED, and ATTACH's referring to a dropbox
         
         replyNeeded = False
 
         # ATTENDEE/PARTSTAT/RSVP
         serverAttendee = serverComponent.getAttendeeProperty((self.attendee,))
         clientAttendee = clientComponent.getAttendeeProperty((self.attendee,))
-        if serverAttendee.params().get("PARTSTAT", ("NEEDS-ACTION",))[0] != clientAttendee.params().get("PARTSTAT", ("NEEDS-ACTION",))[0]:
-            serverAttendee.params()["PARTSTAT"] = clientAttendee.params().get("PARTSTAT", "NEEDS-ACTION")
+        if serverAttendee.parameterValue("PARTSTAT", "NEEDS-ACTION") != clientAttendee.parameterValue("PARTSTAT", "NEEDS-ACTION"):
+            serverAttendee.setParameter("PARTSTAT", clientAttendee.parameterValue("PARTSTAT", "NEEDS-ACTION"))
             replyNeeded = True
-        if serverAttendee.params().get("RSVP", ("FALSE",))[0] != clientAttendee.params().get("RSVP", ("FALSE",))[0]:
-            if clientAttendee.params().get("RSVP", ("FALSE",))[0] == "FALSE":
+        if serverAttendee.parameterValue("RSVP", "FALSE") != clientAttendee.parameterValue("RSVP", "FALSE"):
+            if clientAttendee.parameterValue("RSVP", "FALSE") == "FALSE":
                 try:
-                    del serverAttendee.params()["RSVP"]
+                    serverAttendee.removeParameter("RSVP")
                 except KeyError:
                     pass
             else:
-                serverAttendee.params()["RSVP"] = ["TRUE",]
+                serverAttendee.setParameter("RSVP", "TRUE")
 
         # Transfer these properties from the client data
         replyNeeded |= self._transferProperty("X-CALENDARSERVER-PRIVATE-COMMENT", serverComponent, clientComponent)
@@ -472,6 +477,7 @@ class iCalDiff(object):
         self._transferProperty("DTSTAMP", serverComponent, clientComponent)
         self._transferProperty("LAST-MODIFIED", serverComponent, clientComponent)
         self._transferProperty("X-APPLE-NEEDS-REPLY", serverComponent, clientComponent)
+        self._transferProperty("COMPLETED", serverComponent, clientComponent)
         
         # Dropbox - this now never returns false
         self._transferDropBoxData(serverComponent, clientComponent)
@@ -504,7 +510,7 @@ class iCalDiff(object):
 
             # Remove existing ATTACH's from server
             for attachment in tuple(serverComponent.properties("ATTACH")):
-                valueType = attachment.paramValue("VALUE")
+                valueType = attachment.parameterValue("VALUE")
                 if valueType in (None, "URI"):
                     dataValue = attachment.value()
                     if dataValue.find(serverDropbox) != -1:
@@ -512,7 +518,7 @@ class iCalDiff(object):
         
             # Copy new ATTACH's to server
             for attachment in tuple(clientComponent.properties("ATTACH")):
-                valueType = attachment.paramValue("VALUE")
+                valueType = attachment.parameterValue("VALUE")
                 if valueType in (None, "URI"):
                     dataValue = attachment.value()
                     if dataValue.find(serverDropbox) != -1:
@@ -530,7 +536,7 @@ class iCalDiff(object):
         # Need to special case EXDATEs as an Attendee can effectively DECLINE by adding an EXDATE
         if serverProps[:-1] != clientProps[:-1]:
             invalidChanges = []
-            propNames = ("DTSTART", "DTEND", "RRULE", "RDATE", "EXDATE")
+            propNames = ("DTSTART", "DTEND", "DUE", "RRULE", "RDATE", "EXDATE")
             invalidChanges = [propName for ctr, propName in enumerate(propNames) if serverProps[ctr] != clientProps[ctr]]
             log.debug("Critical properties do not match: %s" % (", ".join(invalidChanges),))
             return False
@@ -538,7 +544,7 @@ class iCalDiff(object):
             # Bad if EXDATEs have been removed
             missing = serverProps[-1] - clientProps[-1]
             if missing:
-                log.debug("EXDATEs missing: %s" % (", ".join([toString(exdate) for exdate in missing]),))
+                log.debug("EXDATEs missing: %s" % (", ".join([exdate.getText() for exdate in missing]),))
                 return False
             declines.extend(clientProps[-1] - serverProps[-1])
             return True
@@ -548,22 +554,40 @@ class iCalDiff(object):
     def _getNormalizedDateTimeProperties(self, component):
         
         # Basic time properties
-        dtstart = component.getProperty("DTSTART")
-        dtend = component.getProperty("DTEND")
-        duration = component.getProperty("DURATION")
-        
-        newdtstart, newdtend = normalizeStartEndDuration(
-            dtstart.value(),
-            dtend.value() if dtend is not None else None,
-            duration.value() if duration is not None else None,
-        )
-        
+        if component.name() in ("VEVENT", "VJOURNAL",):
+            dtstart = component.getProperty("DTSTART")
+            dtend = component.getProperty("DTEND")
+            duration = component.getProperty("DURATION")
+            
+            timeRange = PyCalendarPeriod(
+                start    = dtstart.value()  if dtstart  is not None else None,
+                end      = dtend.value()    if dtend    is not None else None,
+                duration = duration.value() if duration is not None else None,
+            )
+            newdue = None
+            
+        elif component.name() == "VTODO":
+            dtstart = component.getProperty("DTSTART")
+            duration = component.getProperty("DURATION")
+            
+            if dtstart or duration:
+                timeRange = PyCalendarPeriod(
+                    start    = dtstart.value()  if dtstart  is not None else None,
+                    duration = duration.value() if duration is not None else None,
+                )
+            else:
+                timeRange = PyCalendarPeriod()
+
+            newdue = component.getProperty("DUE")
+            if newdue is not None:
+                newdue = newdue.value().duplicate().adjustToUTC()
+            
         # Recurrence rules - we need to normalize the order of the value parts
         newrrules = set()
         rrules = component.properties("RRULE")
         for rrule in rrules:
             indexedTokens = {}
-            indexedTokens.update([valuePart.split("=") for valuePart in rrule.value().split(";")])
+            indexedTokens.update([valuePart.split("=") for valuePart in rrule.value().getText().split(";")])
             sortedValue = ";".join(["%s=%s" % (key, value,) for key, value in sorted(indexedTokens.iteritems(), key=lambda x:x[0])])
             newrrules.add(sortedValue)
         
@@ -571,15 +595,18 @@ class iCalDiff(object):
         newrdates = set()
         rdates = component.properties("RDATE")
         for rdate in rdates:
-            newrdates.update([normalizeToUTC(value) for value in rdate.value()])
+            for value in rdate.value():
+                if isinstance(PyCalendarDateTime()):
+                    value = value.duplicate().adjustToUTC()
+                newrdates.add(value)
         
         # EXDATEs
         newexdates = set()
         exdates = component.properties("EXDATE")
         for exdate in exdates:
-            newexdates.update([normalizeToUTC(value) for value in exdate.value()])
+            newexdates.update([value.getValue().duplicate().adjustToUTC() for value in exdate.value()])
 
-        return newdtstart, newdtend, newrrules, newrdates, newexdates
+        return timeRange.getStart(), timeRange.getEnd(), newdue, newrrules, newrdates, newexdates
 
     def _transferProperty(self, propName, serverComponent, clientComponent):
 
@@ -605,8 +632,8 @@ class iCalDiff(object):
         @return: C{bool} indicating whether the PARTSTAT value was in fact changed
         """
         attendee = component.getAttendeeProperty((self.attendee,))
-        partstatChanged = attendee.params().get("PARTSTAT", ("NEEDS-ACTION",))[0] != "DECLINED"
-        attendee.params()["PARTSTAT"] = ["DECLINED",]
+        partstatChanged = attendee.parameterValue("PARTSTAT", "NEEDS-ACTION") != "DECLINED"
+        attendee.setParameter("PARTSTAT", "DECLINED")
         prop = component.getProperty("X-APPLE-NEEDS-REPLY")
         if prop:
             component.removeProperty(prop)
@@ -685,11 +712,7 @@ class iCalDiff(object):
         comp2 = self._componentDuplicateAndNormalize(comp2)
 
         # Diff all the properties
-        comp1.transformAllFromNative()
-        comp2.transformAllFromNative()
         propdiff = set(comp1.properties()) ^ set(comp2.properties())
-        comp1.transformAllToNative()
-        comp2.transformAllToNative()
         addedChanges = False
         
         propsChanged = {}
@@ -699,7 +722,6 @@ class iCalDiff(object):
                 "DTSTAMP",
                 "CREATED",
                 "LAST-MODIFIED",
-                "SEQUENCE",
                 "X-CALENDARSERVER-PRIVATE-COMMENT",
             ):
                 continue
@@ -708,17 +730,17 @@ class iCalDiff(object):
             prop1s = tuple(comp1.properties(prop.name()))
             prop2s = tuple(comp2.properties(prop.name()))
             if len(prop1s) == 1 and len(prop2s) == 1:
-                param1s = set(["%s=%s" % (name, value) for name, value in prop1s[0].params().iteritems()])
-                param2s = set(["%s=%s" % (name, value) for name, value in prop2s[0].params().iteritems()])
+                param1s = set(["%s=%s" % (name, prop1s[0].parameterValue(name)) for name in prop1s[0].parameterNames()])
+                param2s = set(["%s=%s" % (name, prop2s[0].parameterValue(name)) for name in prop2s[0].parameterNames()])
                 paramDiffs = param1s ^ param2s
                 propsChanged[prop.name()].update([param.split("=")[0] for param in paramDiffs])
-            if "ORIGINAL-TZID" in propsChanged[prop.name()]:
-                propsChanged[prop.name()].remove("ORIGINAL-TZID")
+            if "_TZID" in propsChanged[prop.name()]:
+                propsChanged[prop.name()].remove("_TZID")
                 propsChanged[prop.name()].add("TZID")
         
         if addedChanges:
             rid = comp1.getRecurrenceIDUTC()
-            rids[toString(rid) if rid is not None else ""] = propsChanged
+            rids[rid.getText() if rid is not None else ""] = propsChanged
 
     def _logDiffError(self, title):
 
