@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2006-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2011 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,7 @@
 ##
 
 """
-iTIP (RFC2446) processing.
+iTIP (RFC5546) processing.
 """
 
 #
@@ -30,15 +30,13 @@ iTIP (RFC2446) processing.
 # know how to deal with overridden instances.
 #
 
-import datetime
+from twext.python.log import Logger
 
 from twistedcaldav.config import config
-from twistedcaldav.dateops import normalizeToUTC, toString
-from twistedcaldav.log import Logger
-from twistedcaldav.ical import Property, iCalendarProductID, Component
+from twistedcaldav.ical import Property, iCalendarProductID, Component,\
+    ignoredComponents
 
-from vobject.icalendar import utc
-from vobject.icalendar import dateTimeToString
+from pycalendar.datetime import PyCalendarDateTime
 
 log = Logger()
 
@@ -76,7 +74,8 @@ class iTipProcessing(object):
     @staticmethod
     def processRequest(itip_message, calendar, recipient, autoprocessing=False):
         """
-        Process a METHOD=REQUEST.
+        Process a METHOD=REQUEST. We need to merge per-attendee properties such as TRANPS, COMPLETED etc
+        with the data coming from the organizer.
         
         @param itip_message: the iTIP message calendar object to process.
         @type itip_message:
@@ -85,26 +84,34 @@ class iTipProcessing(object):
         
         @return: a C{tuple} of:
             calendar object ready to save, or C{None} (request should be ignored)
-            a C{set} of iCalendar properties that changed, or C{None},
             a C{set} of recurrences that changed, or C{None}
         """
         
+        # Check sequencing
+        if not iTipProcessing.sequenceComparison(itip_message, calendar):
+            # Ignore out of sequence message
+            return None, None
+
         # Merge Organizer data with Attendee's own changes (VALARMs, Comment only for now).
         from twistedcaldav.scheduling.icaldiff import iCalDiff
         rids = iCalDiff(calendar, itip_message, False).whatIsDifferent()
 
         # Different behavior depending on whether a master component is present or not
+        # Here we cache per-attendee data from the master that we need to use in any new
+        # overridden components that the organizer added
         current_master = calendar.masterComponent()
         if current_master:
             master_valarms = [comp for comp in current_master.subcomponents() if comp.name() == "VALARM"]
             private_comments = current_master.properties("X-CALENDARSERVER-PRIVATE-COMMENT")
             transps = current_master.properties("TRANSP")
+            completeds = current_master.properties("COMPLETED")
             organizer = current_master.getProperty("ORGANIZER")
-            organizer_schedule_status = organizer.params().get("SCHEDULE-STATUS", None) if organizer else None
+            organizer_schedule_status = organizer.parameterValue("SCHEDULE-STATUS", None) if organizer else None
         else:
             master_valarms = ()
             private_comments = ()
             transps = ()
+            completeds = ()
             organizer_schedule_status = None
 
         if itip_message.masterComponent() is not None:
@@ -120,15 +127,17 @@ class iTipProcessing(object):
                 master_component.addProperty(comment)
             for transp in transps:
                 master_component.replaceProperty(transp)
+            for completed in completeds:
+                master_component.replaceProperty(completed)
             if organizer_schedule_status: 
                 organizer = master_component.getProperty("ORGANIZER")
                 if organizer:
-                    organizer.params()["SCHEDULE-STATUS"] = organizer_schedule_status
+                    organizer.setParameter("SCHEDULE-STATUS", organizer_schedule_status)
                 
             # Now try to match recurrences
             for component in new_calendar.subcomponents():
                 if component.name() != "VTIMEZONE" and component.getRecurrenceIDUTC() is not None:
-                    iTipProcessing.transferItems(calendar, master_valarms, private_comments, transps, organizer_schedule_status, component)
+                    iTipProcessing.transferItems(calendar, master_valarms, private_comments, transps, completeds, organizer_schedule_status, component)
             
             # Now try to match recurrences
             for component in calendar.subcomponents():
@@ -139,7 +148,7 @@ class iTipProcessing(object):
                         new_component = new_calendar.deriveInstance(rid, allowCancelled=allowCancelled)
                         if new_component:
                             new_calendar.addComponent(new_component)
-                            iTipProcessing.transferItems(calendar, master_valarms, private_comments, transps, organizer_schedule_status, new_component)
+                            iTipProcessing.transferItems(calendar, master_valarms, private_comments, transps, completeds, organizer_schedule_status, new_component)
             
             # Replace the entire object
             return new_calendar, rids
@@ -156,7 +165,7 @@ class iTipProcessing(object):
                         calendar.addComponent(component)
                 else:
                     component = component.duplicate()
-                    iTipProcessing.transferItems(calendar, master_valarms, private_comments, transps, organizer_schedule_status, component, remove_matched=True)
+                    iTipProcessing.transferItems(calendar, master_valarms, private_comments, transps, completeds, organizer_schedule_status, component, remove_matched=True)
                     calendar.addComponent(component)
                     if recipient and not autoprocessing:
                         iTipProcessing.fixForiCal3((component,), recipient, config.Scheduling.CalDAV.OldDraftCompatibility)
@@ -185,6 +194,11 @@ class iTipProcessing(object):
         assert itip_message.propertyValue("METHOD") == "CANCEL", "iTIP message must have METHOD:CANCEL"
         assert itip_message.resourceUID() == calendar.resourceUID(), "UIDs must be the same to process iTIP message"
 
+        # Check sequencing
+        if not iTipProcessing.sequenceComparison(itip_message, calendar):
+            # Ignore out of sequence message
+            return False, False, None
+
         # Check to see if this is a cancel of the entire event
         if itip_message.masterComponent() is not None:
             if autoprocessing:
@@ -259,6 +273,8 @@ class iTipProcessing(object):
         Process a METHOD=REPLY.
         
         TODO: Yes, I am going to ignore RANGE= on RECURRENCE-ID for now...
+        TODO: We have no way to track SEQUENCE/DTSTAMP on a per-attendee basis to correctly serialize out-of-order
+              replies.
         
         @param itip_message: the iTIP message calendar object to process.
         @type itip_message:
@@ -320,7 +336,7 @@ class iTipProcessing(object):
             if attendee:
                 attendees.add(attendee)
                 if rids is not None and (partstat or private_comment):
-                    rids.add((toString(rid), partstat, private_comment,))
+                    rids.add((rid.getText(), partstat, private_comment,))
 
         # Check for an invalid instance by itself
         len_attendees = len(attendees)
@@ -362,20 +378,20 @@ class iTipProcessing(object):
             return None, False, False
 
         attendee = attendees[0]
-        partstat = attendee.params().get("PARTSTAT", ("NEEDS-ACTION",))[0]
+        partstat = attendee.parameterValue("PARTSTAT", "NEEDS-ACTION")
         
         # Now find matching ATTENDEE in to_component
         existing_attendee = to_component.getAttendeeProperty((attendee.value(),))
         if existing_attendee:
-            oldpartstat = existing_attendee.params().get("PARTSTAT", ("NEEDS-ACTION",))[0]
-            existing_attendee.params()["PARTSTAT"] = [partstat]
-            existing_attendee.params()["SCHEDULE-STATUS"] = [reqstatus]
+            oldpartstat = existing_attendee.parameterValue("PARTSTAT", "NEEDS-ACTION")
+            existing_attendee.setParameter("PARTSTAT", partstat)
+            existing_attendee.setParameter("SCHEDULE-STATUS", reqstatus)
             partstat_changed = (oldpartstat != partstat)
             
             # Always delete RSVP on PARTSTAT change
             if partstat_changed:
                 try:
-                    del existing_attendee.params()["RSVP"]
+                    existing_attendee.removeParameter("RSVP")
                 except KeyError:
                     pass
 
@@ -388,12 +404,8 @@ class iTipProcessing(object):
                 # Look for matching X-CALENDARSERVER-ATTENDEE-COMMENT property in existing data (State 2 in spec)
                 private_comments = tuple(to_component.properties("X-CALENDARSERVER-ATTENDEE-COMMENT"))
                 for comment in private_comments:
-                    params = comment.params()["X-CALENDARSERVER-ATTENDEE-REF"]
-                    if len(params) != 1:
-                        log.error("Must be one and only one X-CALENDARSERVER-ATTENDEE-REF parameter in X-CALENDARSERVER-ATTENDEE-COMMENT")
-                        params = (None,)
-                    param = params[0]
-                    if param == attendee.value():
+                    attendeeref = comment.parameterValue("X-CALENDARSERVER-ATTENDEE-REF")
+                    if attendeeref == attendee.value():
                         private_comment = comment
                         break
                 else:
@@ -409,11 +421,11 @@ class iTipProcessing(object):
  
             elif attendee_comment is None and private_comment is not None:
                 # Remove all property parameters
-                private_comment.params().clear()
+                private_comment.removeAllParameters()
                 
                 # Add default parameters
-                private_comment.params()["X-CALENDARSERVER-ATTENDEE-REF"] = [attendee.value()]
-                private_comment.params()["X-CALENDARSERVER-DTSTAMP"] = [dateTimeToString(datetime.datetime.now(tz=utc))]
+                private_comment.setParameter("X-CALENDARSERVER-ATTENDEE-REF", attendee.value())
+                private_comment.setParameter("X-CALENDARSERVER-DTSTAMP", PyCalendarDateTime.getNowUTC().getText())
                 
                 # Set value empty
                 private_comment.setValue("")
@@ -427,8 +439,8 @@ class iTipProcessing(object):
                     "X-CALENDARSERVER-ATTENDEE-COMMENT",
                     attendee_comment.value(),
                     params = {
-                        "X-CALENDARSERVER-ATTENDEE-REF":     [attendee.value()],
-                        "X-CALENDARSERVER-DTSTAMP": [dateTimeToString(datetime.datetime.now(tz=utc))],
+                        "X-CALENDARSERVER-ATTENDEE-REF": attendee.value(),
+                        "X-CALENDARSERVER-DTSTAMP":      PyCalendarDateTime.getNowUTC().getText(),
                     }
                 )
                 to_component.addProperty(private_comment)
@@ -439,11 +451,11 @@ class iTipProcessing(object):
                 # Only change if different
                 if private_comment.value() != attendee_comment.value():
                     # Remove all property parameters
-                    private_comment.params().clear()
+                    private_comment.removeAllParameters()
                     
                     # Add default parameters
-                    private_comment.params()["X-CALENDARSERVER-ATTENDEE-REF"] = [attendee.value()]
-                    private_comment.params()["X-CALENDARSERVER-DTSTAMP"] = [dateTimeToString(datetime.datetime.now(tz=utc))]
+                    private_comment.setParameter("X-CALENDARSERVER-ATTENDEE-REF", attendee.value())
+                    private_comment.setParameter("X-CALENDARSERVER-DTSTAMP", PyCalendarDateTime.getNowUTC().getText())
                     
                     # Set new value
                     private_comment.setValue(attendee_comment.value())
@@ -453,7 +465,12 @@ class iTipProcessing(object):
         return attendee.value(), partstat_changed, private_comment_changed
 
     @staticmethod
-    def transferItems(from_calendar, master_valarms, private_comments, transps, organizer_schedule_status, to_component, remove_matched=False):
+    def transferItems(from_calendar, master_valarms, private_comments, transps, completeds, organizer_schedule_status, to_component, remove_matched=False):
+        """
+        Transfer properties from a calendar to a component by first trying to match the component in the original calendar and
+        use the properties from that, or use the values provided as arguments (which have been derived from the original calendar's
+        master component).
+        """
 
         rid = to_component.getRecurrenceIDUTC()
 
@@ -464,13 +481,14 @@ class iTipProcessing(object):
             [to_component.addComponent(comp) for comp in matched.subcomponents() if comp.name() == "VALARM"]
             [to_component.addProperty(prop) for prop in matched.properties("X-CALENDARSERVER-ATTENDEE-COMMENT")]
             [to_component.replaceProperty(prop) for prop in matched.properties("TRANSP")]
+            [to_component.replaceProperty(prop) for prop in matched.properties("COMPLETED")]
 
             organizer = matched.getProperty("ORGANIZER")
-            organizer_schedule_status = organizer.params().get("SCHEDULE-STATUS", None) if organizer else None
+            organizer_schedule_status = organizer.parameterValue("SCHEDULE-STATUS", None) if organizer else None
             if organizer_schedule_status: 
                 organizer = to_component.getProperty("ORGANIZER")
                 if organizer:
-                    organizer.params()["SCHEDULE-STATUS"] = organizer_schedule_status
+                    organizer.setParameter("SCHEDULE-STATUS", organizer_schedule_status)
 
             # Remove the old one
             if remove_matched:
@@ -482,31 +500,118 @@ class iTipProcessing(object):
             [to_component.addComponent(alarm) for alarm in master_valarms]
             [to_component.addProperty(comment) for comment in private_comments]
             [to_component.replaceProperty(transp) for transp in transps]
+            [to_component.replaceProperty(completed) for completed in completeds]
             if organizer_schedule_status: 
                 organizer = to_component.getProperty("ORGANIZER")
                 if organizer:
-                    organizer.params()["SCHEDULE-STATUS"] = organizer_schedule_status
+                    organizer.setParameter("SCHEDULE-STATUS", organizer_schedule_status)
     
     @staticmethod
     def fixForiCal3(components, recipient, compatibilityMode):
         # For each component where the ATTENDEE property of the recipient has PARTSTAT
         # NEEDS-ACTION we need to add X-APPLE-NEEDS-REPLY:TRUE
-        # We also add TRANSP:TRANSPARENT
+        # We also add TRANSP:TRANSPARENT for VEVENTs
         for component in components:
             if component.name() == "VTIMEZONE":
                 continue
             attendee = component.getAttendeeProperty((recipient,))
             if attendee:
-                partstat = attendee.params().get("PARTSTAT", ("NEEDS-ACTION",))[0]
+                partstat = attendee.parameterValue("PARTSTAT", "NEEDS-ACTION")
                 if partstat == "NEEDS-ACTION":
                     if compatibilityMode:
                         component.addProperty(Property("X-APPLE-NEEDS-REPLY", "TRUE"))
-                    component.replaceProperty(Property("TRANSP", "TRANSPARENT"))
+                    if component.name() == "VEVENT":
+                        component.replaceProperty(Property("TRANSP", "TRANSPARENT"))
 
+    @staticmethod
+    def sequenceComparison(itip, calendar):
+        """
+        Do appropriate itip message sequencing based by comparison with existing calendar data.
+        
+        @return: C{True} if the itip message is new and should be processed, C{False}
+            if no processing is needed
+        @rtype: C{bool}
+        """
+        
+        # Master component comparison trumps all else
+        itip_master = itip.masterComponent()
+        cal_master = calendar.masterComponent()
+        
+        # If master component exists, compare all in iTIP and update if any are new
+        if cal_master:
+            for itip_component in itip.subcomponents():
+                if itip_component.name() in ignoredComponents:
+                    continue
+                cal_component = calendar.overriddenComponent(itip_component.getRecurrenceIDUTC())
+                if cal_component is None:
+                    cal_component = cal_master
+                    
+                # TODO: No DTSTAMP comparison because we do not track DTSTAMPs
+                # Treat components the same as meaning so an update - in theory no harm in doing that
+                if Component.compareComponentsForITIP(itip_component, cal_component, use_dtstamp=False) >= 0:
+                    return True
+
+            return False
+
+        elif itip_master:
+            
+            # Do comparison of each appropriate component if any one is new, process the itip
+            for cal_component in calendar.subcomponents():
+                if cal_component.name() in ignoredComponents:
+                    continue
+                itip_component = itip.overriddenComponent(cal_component.getRecurrenceIDUTC())
+                if itip_component is None:
+                    itip_component = itip_master
+
+                # TODO: No DTSTAMP comparison because we do not track DTSTAMPs
+                # Treat components the same as meaning so an update - in theory no harm in doing that
+                if Component.compareComponentsForITIP(itip_component, cal_component, use_dtstamp=False) >= 0:
+                    return True
+            
+            return False
+    
+        else:
+            # Do comparison of each matching component if any one is new, process the entire itip.
+            # There is a race condition here, similar to REPLY, where we could reinstate an instance
+            # that has been removed. Not much we can do about it without additional tracking.
+            
+            cal_rids = set()
+            for cal_component in calendar.subcomponents():
+                if cal_component.name() in ignoredComponents:
+                    continue
+                cal_rids.add(cal_component.getRecurrenceIDUTC())
+            itip_rids = set()
+            for itip_component in itip.subcomponents():
+                if itip_component.name() in ignoredComponents:
+                    continue
+                itip_rids.add(itip_component.getRecurrenceIDUTC())
+            
+            # Compare ones that match
+            for rid in cal_rids & itip_rids:
+                cal_component = calendar.overriddenComponent(rid)
+                itip_component = itip.overriddenComponent(rid)
+
+                # TODO: No DTSTAMP comparison because we do not track DTSTAMPs
+                # Treat components the same as meaning so an update - in theory no harm in doing that
+                if Component.compareComponentsForITIP(itip_component, cal_component, use_dtstamp=False) >= 0:
+                    return True
+            
+            # If there are others in one set and not the other - always process, else no process
+            return len(cal_rids ^ itip_rids) > 0
+            
 class iTipGenerator(object):
+    """
+    This assumes that DTSTAMP and SEQUENCE are already at their new values in the original calendar
+    data passed in to each generateXXX() call.
+    """
     
     @staticmethod
     def generateCancel(original, attendees, instances=None, full_cancel=False):
+        """
+        This assumes that SEQUENCE is not already at its new value in the original calendar data. This
+        is because the component passed in is the one that originally contained the attendee that is
+        being removed. 
+        """
         
         itip = Component("VCALENDAR")
         itip.addProperty(Property("VERSION", "2.0"))
@@ -521,7 +626,6 @@ class iTipGenerator(object):
             
             # Create a new component matching the type of the original
             comp = Component(original.mainType())
-            itip.addComponent(comp)
 
             # Use the master component when the instance is None
             if not instance_rid:
@@ -533,18 +637,18 @@ class iTipGenerator(object):
             assert instance is not None, "Need a master component"
 
             # Add some required properties extracted from the original
-            comp.addProperty(Property("DTSTAMP", datetime.datetime.now(tz=utc)))
+            comp.addProperty(Property("DTSTAMP", instance.propertyValue("DTSTAMP")))
             comp.addProperty(Property("UID", instance.propertyValue("UID")))
             seq = instance.propertyValue("SEQUENCE")
-            seq = str(int(seq) + 1) if seq else "1"
+            seq = int(seq) + 1 if seq else 1
             comp.addProperty(Property("SEQUENCE", seq))
             comp.addProperty(instance.getOrganizerProperty())
             if instance_rid:
-                comp.addProperty(Property("RECURRENCE-ID", normalizeToUTC(instance_rid)))
+                comp.addProperty(Property("RECURRENCE-ID", instance_rid.duplicate().adjustToUTC()))
             
             def addProperties(propname):
-                for property in instance.properties(propname):
-                    comp.addProperty(property)
+                for icalproperty in instance.properties(propname):
+                    comp.addProperty(icalproperty)
                     
             addProperties("SUMMARY")
             addProperties("DTSTART")
@@ -565,6 +669,8 @@ class iTipGenerator(object):
                 comp.addProperty(attendeeProp)
 
             tzids.update(comp.timezoneIDs())
+
+            itip.addComponent(comp)
             
         # Now include any referenced tzids
         for comp in original.subcomponents():
@@ -580,24 +686,22 @@ class iTipGenerator(object):
 
     @staticmethod
     def generateAttendeeRequest(original, attendees, filter_rids):
-
+        """
+        This assumes that SEQUENCE is already at its new value in the original calendar data.
+        """
+        
         # Start with a copy of the original as we may have to modify bits of it
         itip = original.duplicate()
         itip.replaceProperty(Property("PRODID", iCalendarProductID))
         itip.addProperty(Property("METHOD", "REQUEST"))
         
-        # Force update to DTSTAMP everywhere
-        itip.replacePropertyInAllComponents(Property("DTSTAMP", datetime.datetime.now(tz=utc)))
-
         # Now filter out components that do not contain every attendee
         itip.attendeesView(attendees, onlyScheduleAgentServer=True)
         
         # Now filter out components except the ones specified
         if itip.filterComponents(filter_rids):
-
             # Strip out unwanted bits
             iTipGenerator.prepareSchedulingMessage(itip)
-    
             return itip
         
         else:
@@ -614,8 +718,8 @@ class iTipGenerator(object):
         # Now filter out components except the ones specified
         itip.filterComponents(changedRids)
 
-        # Force update to DTSTAMP everywhere
-        itip.replacePropertyInAllComponents(Property("DTSTAMP", datetime.datetime.now(tz=utc)))
+        # Force update to DTSTAMP everywhere so reply sequencing will work
+        itip.replacePropertyInAllComponents(Property("DTSTAMP", PyCalendarDateTime.getNowUTC()))
 
         # Remove all attendees except the one we want
         itip.removeAllButOneAttendee(attendee)
@@ -639,6 +743,9 @@ class iTipGenerator(object):
             "ORGANIZER",
             "ATTENDEE",
             "X-CALENDARSERVER-PRIVATE-COMMENT",
+            "SUMMARY",
+            "LOCATION",
+            "DESCRIPTION",
         ))
         
         # Now set each ATTENDEE's PARTSTAT to DECLINED
@@ -646,11 +753,11 @@ class iTipGenerator(object):
             attendeeProps = itip.getAttendeeProperties((attendee,))
             assert attendeeProps, "Must have some matching ATTENDEEs"
             for attendeeProp in attendeeProps:
-                attendeeProp.params().setdefault("PARTSTAT", ["DECLINED",])[0] = "DECLINED"
+                attendeeProp.setParameter("PARTSTAT", "DECLINED")
         
         # Add REQUEST-STATUS to each top-level component
         itip.addPropertyToAllComponents(Property("REQUEST-STATUS", ["2.0", "Success",]))
-        
+
         # Strip out unwanted bits
         iTipGenerator.prepareSchedulingMessage(itip, reply=True)
 
@@ -662,12 +769,15 @@ class iTipGenerator(object):
         Remove properties and parameters that should not be sent in an iTIP message
         """
 
+        # All X- components go away
+        itip.removeXComponents()
+
         # Alarms
         itip.removeAlarms()
 
         # Top-level properties - remove all X-
         itip.removeXProperties(do_subcomponents=False)
-                
+
         # Component properties - remove all X- except for those specified
         if not reply:
             # Organizer properties that need to go to the Attendees
