@@ -1,5 +1,5 @@
 #
-# Copyright (c) 2005-2009 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2011 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,25 +14,30 @@
 # limitations under the License.
 ##
 
+import time
 from hashlib import md5
+
+from twisted.python.log import err as log_traceback
+from twext.python.log import Logger
+
 from twisted.internet import reactor
 from twisted.internet.defer import inlineCallbacks, returnValue
-from twisted.web2.dav.method.report import NumberOfMatchesWithinLimits
-from twisted.web2.dav.util import joinURL
-from twisted.web2.http import HTTPError
+from twext.web2.dav.method.report import NumberOfMatchesWithinLimits
+from twext.web2.dav.util import joinURL
+from twext.web2.http import HTTPError
 from twistedcaldav import customxml, caldavxml
 from twistedcaldav.caldavxml import caldav_namespace
+from twistedcaldav.config import config
 from twistedcaldav.ical import Property
 from twistedcaldav.instance import InvalidOverriddenInstanceError
-from twistedcaldav.log import Logger
-from twistedcaldav.memcachelock import MemcacheLock, MemcacheLockTimeoutError
 from twistedcaldav.method import report_common
 from twistedcaldav.scheduling.cuaddress import normalizeCUAddr
 from twistedcaldav.scheduling.itip import iTipProcessing, iTIPRequestStatus
 from twistedcaldav.scheduling.utils import getCalendarObjectForPrincipals
-from vobject.icalendar import utc
-import datetime
-import time
+from twistedcaldav.memcachelock import MemcacheLock, MemcacheLockTimeoutError
+from pycalendar.duration import PyCalendarDuration
+from pycalendar.datetime import PyCalendarDateTime
+from pycalendar.timezone import PyCalendarTimezone
 
 __all__ = [
     "ImplicitProcessor",
@@ -91,6 +96,7 @@ class ImplicitProcessor(object):
                 # We attempt to recover from this. That involves trying to re-write the attendee data
                 # to match that of the organizer assuming we have the organizer's full data available, then
                 # we try the processing operation again.
+                log_traceback()
                 log.error("ImplicitProcessing - originator '%s' to recipient '%s' with UID: '%s' - exception raised will try to fix: %s" % (self.originator.cuaddr, self.recipient.cuaddr, self.uid, e))
                 result = (yield self.doImplicitAttendeeEventFix(e))
                 if result:
@@ -98,6 +104,7 @@ class ImplicitProcessor(object):
                     try:
                         result = (yield self.doImplicitAttendee())
                     except Exception, e:
+                        log_traceback()
                         log.error("ImplicitProcessing - originator '%s' to recipient '%s' with UID: '%s' - exception raised after fix: %s" % (self.originator.cuaddr, self.recipient.cuaddr, self.uid, e))
                         raise ImplicitProcessorException("5.1;Service unavailable")
                 else:
@@ -133,7 +140,7 @@ class ImplicitProcessor(object):
         self.recipient_calendar_name = None
         calendar_resource, resource_name, calendar_collection, calendar_collection_uri = (yield getCalendarObjectForPrincipals(self.request, self.recipient.principal, self.uid))
         if calendar_resource:
-            self.recipient_calendar = calendar_resource.iCalendar()
+            self.recipient_calendar = (yield calendar_resource.iCalendarForUser(self.request))
             self.recipient_calendar_collection = calendar_collection
             self.recipient_calendar_collection_uri = calendar_collection_uri
             self.recipient_calendar_name = resource_name
@@ -145,7 +152,7 @@ class ImplicitProcessor(object):
         yield self.getRecipientsCopy()
         if self.recipient_calendar is None:
             log.debug("ImplicitProcessing - originator '%s' to recipient '%s' ignoring UID: '%s' - organizer has no copy" % (self.originator.cuaddr, self.recipient.cuaddr, self.uid))
-            returnValue((True, True, None,))
+            returnValue((True, True, False, None,))
 
         # Handle new items differently than existing ones.
         if self.method == "REPLY":
@@ -153,7 +160,7 @@ class ImplicitProcessor(object):
         elif self.method == "REFRESH":
             # With implicit we ignore refreshes.
             # TODO: for iMIP etc we do need to handle them 
-            result = (True, True, None,)
+            result = (True, True, False, None,)
 
         returnValue(result)
 
@@ -164,7 +171,7 @@ class ImplicitProcessor(object):
         result, processed = iTipProcessing.processReply(self.message, self.recipient_calendar)
         if result:
  
-            # Update the attendee's copy of the event
+            # Update the organizer's copy of the event
             log.debug("ImplicitProcessing - originator '%s' to recipient '%s' processing METHOD:REPLY, UID: '%s' - updating event" % (self.originator.cuaddr, self.recipient.cuaddr, self.uid))
             recipient_calendar_resource = (yield self.writeCalendarResource(self.recipient_calendar_collection_uri, self.recipient_calendar_collection, self.recipient_calendar_name, self.recipient_calendar))
             
@@ -197,29 +204,94 @@ class ImplicitProcessor(object):
 
             # Only update other attendees when the partstat was changed by the reply
             if partstatChanged:
-                yield self.updateAllAttendeesExceptSome(recipient_calendar_resource, (attendeeReplying,))
+                yield self.queueAttendeeUpdate(recipient_calendar_resource, (attendeeReplying,))
 
-            result = (True, False, changes,)
+            result = (True, False, True, changes,)
 
         else:
             # Ignore scheduling message
-            result = (True, True, None,)
+            result = (True, True, False, None,)
 
         returnValue(result)
 
     @inlineCallbacks
-    def updateAllAttendeesExceptSome(self, resource, attendees):
+    def queueAttendeeUpdate(self, resource, attendees):
         """
-        Send an update out to all attendees except the specified ones, to refresh the others due to a change
-        by that one.
-        
-        @param attendee: cu-addresses of attendees not to send to
-        @type attendee: C{set}
+        Queue up an update to attendees and use a memcache lock to ensure we don't update too frequently.
         """
         
-        from twistedcaldav.scheduling.implicit import ImplicitScheduler
-        scheduler = ImplicitScheduler()
-        yield scheduler.refreshAllAttendeesExceptSome(self.request, resource, self.recipient_calendar, attendees)
+        # When doing auto-processing of replies, only refresh attendees when the last auto-accept is done.
+        # Note that when we do this we also need to refresh the attendee that is generating the reply because they
+        # are no longer up to date with changes of other auto-accept attendees.
+        if hasattr(self.request, "auto_reply_processing_count") and self.request.auto_reply_processing_count > 1:
+            self.request.auto_reply_suppressed = True
+            returnValue(None)
+        if hasattr(self.request, "auto_reply_suppressed"):
+            attendees = ()
+
+        # Use a memcachelock to ensure others don't refresh whilst we have an enqueued call
+        self.uid = self.recipient_calendar.resourceUID()
+        if config.Scheduling.Options.AttendeeRefreshInterval:
+            attendees = ()
+            lock = MemcacheLock("RefreshUIDLock", self.uid, timeout=0.0, expire_time=config.Scheduling.Options.AttendeeRefreshInterval)
+            
+            # Try lock, but fail immediately if already taken
+            try:
+                yield lock.acquire()
+            except MemcacheLockTimeoutError:
+                returnValue(None)
+        else:
+            lock = None
+
+        @inlineCallbacks
+        def _doRefresh(organizer_resource):
+            log.debug("ImplicitProcessing - refreshing UID: '%s'" % (self.uid,))
+            from twistedcaldav.scheduling.implicit import ImplicitScheduler
+            scheduler = ImplicitScheduler()
+            yield scheduler.refreshAllAttendeesExceptSome(self.request, organizer_resource, self.recipient_calendar, attendees)
+
+        @inlineCallbacks
+        def _doDelayedRefresh():
+
+            # We need to get the UID lock for implicit processing whilst we send the auto-reply
+            # as the Organizer processing will attempt to write out data to other attendees to
+            # refresh them. To prevent a race we need a lock.
+            uidlock = MemcacheLock("ImplicitUIDLock", self.uid, timeout=60.0, expire_time=5*60)
+    
+            try:
+                yield uidlock.acquire()
+            except MemcacheLockTimeoutError:
+                # Just try again to get the lock
+                reactor.callLater(2.0, _doDelayedRefresh)
+            else:
+                
+                # Release lock before sending refresh
+                yield lock.release()
+    
+                # inNewTransaction wipes out the remembered resource<-> URL mappings in the
+                # request object but we need to be able to map the actual reply resource to its
+                # URL when doing auto-processing, so we have to sneak that mapping back in here.
+                txn = yield resource.inNewTransaction(self.request)
+                organizer_resource = (yield self.request.locateResource(resource._url))
+    
+                try:
+                    if organizer_resource.exists():
+                        yield _doRefresh(organizer_resource)
+                    else:
+                        log.debug("ImplicitProcessing - skipping refresh of missing UID: '%s'" % (self.uid,))
+                except Exception, e:
+                    log.debug("ImplicitProcessing - refresh exception UID: '%s', %s" % (self.uid, str(e)))
+                    yield txn.abort()
+                else:
+                    yield txn.commit()
+            finally:
+                # This correctly gets called only after commit or abort is done
+                yield uidlock.clean()
+
+        if lock:
+            reactor.callLater(config.Scheduling.Options.AttendeeRefreshInterval, _doDelayedRefresh)
+        else:
+            yield _doRefresh(resource)
 
     @inlineCallbacks
     def doImplicitAttendee(self):
@@ -230,7 +302,7 @@ class ImplicitProcessor(object):
 
         # Handle new items differently than existing ones.
         if self.new_resource and self.method == "CANCEL":
-            result = (True, True, None)
+            result = (True, True, False, None)
         else:
             result = (yield self.doImplicitAttendeeUpdate())
         
@@ -256,10 +328,10 @@ class ImplicitProcessor(object):
             result = (yield self.doImplicitAttendeeCancel())
         elif self.method == "ADD":
             # TODO: implement ADD
-            result = (False, False, None)
+            result = (False, False, False, None)
         else:
             # NB We should never get here as we will have rejected unsupported METHODs earlier.
-            result = (True, True, None,)
+            result = (True, True, False, None,)
             
         returnValue(result)
 
@@ -283,9 +355,10 @@ class ImplicitProcessor(object):
                 raise ImplicitProcessorException(iTIPRequestStatus.NO_USER_SUPPORT)
 
             log.debug("ImplicitProcessing - originator '%s' to recipient '%s' processing METHOD:REQUEST, UID: '%s' - new processed" % (self.originator.cuaddr, self.recipient.cuaddr, self.uid))
-            autoprocessed = (yield self.recipient.principal.getAutoSchedule())
+            autoprocessed = self.recipient.principal.getAutoSchedule()
+            store_inbox = not autoprocessed or self.recipient.principal.getCUType() == "INDIVIDUAL"
             new_calendar = iTipProcessing.processNewRequest(self.message, self.recipient.cuaddr, autoprocessing=autoprocessed)
-            name =  md5(str(new_calendar) + str(time.time()) + default.fp.path).hexdigest() + ".ics"
+            name =  md5(str(new_calendar) + str(time.time()) + defaultURL).hexdigest() + ".ics"
             
             # Handle auto-reply behavior
             if autoprocessed:
@@ -294,6 +367,11 @@ class ImplicitProcessor(object):
             new_resource = (yield self.writeCalendarResource(defaultURL, default, name, new_calendar))
             
             if autoprocessed and send_reply:
+                # Track outstanding auto-reply processing
+                if not hasattr(self.request, "auto_reply_processing_count"):
+                    self.request.auto_reply_processing_count = 1
+                else:
+                    self.request.auto_reply_processing_count += 1
                 reactor.callLater(2.0, self.sendAttendeeAutoReply, *(new_calendar, new_resource, partstat))
 
             # Build the schedule-changes XML element
@@ -303,10 +381,11 @@ class ImplicitProcessor(object):
                     customxml.Create(),
                 ),
             )
-            result = (True, autoprocessed, changes,)
+            result = (True, autoprocessed, store_inbox, changes,)
         else:
             # Processing update to existing event
-            autoprocessed = (yield self.recipient.principal.getAutoSchedule())
+            autoprocessed = self.recipient.principal.getAutoSchedule()
+            store_inbox = not autoprocessed or self.recipient.principal.getCUType() == "INDIVIDUAL"
             new_calendar, rids = iTipProcessing.processRequest(self.message, self.recipient_calendar, self.recipient.cuaddr, autoprocessing=autoprocessed)
             if new_calendar:
      
@@ -319,6 +398,11 @@ class ImplicitProcessor(object):
                 new_resource = (yield self.writeCalendarResource(self.recipient_calendar_collection_uri, self.recipient_calendar_collection, self.recipient_calendar_name, new_calendar))
                 
                 if autoprocessed and send_reply:
+                    # Track outstanding auto-reply processing
+                    if not hasattr(self.request, "auto_reply_processing_count"):
+                        self.request.auto_reply_processing_count = 1
+                    else:
+                        self.request.auto_reply_processing_count += 1
                     reactor.callLater(2.0, self.sendAttendeeAutoReply, *(new_calendar, new_resource, partstat))
 
                 # Build the schedule-changes XML element
@@ -346,13 +430,14 @@ class ImplicitProcessor(object):
                 # Refresh from another Attendee should not have Inbox item
                 if hasattr(self.request, "doing_attendee_refresh"):
                     autoprocessed = True
+                    store_inbox = False
 
-                result = (True, autoprocessed, changes,)
+                result = (True, autoprocessed, store_inbox, changes,)
                 
             else:
                 # Request needs to be ignored
                 log.debug("ImplicitProcessing - originator '%s' to recipient '%s' processing METHOD:REQUEST, UID: '%s' - ignoring" % (self.originator.cuaddr, self.recipient.cuaddr, self.uid))
-                result = (True, True, None,)
+                result = (True, True, False, None,)
 
         returnValue(result)
 
@@ -366,8 +451,11 @@ class ImplicitProcessor(object):
             result = (True, True, None)
         else:
             # Need to check for auto-respond attendees. These need to suppress the inbox message
-            # if the cancel is processed.
-            autoprocessed = (yield self.recipient.principal.getAutoSchedule())
+            # if the cancel is processed. However, if the principal is a user we always force the
+            # inbox item on them even if auto-schedule is true so that they get a notification
+            # of the cancel.
+            autoprocessed = self.recipient.principal.getAutoSchedule() and self.recipient.principal.getCUType() != "INDIVIDUAL"
+            store_inbox = not autoprocessed or self.recipient.principal.getCUType() == "INDIVIDUAL"
 
             # Check to see if this is a cancel of the entire event
             processed_message, delete_original, rids = iTipProcessing.processCancel(self.message, self.recipient_calendar, autoprocessing=autoprocessed)
@@ -385,7 +473,7 @@ class ImplicitProcessor(object):
                             customxml.Cancel(),
                         ),
                     )
-                    result = (True, autoprocessed, changes,)
+                    result = (True, autoprocessed, store_inbox, changes,)
                     
                 else:
          
@@ -404,10 +492,10 @@ class ImplicitProcessor(object):
                         customxml.DTStamp(),
                         customxml.Action(action),
                     )
-                    result = (True, autoprocessed, changes)
+                    result = (True, autoprocessed, store_inbox, changes)
             else:
                 log.debug("ImplicitProcessing - originator '%s' to recipient '%s' processing METHOD:CANCEL, UID: '%s' - ignoring" % (self.originator.cuaddr, self.recipient.cuaddr, self.uid))
-                result = (True, True, None)
+                result = (True, True, False, None)
 
         returnValue(result)
 
@@ -419,32 +507,49 @@ class ImplicitProcessor(object):
 
         @param calendar: calendar data to examine
         @type calendar: L{Component}
-        
+
         @return: L{Component} for the new calendar data to write
         """
         
         # We need to get the UID lock for implicit processing whilst we send the auto-reply
         # as the Organizer processing will attempt to write out data to other attendees to
         # refresh them. To prevent a race we need a lock.
-        lock = MemcacheLock("ImplicitUIDLock", calendar.resourceUID(), timeout=60.0)
+        lock = MemcacheLock("ImplicitUIDLock", calendar.resourceUID(), timeout=60.0, expire_time=5*60)
 
+        # Note that this lock also protects the request, as this request is
+        # being re-used by potentially multiple transactions and should not be
+        # used concurrency (the locateResource cache needs to be cleared each
+        # time, by inNewTransaction). -glyph
         try:
             yield lock.acquire()
-
-            # Send out a reply
-            log.debug("ImplicitProcessing - recipient '%s' processing UID: '%s' - auto-reply: %s" % (self.recipient.cuaddr, self.uid, partstat))
-            from twistedcaldav.scheduling.implicit import ImplicitScheduler
-            scheduler = ImplicitScheduler()
-            yield scheduler.sendAttendeeReply(self.request, resource, calendar, self.recipient)
-
         except MemcacheLockTimeoutError:
-            
             # Just try again to get the lock
             reactor.callLater(2.0, self.sendAttendeeAutoReply, *(calendar, resource, partstat))
-    
+        else:
+            # inNewTransaction wipes out the remembered resource<-> URL mappings in the
+            # request object but we need to be able to map the actual reply resource to its
+            # URL when doing auto-processing, so we have to sneak that mapping back in here.
+            txn = yield resource.inNewTransaction(self.request)
+            self.request._rememberResource(resource, resource._url)
+
+            try:
+                # Send out a reply
+                log.debug("ImplicitProcessing - recipient '%s' processing UID: '%s' - auto-reply: %s" % (self.recipient.cuaddr, self.uid, partstat))
+                from twistedcaldav.scheduling.implicit import ImplicitScheduler
+                scheduler = ImplicitScheduler()
+                yield scheduler.sendAttendeeReply(self.request, resource, calendar, self.recipient)
+            except:
+                yield txn.abort()
+            else:
+                yield txn.commit()
         finally:
+            # This correctly gets called only after commit or abort is done
             yield lock.clean()
 
+            # Track outstanding auto-reply processing
+            if hasattr(self.request, "auto_reply_processing_count"):
+                self.request.auto_reply_processing_count -= 1
+
     @inlineCallbacks
     def checkAttendeeAutoReply(self, calendar):
         """
@@ -462,8 +567,8 @@ class ImplicitProcessor(object):
         log.debug("ImplicitProcessing - recipient '%s' processing UID: '%s' - checking for auto-reply" % (self.recipient.cuaddr, self.uid))
 
         # First expand current one to get instances (only go 1 year into the future)
-        default_future_expansion_duration = datetime.timedelta(days=356*1)
-        expand_max = datetime.date.today() + default_future_expansion_duration
+        default_future_expansion_duration = PyCalendarDuration(days=356*1)
+        expand_max = PyCalendarDateTime.getToday() + default_future_expansion_duration
         instances = calendar.expandTimeRanges(expand_max, ignoreInvalidInstances=True)
         instance_states = dict([(instance, True) for instance in instances.instances.itervalues()])
         
@@ -483,9 +588,9 @@ class ImplicitProcessor(object):
             has_prop = (yield testcal.hasProperty((caldav_namespace, "calendar-timezone"), self.request))
             if has_prop:
                 tz = (yield testcal.readProperty((caldav_namespace, "calendar-timezone"), self.request))
-                tzinfo = tz.calendar().gettzinfo()
+                tzinfo = tz.calendar().gettimezone()
             else:
-                tzinfo = utc
+                tzinfo = PyCalendarTimezone(utc=True)
 
             # Now do search for overlapping time-range
             for instance in instances.instances.itervalues():
@@ -495,17 +600,21 @@ class ImplicitProcessor(object):
                         fbinfo = ([], [], [])
                         
                         def makeTimedUTC(dt):
-                            if isinstance(dt, datetime.date) and not isinstance(dt, datetime.datetime):
-                                dt = datetime.datetime.fromordinal(dt.toordinal())
-                            if dt.tzinfo is None:
-                                dt = dt.replace(tzinfo=tzinfo).astimezone(utc)
+                            dt = dt.duplicate()
+                            if dt.isDateOnly():
+                                dt.setDateOnly(False)
+                                dt.setHHMMSS(0, 0, 0)
+                            if dt.floating():
+                                dt.setTimezone(tzinfo)
+                                dt.adjustToUTC()
                             return dt
                         
-                        tr = caldavxml.TimeRange(start="20000101", end="20000101")
-                        tr.start = makeTimedUTC(instance.start)
-                        tr.end = makeTimedUTC(instance.end)
+                        tr = caldavxml.TimeRange(
+                            start=str(makeTimedUTC(instance.start)),
+                            end=str(makeTimedUTC(instance.end)),
+                        )
 
-                        yield report_common.generateFreeBusyInfo(self.request, testcal, fbinfo, tr, 0, uid)
+                        yield report_common.generateFreeBusyInfo(self.request, testcal, fbinfo, tr, 0, uid, servertoserver=True)
                         
                         # If any fbinfo entries exist we have an overlap
                         if len(fbinfo[0]) or len(fbinfo[1]) or len(fbinfo[2]):
@@ -605,19 +714,20 @@ class ImplicitProcessor(object):
         resource or by creating a new one.
         
         @param collURL: the C{str} containing the URL of the calendar collection.
-        @param collection: the L{CalDAVFile} for the calendar collection to store the resource in.
+        @param collection: the L{CalDAVResource} for the calendar collection to store the resource in.
         @param name: the C{str} for the resource name to write into, or {None} to write a new resource.
         @param calendar: the L{Component} calendar to write.
-        @return: L{Deferred} -> L{CalDAVFile}
+        @return: L{Deferred} -> L{CalDAVResource}
         """
         
         # Create a new name if one was not provided
         if name is None:
-            name =  md5(str(calendar) + str(time.time()) + collection.fp.path).hexdigest() + ".ics"
+            name =  md5(str(calendar) + str(time.time()) + collURL).hexdigest() + ".ics"
     
         # Get a resource for the new item
         newchildURL = joinURL(collURL, name)
         newchild = yield self.request.locateResource(newchildURL)
+        newchild._url = newchildURL
         
         # Now write it to the resource
         from twistedcaldav.method.put_common import StoreCalendarObjectResource
@@ -636,6 +746,7 @@ class ImplicitProcessor(object):
     
         returnValue(newchild)
 
+
     @inlineCallbacks
     def deleteCalendarResource(self, collURL, collection, name):
         """
@@ -644,18 +755,15 @@ class ImplicitProcessor(object):
         @param collURL: the URL of the calendar collection.
         @type name: C{str}
         @param collection: the calendar collection to delete the resource from.
-        @type collection: L{CalDAVFile}
+        @type collection: L{CalDAVResource}
         @param name: the resource name to write into, or {None} to write a new resource.
         @type name: C{str}
         """
-        
-        from twistedcaldav.method.delete_common import DeleteResource
-        delchild = collection.getChild(name)
+        delchild = yield collection.getChild(name)
         childURL = joinURL(collURL, name)
         self.request._rememberResource(delchild, childURL)
+        yield delchild.storeRemove(self.request, False, childURL)
 
-        deleter = DeleteResource(self.request, delchild, childURL, collection, "0", internal_request=True)
-        yield deleter.run()
 
     def changeAttendeePartstat(self, attendees, partstat):
         """
@@ -674,14 +782,14 @@ class ImplicitProcessor(object):
 
         madeChanges = False
         for attendee in attendees:
-            if attendee.params().get("PARTSTAT", ("NEEDS-ACTION",))[0] != partstat:
-                attendee.params()["PARTSTAT"] = [partstat]
+            if attendee.parameterValue("PARTSTAT", "NEEDS-ACTION") != partstat:
+                attendee.setParameter("PARTSTAT", partstat)
                 madeChanges = True
 
             # Always remove RSVP - this is only an attendee change so madeChanges
             # does not need to be changed
             try:
-                del attendee.params()["RSVP"]
+                attendee.removeParameter("RSVP")
             except KeyError:
                 pass
 
@@ -705,7 +813,7 @@ class ImplicitProcessor(object):
         calendar_resource, _ignore_name, _ignore_collection, _ignore_uri = (yield getCalendarObjectForPrincipals(self.request, self.originator.principal, self.uid))
         if not calendar_resource:
             raise ImplicitProcessorException("5.1;Service unavailable")
-        originator_calendar = calendar_resource.iCalendar()
+        originator_calendar = (yield calendar_resource.iCalendarForUser(self.request))
 
         # Get attendee's view of that
         originator_calendar.attendeesView((self.recipient.cuaddr,))
