@@ -17,9 +17,8 @@
 from twistedcaldav.ical import Component
 import twistedcaldav.test.util
 from twistedcaldav.scheduling.implicit import ImplicitScheduler
-from dateutil.tz import tzutc
-from twisted.web2.dav import davxml
-import datetime
+from pycalendar.datetime import PyCalendarDateTime
+from pycalendar.timezone import PyCalendarTimezone
 
 class Implicit (twistedcaldav.test.util.TestCase):
     """
@@ -190,9 +189,9 @@ END:VEVENT
 END:VCALENDAR
 """,
                 (
-                    ("mailto:user1@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user2@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user3@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
+                    ("mailto:user1@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user2@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user3@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
                 ),
             ),
             (
@@ -229,12 +228,12 @@ END:VEVENT
 END:VCALENDAR
 """,
                 (
-                    ("mailto:user1@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user2@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user3@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user1@example.com", datetime.datetime(2008, 9, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user2@example.com", datetime.datetime(2008, 9, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user3@example.com", datetime.datetime(2008, 9, 1, 12, 0, 0, tzinfo=tzutc())),
+                    ("mailto:user1@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user2@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user3@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user1@example.com", PyCalendarDateTime(2008, 9, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user2@example.com", PyCalendarDateTime(2008, 9, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user3@example.com", PyCalendarDateTime(2008, 9, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
                 ),
             ),
             (
@@ -272,15 +271,15 @@ END:VEVENT
 END:VCALENDAR
 """,
                 (
-                    ("mailto:user1@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user2@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user3@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user1@example.com", datetime.datetime(2008, 9, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user2@example.com", datetime.datetime(2008, 9, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user3@example.com", datetime.datetime(2008, 9, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user1@example.com", datetime.datetime(2008, 12, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user2@example.com", datetime.datetime(2008, 12, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user3@example.com", datetime.datetime(2008, 12, 1, 12, 0, 0, tzinfo=tzutc())),
+                    ("mailto:user1@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user2@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user3@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user1@example.com", PyCalendarDateTime(2008, 9, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user2@example.com", PyCalendarDateTime(2008, 9, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user3@example.com", PyCalendarDateTime(2008, 9, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user1@example.com", PyCalendarDateTime(2008, 12, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user2@example.com", PyCalendarDateTime(2008, 12, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user3@example.com", PyCalendarDateTime(2008, 12, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
                 ),
             ),
             (
@@ -389,7 +388,7 @@ END:VCALENDAR
 """,
                 (
                     ("mailto:user3@example.com", None),
-                    ("mailto:user3@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
+                    ("mailto:user3@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
                 ),
             ),
             (
@@ -444,7 +443,7 @@ END:VEVENT
 END:VCALENDAR
 """,
                 (
-                    ("mailto:user3@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
+                    ("mailto:user3@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
                 ),
             ),
             (
@@ -590,9 +589,9 @@ END:VEVENT
 END:VCALENDAR
 """,
                 (
-                    ("mailto:user1@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user2@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user3@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
+                    ("mailto:user1@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user2@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user3@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
                 ),
             ),
             (
@@ -647,7 +646,7 @@ END:VCALENDAR
 """,
                 (
                     ("mailto:user3@example.com", None),
-                    ("mailto:user4@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
+                    ("mailto:user4@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
                 ),
             ),
             (
@@ -693,7 +692,7 @@ END:VEVENT
 END:VCALENDAR
 """,
                 (
-                    ("mailto:user4@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
+                    ("mailto:user4@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
                 ),
             ),
             (
@@ -740,31 +739,21 @@ END:VEVENT
 END:VCALENDAR
 """,
                 (
-                    ("mailto:user1@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user2@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
-                    ("mailto:user4@example.com", datetime.datetime(2008, 8, 1, 12, 0, 0, tzinfo=tzutc())),
+                    ("mailto:user1@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user2@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
+                    ("mailto:user4@example.com", PyCalendarDateTime(2008, 8, 1, 12, 0, 0, tzid=PyCalendarTimezone(utc=True))),
                 ),
             ),
         )
 
-        class TestResource(object):
-            def currentPrincipal(self, request):
-                return davxml.Principal(davxml.Unauthenticated)
-
-        resource = TestResource()
-
         for description, calendar1, calendar2, result in data:
             scheduler = ImplicitScheduler()
-            scheduler.resource = resource
+            scheduler.resource = None
             scheduler.request = None
             scheduler.oldcalendar = Component.fromString(calendar1)
+            scheduler.oldAttendeesByInstance = scheduler.oldcalendar.getAttendeesByInstance(True, onlyScheduleAgentServer=True)
             scheduler.calendar = Component.fromString(calendar2)
             scheduler.extractCalendarData()
             scheduler.findRemovedAttendees()
-#            if not description.startswith("#4.3"):
-#                continue
-#            print description
-#            print scheduler.cancelledAttendees
-#            print set(result)
             self.assertEqual(scheduler.cancelledAttendees, set(result), msg=description)
 
