@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2007 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2011 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,12 +14,12 @@
 # limitations under the License.
 ##
 
-from dateutil.tz import tzutc
+from pycalendar.datetime import PyCalendarDateTime
+from pycalendar.timezone import PyCalendarTimezone
 from twistedcaldav.ical import Component
 from twistedcaldav.scheduling.itip import iTipProcessing, iTipGenerator
-import datetime
-import os
 import twistedcaldav.test.util
+import os
 
 class iTIPProcessing (twistedcaldav.test.util.TestCase):
     """
@@ -814,6 +814,48 @@ ATTENDEE;PARTSTAT=NEEDS-ACTION:mailto:user2@example.com
 ORGANIZER;CN=User 01:mailto:user1@example.com
 END:VEVENT
 END:VCALENDAR
+""",
+                True, "mailto:user1@example.com", (("", True, False),),
+            ),
+            (
+                "#6.3 Bad REQUEST-STATUS",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890
+DTSTART:20080601T120000Z
+DTEND:20080601T130000Z
+ORGANIZER;CN="User 01":mailto:user1@example.com
+ATTENDEE:mailto:user1@example.com
+ATTENDEE:mailto:user2@example.com
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+METHOD:REPLY
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890
+ORGANIZER;CN="User 01":mailto:user1@example.com
+ATTENDEE;PARTSTAT=ACCEPTED:mailto:user1@example.com
+REQUEST-STATUS:2.0\;Success
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890
+DTSTART:20080601T120000Z
+DTEND:20080601T130000Z
+ATTENDEE;PARTSTAT=ACCEPTED;SCHEDULE-STATUS=2.0:mailto:user1@example.com
+ATTENDEE:mailto:user2@example.com
+ORGANIZER;CN=User 01:mailto:user1@example.com
+END:VEVENT
+END:VCALENDAR
 """,
                 True, "mailto:user1@example.com", (("", True, False),),
             ),
@@ -857,6 +899,501 @@ END:VCALENDAR
                     msg=description
                 )
 
+    def test_sequenceComparison(self):
+        """
+        Test iTIPProcessing.sequenceComparison
+        """
+        
+        data = (
+            (
+                "1.1 Simple Update - SEQUENCE change",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T000000Z
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHOD:REQUEST
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071115T000000Z
+DTSTAMP:20071114T000000Z
+SEQUENCE:1
+END:VEVENT
+END:VCALENDAR
+""",
+                True,
+            ),
+            (
+                "1.2 Simple Update - DTSTAMP change",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T000000Z
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHOD:REQUEST
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T010000Z
+END:VEVENT
+END:VCALENDAR
+""",
+                True,
+            ),
+            (
+                "1.3 Simple Update - no change",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T000000Z
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHOD:REQUEST
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T000000Z
+END:VEVENT
+END:VCALENDAR
+""",
+                True,
+            ),
+            (
+                "2.1 Recurrence add changed SEQUENCE instance",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T000000Z
+RRULE:FREQ=DAILY
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHOD:REQUEST
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071115T000000Z
+DTSTAMP:20071114T000000Z
+RRULE:FREQ=DAILY
+END:VEVENT
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T010000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:1
+END:VEVENT
+END:VCALENDAR
+""",
+                True,
+            ),
+            (
+                "2.2 Recurrence add changed DTSTAMP instance",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T000000Z
+RRULE:FREQ=DAILY
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHOD:REQUEST
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071115T000000Z
+DTSTAMP:20071114T000000Z
+RRULE:FREQ=DAILY
+END:VEVENT
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T010000Z
+END:VEVENT
+END:VCALENDAR
+""",
+                True,
+            ),
+            (
+                "2.3 Recurrence add unchanged instance",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T000000Z
+RRULE:FREQ=DAILY
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHOD:REQUEST
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071115T000000Z
+DTSTAMP:20071114T000000Z
+RRULE:FREQ=DAILY
+END:VEVENT
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T000000Z
+END:VEVENT
+END:VCALENDAR
+""",
+                True,
+            ),
+            (
+                "3.1 Recurrence master/no-master changed SEQUENCE instance",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T000000Z
+RRULE:FREQ=DAILY
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHOD:CANCEL
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:1
+END:VEVENT
+END:VCALENDAR
+""",
+                True,
+            ),
+            (
+                "3.2 Recurrence master/no-master old SEQUENCE instance no prior instance",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T000000Z
+RRULE:FREQ=DAILY
+SEQUENCE:2
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHOD:CANCEL
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:1
+END:VEVENT
+END:VCALENDAR
+""",
+                False,
+            ),
+            (
+                "3.3 Recurrence master/no-master old SEQUENCE instance with prior instance",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T000000Z
+RRULE:FREQ=DAILY
+SEQUENCE:2
+END:VEVENT
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:2
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHOD:CANCEL
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:1
+END:VEVENT
+END:VCALENDAR
+""",
+                False,
+            ),
+            (
+                "4.1 Recurrence no-master/master changed SEQUENCE master",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:0
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHID:REQUEST
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T000000Z
+RRULE:FREQ=DAILY
+SEQUENCE:1
+END:VEVENT
+END:VCALENDAR
+""",
+                True,
+            ),
+            (
+                "4.2 Recurrence no-master/master changed DTSTAMP master",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T000000Z
+SEQUENCE:1
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHID:REQUEST
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T010000Z
+RRULE:FREQ=DAILY
+SEQUENCE:1
+END:VEVENT
+END:VCALENDAR
+""",
+                True,
+            ),
+            (
+                "4.3 Recurrence no-master/master old SEQUENCE instance",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:2
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHID:REQUEST
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T000000Z
+RRULE:FREQ=DAILY
+SEQUENCE:1
+END:VEVENT
+END:VCALENDAR
+""",
+                False,
+            ),
+            (
+                "4.4 Recurrence no-master/master changed SEQUENCE instance",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:0
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHID:REQUEST
+BEGIN:VEVENT
+UID:12345-67890-1
+DTSTART:20071114T000000Z
+DTSTAMP:20071114T000000Z
+RRULE:FREQ=DAILY
+SEQUENCE:1
+END:VEVENT
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:1
+END:VEVENT
+END:VCALENDAR
+""",
+                True,
+            ),
+            (
+                "5.1 Recurrence no-masters changed SEQUENCE same instance",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:0
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHID:REQUEST
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:1
+END:VEVENT
+END:VCALENDAR
+""",
+                True,
+            ),
+            (
+                "5.2 Recurrence no-masters changed DTSTAMP same instance",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:0
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHID:REQUEST
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T020000Z
+SEQUENCE:0
+END:VEVENT
+END:VCALENDAR
+""",
+                True,
+            ),
+            (
+                "5.3 Recurrence no-masters changed SEQUENCE different instances",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071116T000000Z
+DTSTART:20071116T000000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:0
+END:VEVENT
+END:VCALENDAR
+""",
+                """BEGIN:VCALENDAR
+VERSION:2.0
+PRODID:-//CALENDARSERVER.ORG//NONSGML Version 1//EN
+METHID:REQUEST
+BEGIN:VEVENT
+UID:12345-67890-1
+RECURRENCE-ID:20071117T000000Z
+DTSTART:20071117T000000Z
+DTSTAMP:20071114T010000Z
+SEQUENCE:1
+END:VEVENT
+END:VCALENDAR
+""",
+                True,
+            ),
+        )
+        
+        for title, calendar_txt, itip_txt, expected in data:
+            calendar = Component.fromString(calendar_txt)
+            itip = Component.fromString(itip_txt)
+
+            result = iTipProcessing.sequenceComparison(itip, calendar)
+            self.assertEqual(result, expected, msg="Result mismatch: %s" % (title,))
+            
+
 class iTIPGenerator (twistedcaldav.test.util.TestCase):
     """
     iCalendar support tests
@@ -920,6 +1457,7 @@ UID:12345-67890-3
 DTSTART:20071114T000000Z
 ATTENDEE:mailto:user2@example.com
 ORGANIZER:mailto:user1@example.com
+SEQUENCE:1
 END:VEVENT
 END:VCALENDAR
 """,
@@ -932,6 +1470,7 @@ UID:12345-67890-3
 DTSTART:20071114T000000Z
 ATTENDEE:mailto:user2@example.com
 ORGANIZER:mailto:user1@example.com
+SEQUENCE:1
 END:VEVENT
 END:VCALENDAR
 """,
@@ -1187,6 +1726,7 @@ DTSTART:20071114T000000Z
 ATTENDEE:mailto:user2@example.com
 ATTENDEE:mailto:user3@example.com
 ORGANIZER:mailto:user1@example.com
+SEQUENCE:1
 END:VEVENT
 END:VCALENDAR
 """,
@@ -1199,7 +1739,7 @@ UID:12345-67890-1
 DTSTART:20071114T000000Z
 ATTENDEE:mailto:user2@example.com
 ORGANIZER:mailto:user1@example.com
-SEQUENCE:1
+SEQUENCE:2
 END:VEVENT
 END:VCALENDAR
 """,
@@ -1218,7 +1758,7 @@ DTSTART:20071114T000000Z
 ATTENDEE:mailto:user2@example.com
 ATTENDEE:mailto:user3@example.com
 ORGANIZER:mailto:user1@example.com
-SEQUENCE:1
+SEQUENCE:2
 END:VEVENT
 END:VCALENDAR
 """,
@@ -1232,7 +1772,7 @@ DTSTART:20071114T000000Z
 ATTENDEE:mailto:user3@example.com
 ATTENDEE:mailto:user2@example.com
 ORGANIZER:mailto:user1@example.com
-SEQUENCE:2
+SEQUENCE:3
 END:VEVENT
 END:VCALENDAR
 """,
@@ -1251,6 +1791,7 @@ DTSTART:20071114T000000Z
 ATTENDEE:mailto:user2@example.com
 ORGANIZER:mailto:user1@example.com
 RRULE:FREQ=YEARLY
+SEQUENCE:1
 END:VEVENT
 END:VCALENDAR
 """,
@@ -1264,12 +1805,12 @@ RECURRENCE-ID:20081114T000000Z
 DTSTART:20081114T000000Z
 ATTENDEE:mailto:user2@example.com
 ORGANIZER:mailto:user1@example.com
-SEQUENCE:1
+SEQUENCE:2
 END:VEVENT
 END:VCALENDAR
 """,
                 ("mailto:user2@example.com",),
-                (datetime.datetime(2008, 11, 14, 0, 0, tzinfo=tzutc()), ),
+                (PyCalendarDateTime(2008, 11, 14, 0, 0, 0, tzid=PyCalendarTimezone(utc=True)), ),
             ),
 
             # Recurring component with one instance, each with one attendee - cancel instance
@@ -1283,6 +1824,7 @@ DTSTART:20071114T000000Z
 ATTENDEE:mailto:user2@example.com
 ORGANIZER:mailto:user1@example.com
 RRULE:FREQ=YEARLY
+SEQUENCE:1
 END:VEVENT
 BEGIN:VEVENT
 UID:12345-67890-4
@@ -1290,6 +1832,7 @@ RECURRENCE-ID:20081114T000000Z
 DTSTART:20081114T010000Z
 ATTENDEE:mailto:user2@example.com
 ORGANIZER:mailto:user1@example.com
+SEQUENCE:1
 END:VEVENT
 END:VCALENDAR
 """,
@@ -1303,12 +1846,12 @@ RECURRENCE-ID:20081114T000000Z
 DTSTART:20081114T010000Z
 ATTENDEE:mailto:user2@example.com
 ORGANIZER:mailto:user1@example.com
-SEQUENCE:1
+SEQUENCE:2
 END:VEVENT
 END:VCALENDAR
 """,
                 ("mailto:user2@example.com",),
-                (datetime.datetime(2008, 11, 14, 0, 0, tzinfo=tzutc()), ),
+                (PyCalendarDateTime(2008, 11, 14, 0, 0, 0, tzid=PyCalendarTimezone(utc=True)), ),
             ),
 
             # Recurring component with one instance, each with one attendee - cancel master
@@ -1322,6 +1865,7 @@ DTSTART:20071114T000000Z
 ATTENDEE:mailto:user2@example.com
 ORGANIZER:mailto:user1@example.com
 RRULE:FREQ=YEARLY
+SEQUENCE:1
 END:VEVENT
 BEGIN:VEVENT
 UID:12345-67890-5
@@ -1329,6 +1873,7 @@ RECURRENCE-ID:20081114T000000Z
 DTSTART:20071114T010000Z
 ATTENDEE:mailto:user2@example.com
 ORGANIZER:mailto:user1@example.com
+SEQUENCE:1
 END:VEVENT
 END:VCALENDAR
 """,
@@ -1342,7 +1887,7 @@ DTSTART:20071114T000000Z
 ATTENDEE:mailto:user2@example.com
 ORGANIZER:mailto:user1@example.com
 RRULE:FREQ=YEARLY
-SEQUENCE:1
+SEQUENCE:2
 END:VEVENT
 END:VCALENDAR
 """,
