@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2007 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2009 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,7 +31,9 @@ try:
 except ImportError:
     from pysqlite2 import dbapi2 as sqlite
 
-from twistedcaldav.log import Logger
+sqlite.connect(":memory:")
+
+from twext.python.log import Logger
 
 log = Logger()
 
@@ -88,34 +90,26 @@ class AbstractSQLDatabase (object):
                     self._db_connection = sqlite.connect(db_filename, isolation_level=None)
                 else:
                     self._db_connection = sqlite.connect(db_filename)
+
             except DatabaseError:
                 raise DatabaseError("Unable to open database %s" % (self.dbpath,))
 
+            q = self._db_connection.cursor()
+
+            #
+            # Set Journal mode to PERSIST to avoid constant unlink calls
+            #
+            q.execute("PRAGMA journal_mode = PERSIST")
+
             #
             # Set up the schema
             #
-            q = self._db_connection.cursor()
             try:
                 # Create CALDAV table if needed
 
                 if self._test_schema_table(q):
-                    q.execute(
-                        """
-                        select VALUE from CALDAV
-                         where KEY = 'SCHEMA_VERSION'
-                        """)
-                    version = q.fetchone()
-
-                    if version is not None: version = version[0]
-
-                    q.execute(
-                        """
-                        select VALUE from CALDAV
-                         where KEY = 'TYPE'
-                        """)
-                    dbtype = q.fetchone()
-
-                    if dbtype is not None: dbtype = dbtype[0]
+                    
+                    version, dbtype = self._get_schema_version(q)
 
                     if (version != self._db_version()) or (dbtype != self._db_type()):
 
@@ -125,13 +119,6 @@ class AbstractSQLDatabase (object):
                         self._db_connection.close()
                         del(self._db_connection)
 
-                        if version != self._db_version():
-                            log.err("Database %s has different schema (v.%s vs. v.%s)"
-                                    % (db_filename, version, self._db_version()))
-                            
-                            # Upgrade the DB
-                            return self._db_upgrade(version)
-
                         if dbtype != self._db_type():
                             log.err("Database %s has different type (%s vs. %s)"
                                     % (db_filename, dbtype, self._db_type()))
@@ -140,6 +127,13 @@ class AbstractSQLDatabase (object):
                             os.remove(db_filename)
                             return self._db()
 
+                        if version != self._db_version():
+                            log.err("Database %s has different schema (v.%s vs. v.%s)"
+                                    % (db_filename, version, self._db_version()))
+                            
+                            # Upgrade the DB
+                            return self._db_upgrade(version)
+
                 else:
                     self._db_init(db_filename, q)
 
@@ -155,6 +149,27 @@ class AbstractSQLDatabase (object):
         """)
         return q.fetchone()
 
+    def _get_schema_version(self, q):
+        q.execute(
+            """
+            select VALUE from CALDAV
+             where KEY = 'SCHEMA_VERSION'
+            """)
+        version = q.fetchone()
+
+        if version is not None: version = version[0]
+
+        q.execute(
+            """
+            select VALUE from CALDAV
+             where KEY = 'TYPE'
+            """)
+        dbtype = q.fetchone()
+
+        if dbtype is not None: dbtype = dbtype[0]
+
+        return version, dbtype
+
     def _db_init(self, db_filename, q):
         """
         Initialise the underlying database tables.
@@ -177,7 +192,10 @@ class AbstractSQLDatabase (object):
             self._db_init_data_tables(q)
             self._db_recreate(False)
 
-        q.execute("commit")
+        try:
+            q.execute("commit")
+        except DatabaseError:
+            pass
 
         self._db_connection.isolation_level = old_isolation
 
@@ -228,24 +246,38 @@ class AbstractSQLDatabase (object):
         if do_commit:
             self._db_commit()
 
+    def _db_can_upgrade(self, old_version):
+        
+        return self.persistent
+
     def _db_upgrade(self, old_version):
         """
         Upgrade the database tables.
         """
         
-        if self.persistent:
+        if self._db_can_upgrade(old_version):
             self._db_connection = sqlite.connect(self.dbpath, isolation_level=None)
             q = self._db_connection.cursor()
-            self._db_upgrade_data_tables(q, old_version)
-            self._db_upgrade_schema(q)
+            q.execute("begin exclusive transaction")
+
+            # We re-check whether the schema version again AFTER we've got an exclusive
+            # lock as some other server process may have snuck in and already upgraded it
+            # before we got the lock, or whilst we were waiting for it.
+            version, _ignore_dbtype = self._get_schema_version(q)
+
+            if version != self._db_version():
+                self._db_upgrade_data_tables(q, old_version)
+                self._db_upgrade_schema(q)
+
+            q.execute("commit")
             self._db_close()
-            return self._db()
         else:
             # Non-persistent DB's by default can be removed and re-created. However, for simple
             # DB upgrades they SHOULD override this method and handle those for better performance.
             os.remove(self.dbpath)
-            return self._db()
-    
+
+        return self._db()
+
     def _db_upgrade_data_tables(self, q, old_version):
         """
         Upgrade the data from an older version of the DB.
@@ -307,6 +339,7 @@ class AbstractSQLDatabase (object):
         q = self._db().cursor()
         try:
             q.execute(sql, query_params)
+            self.lastrowid = q.lastrowid
             return q.fetchall()
         except DatabaseError:
             log.err("Exception while executing SQL on DB %s: %r %r" % (self, sql, query_params))
