@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2007 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,23 +15,59 @@
 ##
 
 import os
-import shutil
 
 from twisted.trial.unittest import SkipTest
-from twisted.web2 import responsecode
-from twisted.web2.iweb import IResponse
-from twisted.web2.stream import MemoryStream
-from twisted.web2.dav import davxml
-from twisted.web2.dav.fileop import rmdir
-from twisted.web2.dav.util import davXMLFromStream
-from twisted.web2.test.test_server import SimpleRequest
 
-import twistedcaldav.test.util
+from twext.python.filepath import CachingFilePath as FilePath
+
+from twext.web2 import responsecode
+from twext.web2.iweb import IResponse
+from twext.web2.stream import MemoryStream
+from twext.web2.dav import davxml
+from twext.web2.dav.util import davXMLFromStream
+from twext.web2.test.test_server import SimpleRequest
+
 from twistedcaldav import caldavxml
 from twistedcaldav import ical
-from twistedcaldav.index import db_basename
 
-class CalendarQuery (twistedcaldav.test.util.TestCase):
+from twistedcaldav.query import calendarqueryfilter
+from twistedcaldav.config import config
+from twistedcaldav.test.util import HomeTestCase
+from twisted.internet.defer import inlineCallbacks, returnValue
+from txdav.common.datastore.test.util import buildStore, StubNotifierFactory
+
+
+@inlineCallbacks
+def addEventsDir(testCase, eventsDir, uri):
+    """
+    Add events to a L{HomeTestCase} from a directory.
+
+    @param testCase: The test case to add events to.
+    @type testCase: L{HomeTestCase}
+
+    @param eventsDir: A directory full of events.
+    @type eventsDir: L{FilePath}
+
+    @param uri: The URI-path of the calendar to insert events into.
+    @type uri: C{str}
+
+    @return: a L{Deferred} which fires with the number of added calendar object
+        resources.
+    """
+    count = 0
+    for child in eventsDir.children():
+        count += 1
+        if child.basename().split(".")[-1] != "ics":
+            continue
+        request = SimpleRequest(testCase.site, "PUT",
+                                uri + "/" + child.basename())
+        request.stream = MemoryStream(child.getContent())
+        yield testCase.send(request)
+    returnValue(count)
+
+
+
+class CalendarQuery (HomeTestCase):
     """
     calendar-query REPORT
     """
@@ -116,7 +152,7 @@ class CalendarQuery (twistedcaldav.test.util.TestCase):
                             cal = property.calendar()
                             instances = cal.expandTimeRanges(query_timerange.end)
                             vevents = [x for x in cal.subcomponents() if x.name() == "VEVENT"]
-                            if not query_timerange.matchinstance(vevents[0], instances):
+                            if not calendarqueryfilter.TimeRange(query_timerange).matchinstance(vevents[0], instances):
                                 self.fail("REPORT property %r returned calendar %s outside of request time range %r"
                                           % (property, property.calendar, query_timerange))
 
@@ -178,16 +214,65 @@ class CalendarQuery (twistedcaldav.test.util.TestCase):
         All events.
         (CalDAV-access-09, section 7.6.8)
         """
-        uids = [r[0] for r in (os.path.splitext(f) for f in os.listdir(self.holidays_dir)) if r[1] == ".ics"]
+        uids = [r[0] for r in (os.path.splitext(f) for f in
+                os.listdir(self.holidays_dir)) if r[1] == ".ics"]
 
         return self.simple_event_query("/calendar_query_events/", None, uids)
 
-    def simple_event_query(self, cal_uri, event_filter, uids):
-        query = caldavxml.CalendarQuery(
-            davxml.PropertyContainer(
-                davxml.GETETag(),
+    def test_calendar_query_limited_with_data(self):
+        """
+        All events.
+        (CalDAV-access-09, section 7.6.8)
+        """
+        
+        oldValue = config.MaxQueryWithDataResults
+        config.MaxQueryWithDataResults = 1
+        def _restoreValueOK(f):
+            config.MaxQueryWithDataResults = oldValue
+            self.fail("REPORT must fail with 403")
+
+        def _restoreValueError(f):
+            config.MaxQueryWithDataResults = oldValue
+            return None
+
+        uids = [r[0] for r in (os.path.splitext(f) for f in os.listdir(self.holidays_dir)) if r[1] == ".ics"]
+
+        d = self.simple_event_query("/calendar_query_events/", None, uids)
+        d.addCallbacks(_restoreValueOK, _restoreValueError)
+        return d
+
+    def test_calendar_query_limited_without_data(self):
+        """
+        All events.
+        (CalDAV-access-09, section 7.6.8)
+        """
+        
+        oldValue = config.MaxQueryWithDataResults
+        config.MaxQueryWithDataResults = 1
+        def _restoreValueOK(f):
+            config.MaxQueryWithDataResults = oldValue
+            return None
+
+        def _restoreValueError(f):
+            config.MaxQueryWithDataResults = oldValue
+            self.fail("REPORT must not fail with 403")
+
+        uids = [r[0] for r in (os.path.splitext(f) for f in os.listdir(self.holidays_dir)) if r[1] == ".ics"]
+
+        d = self.simple_event_query("/calendar_query_events/", None, uids, withData=False)
+        d.addCallbacks(_restoreValueOK, _restoreValueError)
+        return d
+
+    def simple_event_query(self, cal_uri, event_filter, uids, withData=True):
+        props = (
+            davxml.GETETag(),
+        )
+        if withData:
+            props += (
                 caldavxml.CalendarData(),
-            ),
+            )
+        query = caldavxml.CalendarQuery(
+            davxml.PropertyContainer(*props),
             caldavxml.Filter(
                 caldavxml.ComponentFilter(
                     caldavxml.ComponentFilter(
@@ -238,41 +323,42 @@ class CalendarQuery (twistedcaldav.test.util.TestCase):
 
         return self.calendar_query(cal_uri, query, got_xml)
 
+
+    @inlineCallbacks
     def calendar_query(self, calendar_uri, query, got_xml):
-        calendar_path = os.path.join(self.docroot, calendar_uri[1:])
 
-        if os.path.exists(calendar_path): rmdir(calendar_path)
+        response = yield self.send(SimpleRequest(self.site, "MKCALENDAR", calendar_uri))
+        response = IResponse(response)
 
-        def do_report(response):
-            response = IResponse(response)
+        if response.code != responsecode.CREATED:
+            self.fail("MKCALENDAR failed: %s" % (response.code,))
 
-            if response.code != responsecode.CREATED:
-                self.fail("MKCALENDAR failed: %s" % (response.code,))
+        # Add holiday events to calendar
+        yield addEventsDir(self, FilePath(self.holidays_dir), calendar_uri)
 
-            # Add holiday events to calendar
-            # We're cheating by simply copying the files in
-            for filename in os.listdir(self.holidays_dir):
-                if os.path.splitext(filename)[1] != ".ics": continue
-                path = os.path.join(self.holidays_dir, filename)
-                shutil.copy(path, calendar_path)
+        request = SimpleRequest(self.site, "REPORT", calendar_uri)
+        request.stream = MemoryStream(query.toxml())
+        response = yield self.send(request)
 
-            # Delete the index because we cheated
-            index_path = os.path.join(calendar_path, db_basename)
-            if os.path.isfile(index_path): os.remove(index_path)
+        response = IResponse(response)
 
-            request = SimpleRequest(self.site, "REPORT", calendar_uri)
-            request.stream = MemoryStream(query.toxml())
+        if response.code != responsecode.MULTI_STATUS:
+            self.fail("REPORT failed: %s" % (response.code,))
 
-            def do_test(response):
-                response = IResponse(response)
+        returnValue(
+            (yield davXMLFromStream(response.stream).addCallback(got_xml))
+        )
 
-                if response.code != responsecode.MULTI_STATUS:
-                    self.fail("REPORT failed: %s" % (response.code,))
 
-                return davXMLFromStream(response.stream).addCallback(got_xml)
+class DatabaseQueryTests(CalendarQuery):
 
-            return self.send(request, do_test)
+    @inlineCallbacks
+    def setUp(self):
+        self.calendarStore = yield buildStore(self, StubNotifierFactory())
+        yield super(DatabaseQueryTests, self).setUp()
+
+
+    def createDataStore(self):
+        return self.calendarStore
 
-        request = SimpleRequest(self.site, "MKCALENDAR", calendar_uri)
 
-        return self.send(request, do_report)
