@@ -1,5 +1,4 @@
-##
-# Copyright (c) 2006-2008 Apple Inc. All rights reserved.
+# Copyright (c) 2006-2010 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,23 +14,22 @@
 ##
 
 import os
-import shutil
 
-from twisted.internet.defer import inlineCallbacks
-from twisted.web2 import responsecode
-from twisted.web2.iweb import IResponse
-from twisted.web2.stream import MemoryStream
-from twisted.web2.dav import davxml
-from twisted.web2.dav.fileop import rmdir
-from twisted.web2.dav.util import davXMLFromStream
-from twisted.web2.test.test_server import SimpleRequest
+from twisted.internet.defer import inlineCallbacks, returnValue
+from twext.python.filepath import CachingFilePath as FilePath
+from twext.web2 import responsecode
+from twext.web2.iweb import IResponse
+from twext.web2.stream import MemoryStream
+from twext.web2.dav import davxml
+from twext.web2.dav.util import davXMLFromStream
+from twext.web2.test.test_server import SimpleRequest
 
-import twistedcaldav.test.util
 from twistedcaldav import caldavxml
 from twistedcaldav import ical
-from twistedcaldav.index import db_basename
+from twistedcaldav.test.util import HomeTestCase, todo
+from twistedcaldav.config import config
 
-class CalendarMultiget (twistedcaldav.test.util.TestCase):
+class CalendarMultiget (HomeTestCase):
     """
     calendar-multiget REPORT
     """
@@ -46,7 +44,7 @@ class CalendarMultiget (twistedcaldav.test.util.TestCase):
         okuids = [r[0] for r in (os.path.splitext(f) for f in os.listdir(self.holidays_dir)) if r[1] == ".ics"]
         okuids[:] = okuids[1:10]
 
-        baduids = ["12345@example.com", "67890@example.com"]
+        baduids = ["12345%40example.com", "67890%40example.com"]
 
         return self.simple_event_multiget("/calendar_multiget_events/", okuids, baduids)
 
@@ -57,10 +55,55 @@ class CalendarMultiget (twistedcaldav.test.util.TestCase):
         """
         okuids = [r[0] for r in (os.path.splitext(f) for f in os.listdir(self.holidays_dir)) if r[1] == ".ics"]
 
-        baduids = ["12345@example.com", "67890@example.com"]
+        baduids = ["12345%40example.com", "67890%40example.com"]
 
         return self.simple_event_multiget("/calendar_multiget_events/", okuids, baduids)
 
+    def test_multiget_limited_with_data(self):
+        """
+        All events.
+        (CalDAV-access-09, section 7.6.8)
+        """
+        oldValue = config.MaxMultigetWithDataHrefs
+        config.MaxMultigetWithDataHrefs = 1
+        def _restoreValueOK(f):
+            config.MaxMultigetWithDataHrefs = oldValue
+            self.fail("REPORT must fail with 403")
+
+        def _restoreValueError(f):
+            config.MaxMultigetWithDataHrefs = oldValue
+            return None
+
+        okuids = [r[0] for r in (os.path.splitext(f) for f in os.listdir(self.holidays_dir)) if r[1] == ".ics"]
+
+        baduids = ["12345%40example.com", "67890%40example.com"]
+
+        d = self.simple_event_multiget("/calendar_multiget_events/", okuids, baduids)
+        d.addCallbacks(_restoreValueOK, _restoreValueError)
+        return d
+
+    def test_multiget_limited_no_data(self):
+        """
+        All events.
+        (CalDAV-access-09, section 7.6.8)
+        """
+        oldValue = config.MaxMultigetWithDataHrefs
+        config.MaxMultigetWithDataHrefs = 1
+        def _restoreValueOK(f):
+            config.MaxMultigetWithDataHrefs = oldValue
+            return None
+
+        def _restoreValueError(f):
+            config.MaxMultigetWithDataHrefs = oldValue
+            self.fail("REPORT must not fail with 403")
+
+        okuids = [r[0] for r in (os.path.splitext(f) for f in os.listdir(self.holidays_dir)) if r[1] == ".ics"]
+
+        baduids = ["12345%40example.com", "67890%40example.com"]
+
+        return self.simple_event_multiget("/calendar_multiget_events/", okuids, baduids, withData=False)
+
+    @todo("Remove: Does not work with new store")
     @inlineCallbacks
     def test_multiget_one_broken_event(self):
         """
@@ -78,11 +121,12 @@ BEGIN:VEVENT
 UID:good
 DTSTART;VALUE=DATE:20020101
 DTEND;VALUE=DATE:20020102
+DTSTAMP:20020101T121212Z
 RRULE:FREQ=YEARLY;INTERVAL=1;UNTIL=20031231;BYMONTH=1
 SUMMARY:New Year's Day
 END:VEVENT
 END:VCALENDAR
-""",
+""".replace("\n", "\r\n"),
             "bad":"""BEGIN:VCALENDAR
 CALSCALE:GREGORIAN
 PRODID:-//Apple Computer\, Inc//iCal 2.0//EN
@@ -91,11 +135,12 @@ BEGIN:VEVENT
 UID:bad
 DTSTART;VALUE=DATE:20020214
 DTEND;VALUE=DATE:20020215
+DTSTAMP:20020101T121212Z
 RRULE:FREQ=YEARLY;INTERVAL=1;BYMONTH=2
 SUMMARY:Valentine's Day
 END:VEVENT
 END:VCALENDAR
-"""
+""".replace("\n", "\r\n")
         }
 
         yield self.simple_event_multiget("/calendar_multiget_events/", okuids, baduids, data)
@@ -110,18 +155,26 @@ VERSION:2.0
 BEGIN:VEVENT
 UID:bad
 DTSTART;VALUE=DATE:20020214
-DTEND;VALUE=DATE:20020""")
+DTEND;VALUE=DATE:20020
+DTSTAMP:20020101T121212Z
+END:VCALENDAR
+""".replace("\n", "\r\n"))
         f.close
 
         okuids = ["good", ]
         baduids = ["bad", ]
         yield self.simple_event_multiget("/calendar_multiget_events/", okuids, baduids, data, no_init=True)
 
-    def simple_event_multiget(self, cal_uri, okuids, baduids, data=None, no_init=False):
+    def simple_event_multiget(self, cal_uri, okuids, baduids, data=None, no_init=False, withData=True):
+        props = (
+            davxml.GETETag(),
+        )
+        if withData:
+            props += (
+                caldavxml.CalendarData(),
+            )
         children = []
-        children.append(davxml.PropertyContainer(
-                        davxml.GETETag(),
-                        caldavxml.CalendarData()))
+        children.append(davxml.PropertyContainer(*props))
         
         okhrefs = [cal_uri + x + ".ics" for x in okuids]
         badhrefs = [cal_uri + x + ".ics" for x in baduids]
@@ -164,7 +217,7 @@ DTEND;VALUE=DATE:20020""")
                             self.fail("Got calendar for unexpected UID %r" % (uid,))
 
                         if data:
-                            original_calendar = ical.Component.fromStream(data[uid])
+                            original_calendar = ical.Component.fromString(data[uid])
                         else:
                             original_filename = file(os.path.join(self.holidays_dir, uid + ".ics"))
                             original_calendar = ical.Component.fromStream(original_filename)
@@ -189,57 +242,45 @@ DTEND;VALUE=DATE:20020""")
                         else:
                             self.fail("Got unexpected href %r" % (href,))
         
-            if len(okuids) + len(badhrefs):
+            if withData and (len(okuids) + len(badhrefs)):
                 self.fail("Some components were not returned: %r, %r" % (okuids, badhrefs))
 
         return self.calendar_query(cal_uri, query, got_xml, data, no_init)
 
+    @inlineCallbacks
     def calendar_query(self, calendar_uri, query, got_xml, data, no_init):
-        calendar_path = os.path.join(self.docroot, calendar_uri[1:])
 
-        if not no_init and os.path.exists(calendar_path): rmdir(calendar_path)
+        if not no_init:
+            response = yield self.send(SimpleRequest(self.site, "MKCALENDAR",
+                calendar_uri))
+            response = IResponse(response)
+            if response.code != responsecode.CREATED:
+                self.fail("MKCALENDAR failed: %s" % (response.code,))
 
-        def do_report(response):
-            if not no_init:
-                response = IResponse(response)
-    
-                if response.code != responsecode.CREATED:
-                    self.fail("MKCALENDAR failed: %s" % (response.code,))
-    
-                if data:
-                    for filename, icaldata in data.iteritems():
-                        path = os.path.join(calendar_path, filename + ".ics")
-                        f = open(path, "w")
-                        f.write(icaldata)
-                        f.close()
-                else:
-                    # Add holiday events to calendar
-                    # We're cheating by simply copying the files in
-                    for filename in os.listdir(self.holidays_dir):
-                        if os.path.splitext(filename)[1] != ".ics": continue
-                        path = os.path.join(self.holidays_dir, filename)
-                        shutil.copy(path, calendar_path)
-    
-                # Delete the index because we cheated
-                index_path = os.path.join(calendar_path, db_basename)
-                if os.path.isfile(index_path): os.remove(index_path)
+            if data:
+                for filename, icaldata in data.iteritems():
+                    request = SimpleRequest(self.site, "PUT",
+                        calendar_uri + "/" + filename + ".ics")
+                    request.stream = MemoryStream(icaldata)
+                    yield self.send(request)
+            else:
+                # Add holiday events to calendar
+                for child in FilePath(self.holidays_dir).children():
+                    if os.path.splitext(child.basename())[1] != ".ics": continue
+                    request = SimpleRequest(self.site, "PUT",
+                        calendar_uri + "/" + child.basename())
+                    request.stream = MemoryStream(child.getContent())
+                    yield self.send(request)
 
-            request = SimpleRequest(self.site, "REPORT", calendar_uri)
-            request.stream = MemoryStream(query.toxml())
+        request = SimpleRequest(self.site, "REPORT", calendar_uri)
+        request.stream = MemoryStream(query.toxml())
+        response = yield self.send(request)
 
-            def do_test(response):
-                response = IResponse(response)
+        response = IResponse(response)
 
-                if response.code != responsecode.MULTI_STATUS:
-                    self.fail("REPORT failed: %s" % (response.code,))
+        if response.code != responsecode.MULTI_STATUS:
+            self.fail("REPORT failed: %s" % (response.code,))
 
-                return davXMLFromStream(response.stream).addCallback(got_xml)
-
-            return self.send(request, do_test)
-
-        if no_init:
-            return do_report(None)
-        else:
-            request = SimpleRequest(self.site, "MKCALENDAR", calendar_uri)
-    
-            return self.send(request, do_report)
+        returnValue(
+            (yield davXMLFromStream(response.stream).addCallback(got_xml))
+        )
