@@ -18,36 +18,52 @@ from twisted.internet.task import Clock
 from twisted.words.protocols.jabber.client import IQ
 from twisted.words.protocols.jabber.error import StanzaError
 from twistedcaldav.notify import *
-from twistedcaldav import config as config_mod
 from twistedcaldav.config import Config
+from twistedcaldav.stdconfig import DEFAULT_CONFIG, PListConfigProvider
 from twistedcaldav.test.util import TestCase
 
 
 class StubResource(object):
 
-    def __init__(self, url):
-        self._url = url
+    def __init__(self, id):
+        self._id = id
 
-    def url(self):
-        return self._url
+    def resourceID(self):
+        return self._id
 
 
-class NotificationClientUserTests(TestCase):
 
-    def test_installNoficationClient(self):
-        self.assertEquals(getNotificationClient(), None)
-        self.clock = Clock()
-        installNotificationClient(None, None,
-            klass=StubNotificationClient, reactor=self.clock)
-        notificationClient = getNotificationClient()
-        self.assertNotEquals(notificationClient, None)
+class NotifierTests(TestCase):
+
+    def test_notifier(self):
+        enabledConfig = Config(PListConfigProvider(DEFAULT_CONFIG))
+        enabledConfig.Notifications["Enabled"] = True
+        notifier = Notifier(None, id="test")
+
+        self.assertEquals(notifier._ids, {"default": "test"})
+        clone = notifier.clone(label="alt", id="altID")
+        self.assertEquals("altID", clone.getID(label="alt"))
+        self.assertEquals(clone._ids, {
+            "default" : "test",
+            "alt"     : "altID",
+        })
+        self.assertEquals("test", notifier.getID())
+        self.assertEquals(notifier._ids, {
+            "default" : "test",
+        })
+        self.assertEquals(None, notifier.getID(label="notthere"))
+
+        notifier = Notifier(None, id="urn:uuid:foo")
+        self.assertEquals("foo", notifier.getID())
+
+        notifier.disableNotify()
+        self.assertEquals(notifier._notify, False)
+        notifier.enableNotify(None)
+        self.assertEquals(notifier._notify, True)
+
+        notifier = Notifier(None, id="test", prefix="CalDAV")
+        self.assertEquals("CalDAV|test", notifier.getID())
 
-        enabledConfig = Config(config_mod.defaultConfig)
-        enabledConfig.Notifications['Enabled'] = True
-        clientNotifier = ClientNotifier(StubResource("a"),
-            configOverride=enabledConfig)
-        clientNotifier.notify()
-        self.assertEquals(notificationClient.lines, ["a"])
 
 
 class NotificationClientFactoryTests(TestCase):
@@ -76,8 +92,8 @@ class StubNotificationClient(object):
         self.lines = []
         self.observers = set()
 
-    def send(self, op, uri):
-        self.lines.append(uri)
+    def send(self, op, id):
+        self.lines.append(id)
 
     def addObserver(self, observer):
         self.observers.add(observer)
@@ -88,6 +104,9 @@ class StubNotificationClient(object):
     def connectionMade(self):
         pass
 
+    def clear(self):
+        self.lines = []
+
 class StubNotificationClientProtocol(object):
 
     def __init__(self):
@@ -105,11 +124,11 @@ class StubNotificationClientProtocol(object):
         self.factory.connected = False
 
 
-class NotificationClientTests(TestCase):
+class NotifierFactoryTests(TestCase):
 
     def setUp(self):
         TestCase.setUp(self)
-        self.client = NotificationClient(None, None, reactor=Clock())
+        self.client = NotifierFactory(None, None, reactor=Clock())
         self.client.factory = StubNotificationClientFactory()
 
     def test_sendWhileNotConnected(self):
@@ -172,8 +191,8 @@ class StubNotifier(object):
         self.observers = set()
         self.playbackHistory = []
 
-    def enqueue(self, op, uri):
-        self.notifications.append(uri)
+    def enqueue(self, op, id):
+        self.notifications.append(id)
 
     def playback(self, protocol, old_seq):
         self.playbackHistory.append((protocol, old_seq))
@@ -342,40 +361,41 @@ class StubFailure(object):
 
 class XMPPNotifierTests(TestCase):
 
-    xmppEnabledConfig = Config(config_mod.defaultConfig)
-    xmppEnabledConfig.Notifications['Services']['XMPPNotifier']['Enabled'] = True
+    xmppEnabledConfig = Config(PListConfigProvider(DEFAULT_CONFIG))
+    xmppEnabledConfig.Notifications["Enabled"] = True
+    xmppEnabledConfig.Notifications["Services"]["XMPPNotifier"]["Enabled"] = True
     xmppEnabledConfig.ServerHostName = "server.example.com"
     xmppEnabledConfig.HTTPPort = 80
 
-    xmppDisabledConfig = Config(config_mod.defaultConfig)
-    xmppDisabledConfig.Notifications['Services']['XMPPNotifier']['Enabled'] = False
+    xmppDisabledConfig = Config(PListConfigProvider(DEFAULT_CONFIG))
+    xmppDisabledConfig.Notifications["Services"]["XMPPNotifier"]["Enabled"] = False
 
     def setUp(self):
         TestCase.setUp(self)
         self.xmlStream = StubXmlStream()
-        self.settings = { 'ServiceAddress' : 'pubsub.example.com',
-            'NodeConfiguration' : { 'pubsub#deliver_payloads' : '1' },
-            'HeartbeatMinutes' : 30,
+        self.settings = { "ServiceAddress" : "pubsub.example.com",
+            "NodeConfiguration" : { "pubsub#deliver_payloads" : "1" },
+            "HeartbeatMinutes" : 30,
         }
         self.notifier = XMPPNotifier(self.settings, reactor=Clock(),
             configOverride=self.xmppEnabledConfig, heartbeat=False)
         self.notifier.streamOpened(self.xmlStream)
 
     def test_sendWhileConnected(self):
-        self.notifier.enqueue("update", "/principals/__uids__/test")
+        self.notifier.enqueue("update", "test")
 
         iq = self.xmlStream.elements[1]
         self.assertEquals(iq.name, "iq")
 
         pubsubElement = list(iq.elements())[0]
         self.assertEquals(pubsubElement.name, "pubsub")
-        self.assertEquals(pubsubElement.uri, 'http://jabber.org/protocol/pubsub')
+        self.assertEquals(pubsubElement.uri, "http://jabber.org/protocol/pubsub")
 
         publishElement = list(pubsubElement.elements())[0]
         self.assertEquals(publishElement.name, "publish")
-        self.assertEquals(publishElement.uri, 'http://jabber.org/protocol/pubsub')
-        self.assertEquals(publishElement['node'],
-            "/Public/CalDAV/server.example.com/80/principals/__uids__/test/")
+        self.assertEquals(publishElement.uri, "http://jabber.org/protocol/pubsub")
+        self.assertEquals(publishElement["node"],
+            "/server.example.com/test/")
 
     def test_sendWhileNotConnected(self):
         notifier = XMPPNotifier(self.settings, reactor=Clock(),
@@ -389,39 +409,39 @@ class XMPPNotifierTests(TestCase):
         self.assertEquals(iq.name, "iq")
 
     def test_publishReponse400(self):
-        failure = StubFailure(StanzaError('bad-request'))
+        failure = StubFailure(StanzaError("bad-request"))
         self.assertEquals(len(self.xmlStream.elements), 1)
         self.notifier.publishNodeFailure(failure, "testNodeName")
         self.assertEquals(len(self.xmlStream.elements), 2)
         iq = self.xmlStream.elements[1]
         self.assertEquals(iq.name, "iq")
-        self.assertEquals(iq['type'], "get")
+        self.assertEquals(iq["type"], "get")
 
         pubsubElement = list(iq.elements())[0]
         self.assertEquals(pubsubElement.name, "pubsub")
         self.assertEquals(pubsubElement.uri,
-            'http://jabber.org/protocol/pubsub#owner')
+            "http://jabber.org/protocol/pubsub#owner")
         configElement = list(pubsubElement.elements())[0]
         self.assertEquals(configElement.name, "configure")
-        self.assertEquals(configElement['node'], "testNodeName")
+        self.assertEquals(configElement["node"], "testNodeName")
 
 
     def test_publishReponse404(self):
         self.assertEquals(len(self.xmlStream.elements), 1)
-        failure = StubFailure(StanzaError('item-not-found'))
+        failure = StubFailure(StanzaError("item-not-found"))
         self.notifier.publishNodeFailure(failure, "testNodeName")
         self.assertEquals(len(self.xmlStream.elements), 2)
         iq = self.xmlStream.elements[1]
         self.assertEquals(iq.name, "iq")
-        self.assertEquals(iq['type'], "set")
+        self.assertEquals(iq["type"], "set")
 
         pubsubElement = list(iq.elements())[0]
         self.assertEquals(pubsubElement.name, "pubsub")
         self.assertEquals(pubsubElement.uri,
-            'http://jabber.org/protocol/pubsub')
+            "http://jabber.org/protocol/pubsub")
         createElement = list(pubsubElement.elements())[0]
         self.assertEquals(createElement.name, "create")
-        self.assertEquals(createElement['node'], "testNodeName")
+        self.assertEquals(createElement["node"], "testNodeName")
 
 
     def test_configureResponse(self):
@@ -432,11 +452,11 @@ class XMPPNotifierTests(TestCase):
                     return child
             return None
 
-        response = IQ(self.xmlStream, type='result')
-        pubsubElement = response.addElement('pubsub')
-        configElement = pubsubElement.addElement('configure')
-        formElement = configElement.addElement('x')
-        formElement['type'] = 'form'
+        response = IQ(self.xmlStream, type="result")
+        pubsubElement = response.addElement("pubsub")
+        configElement = pubsubElement.addElement("configure")
+        formElement = configElement.addElement("x")
+        formElement["type"] = "form"
         fields = [
             ( "unknown", "don't edit me", "text-single" ),
             ( "pubsub#deliver_payloads", "0", "boolean" ),
@@ -449,9 +469,9 @@ class XMPPNotifierTests(TestCase):
         }
         for field in fields:
             fieldElement = formElement.addElement("field")
-            fieldElement['var'] = field[0]
-            fieldElement['type'] = field[2]
-            fieldElement.addElement('value', content=field[1])
+            fieldElement["var"] = field[0]
+            fieldElement["type"] = field[2]
+            fieldElement.addElement("value", content=field[1])
 
         self.assertEquals(len(self.xmlStream.elements), 1)
         self.notifier.requestConfigurationFormSuccess(response, "testNodeName",
@@ -460,35 +480,36 @@ class XMPPNotifierTests(TestCase):
 
         iq = self.xmlStream.elements[1]
         self.assertEquals(iq.name, "iq")
-        self.assertEquals(iq['type'], "set")
+        self.assertEquals(iq["type"], "set")
 
         pubsubElement = list(iq.elements())[0]
         self.assertEquals(pubsubElement.name, "pubsub")
         configElement = list(pubsubElement.elements())[0]
         self.assertEquals(configElement.name, "configure")
-        self.assertEquals(configElement['node'], "testNodeName")
+        self.assertEquals(configElement["node"], "testNodeName")
         formElement = list(configElement.elements())[0]
-        self.assertEquals(formElement['type'], "submit")
+        self.assertEquals(formElement["type"], "submit")
         for field in formElement.elements():
             valueElement = _getChild(field, "value")
             if valueElement is not None:
-                self.assertEquals(expectedFields[field['var']],
+                self.assertEquals(expectedFields[field["var"]],
                     str(valueElement))
 
 
     def test_sendHeartbeat(self):
 
-        xmppConfig = Config(config_mod.defaultConfig)
-        xmppConfig.Notifications['Services']['XMPPNotifier']['Enabled'] = True
+        xmppConfig = Config(PListConfigProvider(DEFAULT_CONFIG))
+        xmppConfig.Notifications["Enabled"] = True
+        xmppConfig.Notifications["Services"]["XMPPNotifier"]["Enabled"] = True
         xmppConfig.ServerHostName = "server.example.com"
         xmppConfig.HTTPPort = 80
 
         clock = Clock()
         xmlStream = StubXmlStream()
-        settings = { 'ServiceAddress' : 'pubsub.example.com', 'JID' : 'jid',
-            'Password' : 'password', 'KeepAliveSeconds' : 5,
-            'NodeConfiguration' : { 'pubsub#deliver_payloads' : "1" },
-            'HeartbeatMinutes' : 30 }
+        settings = { "ServiceAddress" : "pubsub.example.com", "JID" : "jid",
+            "Password" : "password", "KeepAliveSeconds" : 5,
+            "NodeConfiguration" : { "pubsub#deliver_payloads" : "1" },
+            "HeartbeatMinutes" : 30 }
         notifier = XMPPNotifier(settings, reactor=clock, heartbeat=True,
             roster=False, configOverride=xmppConfig)
         factory = XMPPNotificationFactory(notifier, settings, reactor=clock,
@@ -498,13 +519,13 @@ class XMPPNotifierTests(TestCase):
 
         self.assertEquals(len(xmlStream.elements), 1)
         heartbeat = xmlStream.elements[0]
-        self.assertEquals(heartbeat.name, 'iq')
+        self.assertEquals(heartbeat.name, "iq")
 
         clock.advance(1800)
 
         self.assertEquals(len(xmlStream.elements), 2)
         heartbeat = xmlStream.elements[1]
-        self.assertEquals(heartbeat.name, 'iq')
+        self.assertEquals(heartbeat.name, "iq")
 
         factory.disconnected(xmlStream)
         clock.advance(1800)
@@ -512,14 +533,15 @@ class XMPPNotifierTests(TestCase):
 
 
 
+
 class XMPPNotificationFactoryTests(TestCase):
 
     def test_sendPresence(self):
         clock = Clock()
         xmlStream = StubXmlStream()
-        settings = { 'ServiceAddress' : 'pubsub.example.com', 'JID' : 'jid',
-            'NodeConfiguration' : { 'pubsub#deliver_payloads' : "1" },
-            'Password' : 'password', 'KeepAliveSeconds' : 5 }
+        settings = { "ServiceAddress" : "pubsub.example.com", "JID" : "jid",
+            "NodeConfiguration" : { "pubsub#deliver_payloads" : "1" },
+            "Password" : "password", "KeepAliveSeconds" : 5 }
         notifier = XMPPNotifier(settings, reactor=clock, heartbeat=False)
         factory = XMPPNotificationFactory(notifier, settings, reactor=clock)
         factory.connected(xmlStream)
@@ -527,16 +549,73 @@ class XMPPNotificationFactoryTests(TestCase):
 
         self.assertEquals(len(xmlStream.elements), 2)
         presence = xmlStream.elements[0]
-        self.assertEquals(presence.name, 'presence')
+        self.assertEquals(presence.name, "presence")
         iq = xmlStream.elements[1]
-        self.assertEquals(iq.name, 'iq')
+        self.assertEquals(iq.name, "iq")
 
         clock.advance(5)
 
         self.assertEquals(len(xmlStream.elements), 3)
         presence = xmlStream.elements[2]
-        self.assertEquals(presence.name, 'presence')
+        self.assertEquals(presence.name, "presence")
 
         factory.disconnected(xmlStream)
         clock.advance(5)
         self.assertEquals(len(xmlStream.elements), 3)
+
+
+
+class ConfigurationTests(TestCase):
+
+    def test_disabled(self):
+        disabledConfig = Config(PListConfigProvider(DEFAULT_CONFIG))
+
+        # Overall notifications are disabled
+        disabledConfig.Notifications["Enabled"] = False
+        conf = getPubSubConfiguration(disabledConfig)
+        self.assertEquals(conf, { "enabled" : False, "host" : "" })
+        conf = getXMPPSettings(disabledConfig)
+        self.assertEquals(conf, None)
+
+        # Overall notifications are enabled, but XMPP disabled
+        disabledConfig.Notifications["Enabled"] = True
+        settings = getXMPPSettings(disabledConfig)
+        self.assertEquals(settings, None)
+
+        # Overall notifications are enabled, XMPP enabled, but no APS
+        service = disabledConfig.Notifications["Services"]["XMPPNotifier"]
+        service.Enabled = True
+        conf = getPubSubAPSConfiguration("CalDAV|foo", disabledConfig)
+        self.assertEquals(conf, None)
+
+    def test_enabled(self):
+        enabledConfig = Config(PListConfigProvider(DEFAULT_CONFIG))
+        enabledConfig.Notifications["Enabled"] = True
+        service = enabledConfig.Notifications["Services"]["XMPPNotifier"]
+        service.Enabled = True
+        service.Host = "example.com"
+        service.Port = 5222
+        service.ServiceAddress = "pubsub.example.com"
+        service.CalDAV.APSBundleID = "CalDAVAPSBundleID"
+        service.CalDAV.SubscriptionURL = "CalDAVSubscriptionURL"
+        conf = getPubSubConfiguration(enabledConfig)
+        self.assertEquals(conf, {'heartrate': 30, 'service': 'pubsub.example.com', 'xmpp-server': 'example.com', 'enabled': True, 'host': '', 'port': 0} )
+        conf = getPubSubAPSConfiguration("CalDAV|foo", enabledConfig)
+        self.assertEquals(conf, {'SubscriptionURL': 'CalDAVSubscriptionURL', 'APSBundleID': 'CalDAVAPSBundleID', 'APSEnvironment' : 'PRODUCTION'} )
+        conf = getPubSubAPSConfiguration("noprefix", enabledConfig)
+        self.assertEquals(conf, None)
+        conf = getPubSubAPSConfiguration("UnknownPrefix|foo", enabledConfig)
+        self.assertEquals(conf, None)
+
+    def test_allowedInRoster(self):
+        """
+        Our own JID is implicitly included in AllowedJIDs
+        """
+        settings = {
+            "JID" : "test1@example.com",
+            "AllowedJIDs" : ["test2@example.com"]
+        }
+        notifier = XMPPNotifier(settings, heartbeat=False)
+        self.assertTrue(notifier.allowedInRoster("test1@example.com"))
+        self.assertTrue(notifier.allowedInRoster("test2@example.com"))
+        self.assertFalse(notifier.allowedInRoster("test3@example.com"))
