@@ -14,20 +14,20 @@
 # limitations under the License.
 ##
 
-from twisted.web2.stream import MemoryStream
+from twext.web2.stream import MemoryStream
 
 import os
 
-from twisted.web2 import responsecode
-from twisted.web2.iweb import IResponse
-from twisted.web2.dav import davxml
-from twisted.web2.dav.util import davXMLFromStream
-from twisted.web2.test.test_server import SimpleRequest
+from twext.web2 import responsecode
+from twext.web2.iweb import IResponse
+from twext.web2.dav import davxml
+from twext.web2.dav.util import davXMLFromStream
+from twext.web2.test.test_server import SimpleRequest
 from twistedcaldav import caldavxml
 
-import twistedcaldav.test.util
+from twistedcaldav.test.util import HomeTestCase
 
-class Properties (twistedcaldav.test.util.TestCase):
+class Properties(HomeTestCase):
     """
     CalDAV properties
     """
@@ -128,7 +128,6 @@ class Properties (twistedcaldav.test.util.TestCase):
 
             query = davxml.PropertyFind(
                         davxml.PropertyContainer(
-                            davxml.GETETag(),
                             caldavxml.SupportedCalendarData(),
                             caldavxml.SupportedCalendarComponentSet(),
                             davxml.SupportedReportSet(),
