@@ -1,5 +1,5 @@
 ##
-# Copyright (c) 2005-2007 Apple Inc. All rights reserved.
+# Copyright (c) 2005-2010 Apple Inc. All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,8 +18,10 @@ import os
 
 from twisted.trial.unittest import SkipTest
 from twistedcaldav.ical import Component
-from twistedcaldav.caldavxml import *
+from twistedcaldav.query import calendarqueryfilter
 import twistedcaldav.test.util
+from twistedcaldav.caldavxml import ComponentFilter, PropertyFilter, TextMatch,\
+    Filter, TimeRange
 
 class XML (twistedcaldav.test.util.TestCase):
     """
@@ -28,7 +30,8 @@ class XML (twistedcaldav.test.util.TestCase):
     calendar_file = os.path.join(os.path.dirname(__file__), "data", "Holidays",
                                  "C3184A66-1ED0-11D9-A5E0-000A958A3252.ics")
     calendar = Component.fromStream(file(calendar_file))
-    calendar.validateForCalDAV()
+    calendar.validCalendarData()
+    calendar.validCalendarForCalDAV(methodAllowed=False)
 
     def test_ComponentFilter(self):
         """
@@ -41,11 +44,13 @@ class XML (twistedcaldav.test.util.TestCase):
             if has: no = "no "
             else:   no = ""
 
-            if has != ComponentFilter(
+            if has != calendarqueryfilter.ComponentFilter(
                 ComponentFilter(
-                    name=component_name
-                ),
-                name="VCALENDAR"
+                    ComponentFilter(
+                        name=component_name
+                    ),
+                    name="VCALENDAR"
+                )
             ).match(self.calendar, None):
                 self.fail("Calendar has %s%s?" % (no, component_name))
 
@@ -60,14 +65,16 @@ class XML (twistedcaldav.test.util.TestCase):
             if has: no = "no "
             else:   no = ""
 
-            if has != ComponentFilter(
+            if has != calendarqueryfilter.ComponentFilter(
                 ComponentFilter(
-                    PropertyFilter(
-                        name=property_name
+                    ComponentFilter(
+                        PropertyFilter(
+                            name=property_name
+                        ),
+                        name="VEVENT"
                     ),
-                    name="VEVENT"
-                ),
-                name="VCALENDAR"
+                    name="VCALENDAR"
+                )
             ).match(self.calendar, None):
                 self.fail("Calendar has %sVEVENT with %s?" % (no, property_name))
 
@@ -90,15 +97,17 @@ class XML (twistedcaldav.test.util.TestCase):
             if has: no = "no "
             else:   no = ""
 
-            if has != ComponentFilter(
+            if has != calendarqueryfilter.ComponentFilter(
                 ComponentFilter(
-                    PropertyFilter(
-                        TextMatch.fromString(uid, caseless=caseless),
-                        name="UID"
+                    ComponentFilter(
+                        PropertyFilter(
+                            TextMatch.fromString(uid, caseless=caseless),
+                            name="UID"
+                        ),
+                        name="VEVENT"
                     ),
-                    name="VEVENT"
-                ),
-                name="VCALENDAR"
+                    name="VCALENDAR"
+                )
             ).match(self.calendar, None):
                 self.fail("Calendar has %sVEVENT with UID %s? (caseless=%s)" % (no, uid, caseless))
 
@@ -127,13 +136,17 @@ class XML (twistedcaldav.test.util.TestCase):
             if has: no = "no "
             else:   no = ""
 
-            if has != Filter(ComponentFilter(
-                ComponentFilter(
-                    TimeRange(start=start, end=end),
-                    name="VEVENT"
-                ),
-                name="VCALENDAR"
-            )).match(self.calendar):
+            if has != calendarqueryfilter.Filter(
+                Filter(
+                    ComponentFilter(
+                        ComponentFilter(
+                            TimeRange(start=start, end=end),
+                            name="VEVENT"
+                        ),
+                        name="VCALENDAR"
+                    )
+                )
+            ).match(self.calendar):
                 self.fail("Calendar has %sVEVENT with timerange %s?" % (no, (start, end)))
 
     test_TimeRange.todo = "recurrence expansion"
