@@ -17,17 +17,18 @@
 import os
 import re
 import sys
+import base64
+
 from subprocess import Popen, PIPE, STDOUT
+from hashlib import md5, sha1
 
 from twisted.internet import ssl, reactor
 from twisted.web import client
-from twistedcaldav.log import LoggingMixIn
 from twisted.python import failure
-from hashlib import md5, sha1
-import base64
+from twext.python.log import LoggingMixIn
 
 ##
-# getNCPU
+# System Resources (Memory size and processor count)
 ##
 
 try:
@@ -41,6 +42,9 @@ if sys.platform == "darwin" and hasCtypes:
     libc = cdll.LoadLibrary(ctypes.util.find_library("libc"))
 
     def getNCPU():
+        """
+        Returns the number of processors detected
+        """
         ncpu = c_int(0)
         size = c_size_t(sizeof(ncpu))
 
@@ -57,12 +61,36 @@ if sys.platform == "darwin" and hasCtypes:
 
         return int(ncpu.value)
 
+    def getMemorySize():
+        """
+        Returns the physical amount of RAM installed, in bytes
+        """
+        memsize = c_uint64(0)
+        size = c_size_t(sizeof(memsize))
+
+        libc.sysctlbyname.argtypes = [
+            c_char_p, c_void_p, c_void_p, c_void_p, c_ulong
+        ]
+        libc.sysctlbyname(
+            "hw.memsize",
+            c_voidp(addressof(memsize)),
+            c_voidp(addressof(size)),
+            None,
+            0
+        )
+
+        return int(memsize.value)
+
+
 elif sys.platform == "linux2" and hasCtypes:
     libc = cdll.LoadLibrary(ctypes.util.find_library("libc"))
 
     def getNCPU():
         return libc.get_nprocs()
 
+    def getMemorySize():
+        return libc.getpagesize() * libc.get_phys_pages()
+
 else:
     def getNCPU():
         if not hasCtypes:
@@ -72,6 +100,9 @@ else:
 
         raise NotImplementedError("getNCPU not supported on %s%s" % (sys.platform, msg))
 
+    def getMemorySize():
+        raise NotImplementedError("getMemorySize not yet supported on %s" % (sys.platform))
+
 ##
 # Module management
 ##
@@ -283,11 +314,11 @@ class AuthorizedHTTPGetter(client.HTTPPageGetter, LoggingMixIn):
         self.transport.loseConnection()
 
         if not hasattr(self.factory, "username"):
-            self.factory.deferred.errback(failure.Failure(Unauthorized("Mail gateway not able to process reply; authentication required for calendar server")))
+            self.factory.deferred.errback(failure.Failure(Unauthorized("Authentication required")))
             return self.factory.deferred
 
         if hasattr(self.factory, "retried"):
-            self.factory.deferred.errback(failure.Failure(Unauthorized("Mail gateway not able to process reply; could not authenticate user %s with calendar server" % (self.factory.username,))))
+            self.factory.deferred.errback(failure.Failure(Unauthorized("Could not authenticate user %s with calendar server" % (self.factory.username,))))
             return self.factory.deferred
 
         self.factory.retried = True
